/*
 * AbstractPacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.List;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Basic abstract implementation for the <code>Packet</code> interface.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public abstract class AbstractPacket extends ElementNode implements Packet {
    //"Constants"
    public static final NSI ATTRIBUTE_TYPE = new NSI("type", "");
    public static final NSI ATTRIBUTE_ID = new NSI("id", "");
    public static final NSI ATTRIBUTE_FROM = new NSI("from", "");
    public static final NSI ATTRIBUTE_TO = new NSI("to", "");

    //Constructors
    public AbstractPacket(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    protected AbstractPacket(StreamElement parent, AbstractPacket base) {
        super(parent, base);
    }
    
    //Methods
    public StreamBuilder createBuilder() {
        throw new IllegalStateException("Cannot create StreamBuilder");
    }
    
    /** <p>
     * Retrieves the type for this <code>Packet</code>.  Each
     * <code>Packet</code> has its own set of allowed types, defined as
     * subclasses to <code>Type</code>.  Depending on the type, different
     * data may be present, or a different interpretation of the data is
     * necessary.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if it has
     * not been set, is not required, or is the default for the specific
     * instance of <code>Packet</code> or its derivatives.</p>
     *
     * @return The type for this <code>Packet</code>.
     */
    public Type getType() {
        Object  typeObj = getAttributeObject(ATTRIBUTE_TYPE);
        Type    typeVal = null;
        
        if          (typeObj instanceof Type) {
            typeVal = (Type)typeObj;
        } else if   (typeObj != null) {
            typeVal = determineType(typeObj.toString());
        }
        
        return typeVal;
    }
    /** <p>
     * Sets the type for this <code>Packet</code>.Each <code>Packet</code> has
     * its own set of allowed types, defined as subclasses to
     * <code>PacketType</code>.</p>
     *
     * <p>
     * This method cannot be used directly to set the type to <code>{@link
     * #ERROR}</code>, and an <code>IllegalArgumentException</code>
     * is thrown if this is the value of <code>type</code>.  Instead, use the
     * method <code>{@link #setError}</code>.</p>
     *
     * <p>
     * The value of <code>type</code> must be one of the allowed types for
     * the specific implementation of <code>Packet</code> in use, or an
     * <code>IllegalArgumentException</code> is thrown.  The value of
     * <code>type</code> cannot be <code>null</code> unless allowed by the
     * specific implementation.</p>
     *
     * @param type The new type for this <code>Packet</code>.
     * @throws IllegalArgumentException If the value of <code>type</code> is
     * not acceptable.
     */
    public void setType(Type type) throws IllegalArgumentException {
        setAttributeObject(ATTRIBUTE_TYPE, type);
    }
    protected Type determineType(String name) {
        Type    type = null;
        
        if (Utilities.equateStrings(ERROR.toString(), name)) {
            type = ERROR;
        } else {
            setError(null);
        }
        
        return type;
    }

    /** <p>
     * Retrieves the "from" <code>JID</code> for this <code>Packet</code>.
     * The definition of "from" is dependent upon which stream direction this
     * <code>Packet</code> is for.  If this is from the input
     * stream, "from" is the <code>JID</code> for the remote side.
     * Otherwise, the <code>JID</code> is for the local side.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if
     * the "from" <code>JID</code> is not defined.  This will not be
     * the case for <code>Packet</code> objects generated from the
     * input stream.</p>
     *
     * @return The "from" <code>JID</code>.
     */
    public JID getFrom() {
        Object  fromObj = getAttributeObject(ATTRIBUTE_FROM);
        JID     fromVal = null;
        
        if          (fromObj instanceof JID) {
            fromVal = (JID)fromObj;
        } else if   (fromObj != null) {
            try {
                fromVal = JID.valueOf(fromObj);
                setAttributeObject(ATTRIBUTE_FROM, fromVal);
            } catch (IllegalArgumentException iae) {}
        }
        
        return fromVal;
    }
    /** <p>
     * Sets the "from" <code>JID</code> for this <code>Packet</code>.
     * If the value of <code>from</code> is <code>null</code>, the current
     * <code>JID</code> is removed.</p>
     *
     * @param from The new "from" <code>JID</code>.
     */
    public void setFrom(JID from) {
        setAttributeObject(ATTRIBUTE_FROM, from);
    }

    /** <p>
     * Retrieves the "to" <code>JID</code> for this <code>Packet</code>.
     * The definition for "to" is dependent upon which stream direction this
     * <code>Packet</code> is for.  If this is from the input
     * stream, "to" is the <code>JID</code> for the remote side.
     * Otherwise, the <code>JID</code> is for the local side.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if
     * the "to" <code>JID</code> is not defined.  This will not be
     * the case for <code>Packet</code> objects generated from the
     * input stream.</p>
     *
     * @return The "to" <code>JID</code>.
     */
    public JID getTo() {
        Object  toObj = getAttributeObject(ATTRIBUTE_TO);
        JID     toVal = null;
        
        if          (toObj instanceof JID) {
            toVal = (JID)toObj;
        } else if   (toObj != null) {
            try {
                toVal = JID.valueOf(toObj);
                setAttributeObject(ATTRIBUTE_TO, toVal);
            } catch (IllegalArgumentException iae) {}
        }
        
        return toVal;
    }
    public void setTo(JID to) {
        setAttributeObject(ATTRIBUTE_TO, to);
    }
    
    public List listExtensions() {
        return listExtensions(null, null);
    }
    public List listExtensions(Class iface) {
        return listExtensions(null, iface);
    }
    public List listExtensions(String ns) {
        return listElements(new NSI(null, ns), Extension.class);
    }
    public List listExtensions(String ns, Class iface) {
        if (iface == null)
            iface = Extension.class;
        
        if (!Extension.class.isAssignableFrom(iface))
            throw new IllegalArgumentException(iface + " does not implement Extension");
        
        return listElements(new NSI(null, ns), iface);
    }
    
    public Extension getExtension() {
        return getExtension(null, null);
    }
    public Extension getExtension(Class iface) {
        return getExtension(null, iface);
    }
    public Extension getExtension(String ns) {
        return (Extension)getFirstElement(new NSI(null, ns), Extension.class);
    }
    public Extension getExtension(String ns, Class iface) {
        if ((iface != null) && !Extension.class.isAssignableFrom(iface))
            throw new IllegalArgumentException(iface + " does not implement Extension");
        return (Extension)getFirstElement(new NSI(null, ns), iface);
    }

    public void addExtension(Extension x) {
        add(x);
    }
    public void removeExtension(Extension x) {
        remove(x);
    }
    
    public PacketError setError(int code) {
        return setError(code, null);
    }
    public PacketError setError(int code, String msg) {
        PacketError     err = getDataFactory().createError(code, msg);
        
        setError(err);
        
        return err;
    }
    public PacketError setError(PacketError.Type type, String cond) throws IllegalArgumentException {
        PacketError     err = getDataFactory().createPacketError(type, cond);
        
        setError(err);
        
        return err;
    }
}