/*
 * ElementBuilder.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class ElementBuilder extends AbstractBuilder {
    //Fields
    private StreamElement   _Base;

    private List            _Elems = new java.util.LinkedList();

    //Constructors
    /** Creates a new instance of UknownNodeBuilder */
    public ElementBuilder(StreamElement base) {
        setBaseElement(base);
    }

    //Methods
    public StreamElement getBaseElement() {
        return _Base;
    }
    protected void setBaseElement(StreamElement base) throws IllegalArgumentException {
        if (base == null)
            throw new IllegalArgumentException("StreamElement cannot be null");
        _Base = base;
    }

    protected int getDepth() {
        return _Elems.size();
    }
    protected StreamElement getTop() {
        return (StreamElement)(!_Elems.isEmpty() ? _Elems.get(0) : null);
    }
    protected StreamElement popElement() {
        StreamElement   e;
        
        if ((e = getTop()) != null)
            _Elems.remove(0);
        return e;
    }
    protected void pushElement(StreamElement e) {
        _Elems.add(0, e);
    }

    public void addExtendedData(StreamNode node) throws IllegalStateException {
        getBaseElement().add(node);
    }
    /** <p>
     * Retrieves the <tt>StreamBuilder</tt> from which to create extended
     * data.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The <tt>StreamBuilder</tt> for the extended data.
     * @throws IllegalStateException If the current status is not <tt>{@link
     * #EXTENDED_DATA}</tt>.
     */
    public StreamBuilder buildExtendedData() throws IllegalStateException {
        return null;
    }
    protected void setExtendedData(StreamBuilder b) {
    }
    
    /** <p>
     * Retrieves the <tt>StreamNode</tt> this <tt>StreamBuilder</tt> has
     * been constructing.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The new <tt>StreamNode</tt> that was built.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #COMPLETE}</tt>.
     */
    public StreamNode getNode() throws IllegalStateException {
        return getBaseElement();
    }
    
    /** <p>
     * Resets this <tt>StreamBuilder</tt>.  By resetting, the status is
     * set to <tt>{@link #READY}</tt>, and all internal data for building
     * an object is either deleted or set to default values.</p>
     */
    public void reset() {
        getBaseElement().reset();
        setStatus(StreamBuilder.READY);
    }

    public Status processElementStart(StreamParseState s) {
        StreamElement   ctx = s.getContext();
        NSI             name = s.getName();
        Set             nss = s.getNamespacePrefixes(s.getDepth());
        Set             attrs = s.getAttributeNames();
        StreamElement   elem;
        Iterator        itr;
        
        //Setup "depth" and StreamElement to process
        if (getDepth() == 0) {
            pushElement(elem = getBaseElement());
        } else {
            pushElement(elem = createElementNode(s));
        }
        
        //Process namespaces
        itr = nss.iterator();
        while (itr.hasNext()) {
            String      prefix = (String)itr.next();
            String      uri = s.getNamespaceURI(prefix);
            
            elem.addNamespace(prefix, uri);
        }
        
        //Process attributes
        itr = attrs.iterator();
        while (itr.hasNext()) {
            NSI         attrName = (NSI)itr.next();
            String      attrVal = s.getAttributeValue(attrName);
            
            elem.setAttributeValue(attrName, attrVal);
        }
        
        //Ensure we are processing
        setStatus(PROCESSING);
        
        return getStatus();
    }
    public Status processElementEnd(StreamParseState s) {
        StreamElement   elem = popElement();
        
        if (getDepth() > 0) {
            StreamElement   parent = getTop();
            
            parent.add(elem);
            setStatus(PROCESSING);
        } else {
            setStatus(COMPLETE);
        }

        return getStatus();
    }
    public Status processText(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        String              txt = s.getText();
        StreamElement       elem = getTop();

        elem.add(ctx.getDataFactory().createText(txt));
        setStatus(PROCESSING);

        return getStatus();
    }
    
    protected StreamElement createElementNode(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        NSI                 name = s.getName();
        
        return ctx.getDataFactory().createElementNode(name, null);
    }
}