/*
 * JSOImplementation.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.io.DOMExporter;
import org.jabberstudio.jso.io.DOMImporter;
import org.jabberstudio.jso.io.Exporter;
import org.jabberstudio.jso.io.Importer;
import org.jabberstudio.jso.io.XMLExporter;
import org.jabberstudio.jso.io.XMLImporter;
import org.jabberstudio.jso.xpath.XPathSupport;

import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Starting point for the JSO library. An instance of this class is used to
 * instantiate implementations for the various other package interfaces and
 * objects. It also acts as the primary "registry" for <tt>StreamElement</tt>
 * objects.</p>
 *
 * <p>
 * Instances of this class are obtained via {@link #getInstance} or
 * {@link #createInstance}. The former provides a "Singleton" instance, while
 * the latter allows for the creation of a specific implementation of
 * this class.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public abstract class JSOImplementation implements StreamElementRegistry, XPathSupport {
    //"Constants"
    /** System property name containing the fully-qualified
     * <tt>JSOImplementation</tt> class name.
     * @since   JSO-0.2
     */
    public static final String CLASS_PROPERTY = "org.jabberstudio.jso.JSOImplementation.class";

    //Fields
    private static JSOImplementation    _Instance;
    
    //Methods
    /**
     * <p>
     * Retrieves the "singleton" instance of <tt>JSOImplementation</tt>,
     * creating it if necessary.  When creating the "singleton", this method
     * calls {@link #createInstance}.</p>
     *
     * @return The <tt>JSOImplementation</tt> "singleton".
     * @throws RuntimeException If the an instance cannot be created.
     * @since   JSO-0.2
     */
    public synchronized static JSOImplementation getInstance() throws RuntimeException {
        if (_Instance == null)
            _Instance = createInstance(null);
        return _Instance;
    }
    /**
     * <p>
     * Creates a new instance of <tt>JSOImplementation</tt>.</p>
     *
     * <p>
     * The value of <tt>clazz</tt> can be <tt>null</tt> or "", in which
     * case the system property "org.jabberstudio.jso.JSOImplementation.class"
     * is analysed for the fully-qualified class name.</p>
     *
     * <p>
     * Any implementation of <tt>JSOImplementation</tt> must have a public
     * default constructor, which is used to create the instance.</p>
     *
     * <p>
     * This method throws a <tt>RuntimeException</tt> if the requested
     * class is not derived from this class, or if it cannot be
     * instantiated.</p>
     *
     * @param clazz  The classname to create.
     * @return An instance of <tt>JSOImplementation</tt>.
     * @throws RuntimeException If the an instance cannot be created.
     * @since   JSO-0.2
     */
    public static final JSOImplementation createInstance(String clazz) throws RuntimeException {
        JSOImplementation   jso = null;
        Class               cls, check;
        
        try {
            if (!Utilities.isValidString(clazz))
                clazz = System.getProperty(CLASS_PROPERTY, "net.outer_planes.jso.JSO");
            check = JSOImplementation.class;
            cls = Class.forName(clazz);
            
            if (!check.isAssignableFrom(cls))
                throw new ClassNotFoundException(clazz + " is not derived from JSOImplementation");

            jso = (JSOImplementation)cls.newInstance();
        } catch (RuntimeException rte) {
            throw rte;
        } catch (Exception e) {
            throw new RuntimeException("Cannot create JSOImplementation", e);
        }
        
        return jso;
    }
    
    /**
     * <p>
     * Creates a new <tt>StreamDataFactory</tt>, with the registered
     * factories.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The preconfigured <tt>StreamDataFactory</tt>.
     * @since   JSO-0.10
     */
    public abstract StreamDataFactory createDataFactory();
    /**
     * <p>
     * Retrieves the "default" <tt>StreamDataFactory</tt>, with the
     * registered factories. This is the <tt>StreamDataFactory</tt> that
     * instances from {@link #createDataFactory} are created from.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The preconfigured <tt>StreamDataFactory</tt>.
     * @since   JSO-0.10
     */
    public abstract StreamDataFactory getDataFactory();
    
    /**
     * <p>
     * Creates a new <tt>StreamContext</tt>.  The returned
     * <tt>StreamContext</tt> returns <tt>false</tt> for both
     * {@link StreamContext#isInbound} and {@link StreamContext#isOutbound}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return A new <tt>StreamContext</tt>.
     * @since   JSO-0.2
     */
    public abstract StreamContext createStreamContext();
    /**
     * <p>
     * Creates a new <tt>StreamContext</tt> with the given <tt>NSI</tt> and
     * prefix. The returned <tt>StreamContext</tt> returns <tt>false</tt> for
     * both {@link StreamContext#isInbound} and {@link
     * StreamContext#isOutbound}. Also, a declared namespace is added for
     * the namespace URI/prefix provided.</p>
     *
     * <p>
     * The value of <tt>name</tt> should not be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> may be thrown. Implementations may
     * change a <tt>null</tt> value for this to their own default. this
     * <tt>NSI</tt> may be subject to the following preconditions
     * (Implementations may use suitable defaults):</p>
     * <ul>
     * <li>{@link NSI#getLocalName} should not return <tt>null</tt>
     * or "", or an <tt>IllegalArgumentException</tt> may be thrown. </li>
     * <li>{@link NSI#getNamespaceURI} should not return <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> may be thrown.</li>
     * </ul>
     *
     * <p>
     * The value of <tt>prefix</tt> should not be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> may be thrown. Implementations may
     * change a <tt>null</tt> or "" value for this to their own default,
     * including the default prefix ("").</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param name The <tt>NSI</tt> for the new <tt>StreamContext</tt>.
     * @param prefix The namespace prefix for this <tt>StreamContext</tt>.
     * @return The new <tt>StreamContext</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.10
     */
    public abstract StreamContext createStreamContext(NSI name, String prefix) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates a new <tt>Stream</tt> with the given default namespace. The
     * returned <tt>Stream</tt> is setup with an "inbound" and "outbound"
     * <tt>StreamContext</tt> obtained via {@link #createStreamContext}.</p>
     *
     * <p>
     * The value of <tt>ns</tt> should not be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> may be thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param ns The "default" namespace for the <tt>Stream</tt>.
     * @throws IllegalArgumentException If the argument is invalid.
     * @return The <tt>Stream</tt> for the given namespace.
     * @since   JSO-0.2
     */
    public abstract Stream createStream(String ns) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <tt>Stream</tt> with the given default namespace and
     * <tt>StreamContext</tt> objects.</p>
     *
     * <p>
     * The value of <tt>ns</tt> should not be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> may be thrown.</p>
     *
     * <p>
     * If the value of <tt>inCtx</tt> is <tt>null</tt>, a valid
     * <tt>StreamContext</tt> is obtained via {@link
     * #createStreamContext}.</p>
     *
     * <p>
     * If the value of <tt>outCtx</tt> is <tt>null</tt>, a valid
     * <tt>StreamContext</tt> is obtained via {@link
     * #createStreamContext}.</p>
     *
     * @param ns The "default" namespace for the <tt>Stream</tt>.
     * @param inCtx The "inbound" <tt>StreamContext</tt>.
     * @param outCtx the "outbound" <tt>StreamContext</tt>.
     * @throws IllegalArgumentException If an argument is invalid.
     * @return The <tt>Stream</tt> for the given namespace.
     * @since   JSO-0.2
     */
    public abstract Stream createStream(String ns, StreamContext inCtx, StreamContext outCtx) throws IllegalArgumentException;

    /**
     * <p>
     * Creates an <tt>Exporter</tt> for the given type and
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> must be a registered <tt>Exporter</tt>
     * implementation, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param type The type of <tt>Exporter</tt> to create, as a
     * fully-qualified class or interface name.
     * @param ctx The context <tt>StreamElement</tt>.
     * @return The new <tt>Exporter</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws RuntimeException If any errors occur creating the
     * <tt>Exporter</tt>.
     * @since   JSO-0.10
     */
    public abstract Exporter createExporter(String type, StreamElement ctx) throws IllegalArgumentException, RuntimeException;
    /**
     * <p>
     * Creates a <tt>DOMExporter</tt> for the given
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param ctx The context <tt>StreamElement</tt>.
     * @return The <tt>DOMExporter</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws RuntimeException If any errors occur creating the
     * <tt>DOMExporter</tt>.
     * @since   JSO-0.2
     */
    public abstract DOMExporter createDOMExporter(StreamElement ctx) throws IllegalArgumentException, RuntimeException;
    /**
     * <p>
     * Creates a <tt>XMLExporter</tt> for the given
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param ctx The context <tt>StreamElement</tt>.
     * @return The <tt>XMLExporter</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws RuntimeException If any errors occur creating the
     * <tt>DOMExporter</tt>.
     * @since   JSO-0.6
     */
    public abstract XMLExporter createXMLExporter(StreamElement ctx) throws IllegalArgumentException, RuntimeException;

    /**
     * <p>
     * Creates an <tt>Importer</tt> for the given type and
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> must be a registered <tt>Importer</tt>
     * implementation, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param type The type of <tt>Importer</tt> to create, as a
     * fully-qualified class or interface name.
     * @param ctx The context <tt>StreamElement</tt>.
     * @return The new <tt>Importer</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws RuntimeException If any errors occur creating the
     * <tt>Importer</tt>.
     * @since   JSO-0.10
     */
    public abstract Importer createImporter(String type, StreamElement ctx) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a <tt>DOMImporter</tt> for the given
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param ctx The context <tt>StreamElement</tt>.
     * @return The <tt>DOMEImporter</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public abstract DOMImporter createDOMImporter(StreamElement ctx) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a <tt>DOMImporter</tt> for the given
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param ctx The context <tt>StreamElement</tt>.
     * @return The <tt>DOMEImporter</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.6
     */
    public abstract XMLImporter createXMLImporter(StreamElement ctx) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates an <tt>XPath</tt> object for the given expression.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or
     * a <tt>SAXPathException</tt> is thrown.</p>
     *
     * @param expr The XPath expression to create from.
     * @return The created <tt>XPath</tt> object.
     * @throws SAXPathException If there are errors generating the
     * <tt>XPath</tt> object.
     * @since   JSO-0.4
     */
    public abstract XPath createXPath(String expr) throws SAXPathException;
}