/*
 * FeatureConsumerEvent.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;


import java.util.EventObject;

import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.features.FeatureConsumer;
import org.jabberstudio.jso.features.FeaturesetConsumerManager;

/**
 * <p>
 * Class for notification about <tt>FeaturesetConsumerManager</tt> events.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class FeatureConsumerEvent extends EventObject {
    //Fields
    private FeatureConsumer _Consumer;
    private StreamFeature   _Feature;
    private Throwable       _Failure;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>FeatureConsumerEvent</tt> using the given
     * paramters.</p>
     *
     * <p>
     * This constructor does not attempt to validate any of the arguments.</p>
     *
     * @param src The source of the event.
     * @param fc The <tt>FeatureConsumer</tt> consuming.
     * @param feat The <tt>StreamFeature</tt> being consumed.
     * @param t The failure from attempting to consume.
     * @since   JSO-0.12
     */
    public FeatureConsumerEvent(FeaturesetConsumerManager src, FeatureConsumer fc, StreamFeature feat, Throwable t) {
        super(src);
        
        _Consumer = fc;
        _Feature = feat;
        _Failure = t;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>FeaturesetConsumerManager</tt> from which the event
     * is triggered. This method is a convenience over <tt>getSource()</tt> to
     * automatically cast the result to a
     * <tt>FeaturesetConsumerManager</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>FeaturesetConsumerManager</tt> source.
     * @since   JSO-0.12
     */
    public FeaturesetConsumerManager getFeaturesetConsumerManager() {
        return (FeaturesetConsumerManager)getSource();
    }

    /**
     * <p>
     * Retrieves the <tt>FeatureConsumer</tt> consuming.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>FeatureConsumer</tt> in operation.
     * @since   JSO-0.12
     */
    public FeatureConsumer getFeatureConsumer() {
        return _Consumer;
    }
    /**
     * <p>
     * Retrieves the <tt>StreamFeature</tt> being consumed.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamFeature</tt> being consumed.
     * @since   JSO-0.12
     */
    public StreamFeature getStreamFeature() {
        return _Feature;
    }
    /**
     * <p>
     * Retrieves the failure from consumption.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The <tt>Throwable</tt> that occurred during consumption.
     * @since   JSO-0.12
     */
    public Throwable getFailure() {
        return _Failure;
    }
}