/*
 * MonitorEvent.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;


import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.util.Enumerator;
import org.jabberstudio.jso.util.Monitor;
import org.jabberstudio.jso.util.PacketMonitor;

/**
 * <p>
 * Class for notification about monitoring events.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class MonitorEvent extends java.util.EventObject {
    //Inner-classes
    /**
     * <p>
     * Class to enumerate the reason for a <tt>MonitorEvent</tt>.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static final class Reason extends Enumerator {
        //Constants
        private Reason(String name) {
            super(name);
        }
    }

    //"Constants"
    /**
     * Flags that this <tt>MonitorEvent</tt> is triggered by finding the
     * expected <tt>Packet</tt>.
     *
     * @since   JSO-0.12
     */
    public static final Reason FOUND = new Reason("found");
    /**
     * Flags that this <tt>MonitorEvent</tt> is triggered by reaching the
     * timeout limit.
     *
     * @since   JSO-0.12
     */
    public static final Reason TIMEOUT = new Reason("timeout");
    /**
     * Flags that this <tt>MonitorEvent</tt> is triggered by a failure
     * trying to monitor for the expected <tt>Packet</tt>.
     *
     * @since   JSO-0.12
     */
    public static final Reason FAILED = new Reason("failed");

    //Fields
    private Packet          _Expected, _Actual;
    private Reason          _Reason;
    private StreamException _Cause;

    //Constructors
    /**
     * <p>
     * Creates a new <tt>MonitorEvent</tt> with the given source, actual
     * <tt>Packet</tt>, reason, and cause.</p>
     *
     * <p>
     * This constructor does not attempt to validate any of the arguments.</p>
     *
     * @param src The source of the event.
     * @param actual The <tt>Packet</tt> actually received.
     * @param reason The reason for the event.
     * @param cause The cause of a failed event.
     * @since   JSO-0.12
     */
    protected MonitorEvent(Monitor src, Packet actual, Reason reason, StreamException cause) {
        super(src);

        _Actual = actual;
        _Reason = reason;
        _Cause = cause;
    }

    //Methods
    /**
     * <p>
     * Retrieves the <tt>Monitor</tt> triggering this
     * <tt>MonitorEvent</tt>. This method is a convenience over
     * <tt>getSource()</tt> to automatically cast the result to a
     * <tt>Monitor</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>Monitor</tt> source of this <tt>MonitorEvent</tt>.
     * @since   JSO-0.12
     */
    public Monitor getMonitor() {
        return (Monitor)getSource();
    }
    /**
     * <p>
     * Retrieves the actual <tt>Packet</tt> received for this
     * <tt>MonitorEvent</tt>.</p>
     *
     * <p>
     * If {@link #getReason} returns {@link #FOUND}, the value returned by
     * this method is never <tt>null</tt>. Otherwise, it is always
     * <tt>null</tt>.</p>
     *
     * @return The <tt>Packet</tt> expected for this <tt>MonitorEvent</tt>.
     * @since   JSO-0.12
     */
    public Packet getActualPacket() {
        return _Actual;
    }
    /**
     * <p>
     * Retrieves the reason for this <tt>MonitorEvent</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The reason for this <tt>MonitorEvent</tt>.
     * @since   JSO-0.12
     */
    public Reason getReason() {
        return _Reason;
    }
    /**
     * <p>
     * Retrieves the cause for this <tt>MonitorEvent</tt>.</p>
     *
     * <p>
     * If {@link #getReason} returns {@link #FAILED}, the value returned by
     * this method is never <tt>null</tt>. Otherwise, it is always
     * <tt>null</tt>.</p>
     *
     * @return The cause of this <tt>MonitorEvent</tt>.
     * @since   JSO-0.12
     */
    public StreamException getCause() {
        return _Cause;
    }
}