/*
 * PacketMonitorEvent.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;


import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.util.PacketMonitor;

/**
 * <p>
 * Class for monitoring events specific to <tt>PacketMonitor</tt>.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class PacketMonitorEvent extends MonitorEvent {
    //Fields
    private Packet  _Expected;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>PacketMonitorEvent</tt> with the given source,
     * expected and actual <tt>Packet</tt>, reason, and cause.</p>
     *
     * <p>
     * This constructor does not attempt to validate any of the arguments.</p>
     *
     * @param src The source of the event.
     * @param expect The <tt>Packet</tt> "template" expected.
     * @param actual The <tt>Packet</tt> actually received.
     * @param reason The reason for the event.
     * @param cause The cause of a failed event.
     * @since   JSO-0.12
     */
    protected PacketMonitorEvent(PacketMonitor src, Packet expect, Packet actual, Reason reason, StreamException cause) {
        super(src, actual, reason, cause);

        _Expected = expect;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>PacketMonitor</tt> triggering this
     * <tt>PacketMonitorEvent</tt>. This method is a convenience over
     * <tt>getSource()</tt> to automatically cast the result to a
     * <tt>PacketMonitor</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>PacketMonitor</tt> source of this
     * <tt>PacketMonitorEvent</tt>.
     * @since   JSO-0.12
     */
    public PacketMonitor getPacketMonitor() {
        return (PacketMonitor)getSource();
    }
    /**
     * <p>
     * Retrieves the expected <tt>Packet</tt> "template" for this
     * <tt>PacketMonitorEvent</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>Packet</tt> expected for this <tt>PacketMonitorEvent</tt>.
     * @since   JSO-0.12
     */
    public Packet getExpectedPacket() {
        return _Expected;
    }
    
    /**
     * <p>
     * Creates a new <tt>PacketMonitorEvent</tt> for the {@link #FOUND}
     * trigger.</p>
     *
     * <p>
     * The value of <tt>mon</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expect</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>actual</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param mon The source <tt>PacketMonitor</tt> of the event.
     * @param expect The <tt>Packet</tt> "template" expected.
     * @param actual The <tt>Packet</tt> actually received.
     * @return The <tt>PacketMonitorEvent</tt>.
     * @throws IllegalArgumentException If any paramters are invalid.
     * @since   JSO-0.12
     */
    public static PacketMonitorEvent createFoundEvent(PacketMonitor mon, Packet expect, Packet actual) throws IllegalArgumentException {
        //Validate input
        if (mon == null)
            throw new IllegalArgumentException("PacketMonitor cannot be null");
        if (expect == null)
            throw new IllegalArgumentException("Expected Packet cannot be null");
        if (actual == null)
            throw new IllegalArgumentException("Actual Packet cannot be null");
        
        return new PacketMonitorEvent(mon, expect, actual, FOUND, null);
    }
    /**
     * <p>
     * Creates a new <tt>PacketMonitorEvent</tt> for the {@link #TIMEOUT}
     * trigger.</p>
     *
     * <p>
     * The value of <tt>mon</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expect</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param mon The source <tt>PacketMonitor</tt> of the event.
     * @param expect The <tt>Packet</tt> "template" expected.
     * @return The <tt>PacketMonitorEvent</tt>.
     * @throws IllegalArgumentException If any paramters are invalid.
     * @since   JSO-0.12
     */
    public static PacketMonitorEvent createTimeoutEvent(PacketMonitor mon, Packet expect) throws IllegalArgumentException {
        PacketMonitorEvent  evt;
        
        //Validate input
        if (mon == null)
            throw new IllegalArgumentException("PacketMonitor cannot be null");
        if (expect == null)
            throw new IllegalArgumentException("Expected Packet cannot be null");
        
        return new PacketMonitorEvent(mon, expect, null, TIMEOUT, null);
    }
    /**
     * <p>
     * Creates a new <tt>PacketMonitorEvent</tt> for the {@link #FAILED}
     * trigger.</p>
     *
     * <p>
     * The value of <tt>mon</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expect</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>cause</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param mon The source <tt>PacketMonitor</tt> of the event.
     * @param expect The <tt>Packet</tt> "template" expected.
     * @param cause The cause of a failed event.
     * @return The <tt>PacketMonitorEvent</tt>.
     * @throws IllegalArgumentException If any paramters are invalid.
     * @since   JSO-0.12
     */
    public static MonitorEvent createFailedEvent(PacketMonitor mon, Packet expect, StreamException cause) throws IllegalArgumentException {
        PacketMonitorEvent  evt;
        
        //Validate input
        if (mon == null)
            throw new IllegalArgumentException("PacketMonitor cannot be null");
        if (expect == null)
            throw new IllegalArgumentException("Expected Packet cannot be null");
        if (cause == null)
            throw new IllegalArgumentException("Cause cannot be null");
        
        return new PacketMonitorEvent(mon, expect, null, FAILED, cause);
    }
}