/*
 * StreamFeaturesEvent.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;


import java.util.List;

import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamFeatureset;

/**
 * <p>
 * Event for reported <tt>StreamFeatureset</tt> objects. Instances of this
 * class are used in conjunction with <tt>StreamFeaturesListener</tt>to report
 * the announcement (outbound) or availability (inbound) of features.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class StreamFeaturesEvent extends PacketEvent {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>StreamFeaturesEvent</tt> with the given
     * <tt>StreamContext</tt>, <tt>StreamFeatureset</tt>, and
     * <tt>PacketEvent.Type</tt>.  The given <tt>StreamContext</tt> is the
     * source (as returned by <tt>getSource()</tt>), and <tt>data</tt> is the
     * <tt>StreamFeaturesEvent</tt> object received or sent.</p>
     *
     * @param src The <tt>StreamContext</tt> event source.
     * @param data The <tt>StreamFeaturesEvent</tt> data received/sent.
     * @param type The <tt>PacketEvent.Type</tt>.
     * @since   JSO-0.12
     */
    public StreamFeaturesEvent(StreamContext src, StreamFeatureset data, PacketEvent.Type type) {
        super(src, data, type);
    }
    /**
     * <p>
     * Creates a new <tt>StreamFeaturesEvent</tt> that is a duplicate of the
     * given <tt>StreamFeaturesEvent</tt>. The references that <tt>og</tt>
     * contains are referenced by this new <tt>StreamFeaturesEvent</tt>, not
     * copied.</p>
     *
     * @param og The <tt>StreamFeaturesEvent</tt> to copy from.
     * @since   JSO-0.12
     */
    protected StreamFeaturesEvent(StreamFeaturesEvent og) {
        super(og);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>StreamFeatureset</tt> that was received or sent.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamFeatureset</tt> data received or sent.
     * @since   JSO-0.12
     */
    public StreamFeatureset getFeatureset() {
        return (StreamFeatureset)getData();
    }
    /**
     * <p>
     * Retrieves the list of reported <tt>StreamFeature</tt> objects.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is an instance of <tt>StreamFeature</tt>.</p>
     *
     * @return The list of reported <tt>StreamFeature</tt> objects.
     * @since   JSO-0.12
     */
    public List listFeatures() {
        return getFeatureset().listFeatures();
    }
    
    /**
     * <p>
     * Clones this <tt>StreamFeaturesEvent</tt> object. This operation performs
     * a "shallow copy" of the <tt>StreamFeaturesEvent</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The clone of this <tt>StreamFeaturesEvent</tt>.
     * @since   JSO-0.12
     */
    public Object clone() {
        return new StreamFeaturesEvent(this);
    }
}