/*
 * FeatureContext.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.features;


import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.event.PacketDispatcher;

/**
 * <p>
 * Interface to provide context information to a <tt>FeatureConsumer</tt>.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface FeatureContext {
    //Methods
    /**
     * <p>
     * Retrieves the <tt>StreamFeature</tt>  being consumed.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamFeature</tt> being consumed.
     * @since   JSO-0.12
     */
    public StreamFeature getFeature();
    /**
     * <p>
     * Retrieves the <tt>PacketRouter</tt> for this <tt>FeatureContext</tt>.
     * This is used for sending and processing <tt>Packet</tt> data.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>PacketRouter</tt> for this <tt>FeatureContext</tt>.
     * @since   JSO-0.12
     */
    public PacketRouter getRouter();
    /**
     * <p>
     * Retrieves the <tt>PacketDispatcher</tt> for this <tt>FeatureContext</tt>.
     * This is used for registering and unregistering listeners for
     * <tt>Packet</tt> events.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>PacketDispatcher</tt> for this <tt>FeatureContext</tt>.
     * @since   JSO-0.12
     */
    public PacketDispatcher getDispatcher();
    
    /**
     * <p>
     * Completes the <tt>FeatureConsumer</tt>, in a failed state.</p>
     *
     * <p>
     * The value of <tt>t</tt> may be <tt>null</tt>.</p>
     *
     * @param t The cause of the failure.
     */
    public void completeFailed(Throwable t);
    /**
     * <p>
     * Completes the <tt>FeatureConsumer</tt>, in a succeeded state.</p>
     *
     * <p>
     * If <tt>restart</tt> is <tt>true</tt>, the <tt>Stream</tt> attached to
     * the owning <tt>FeaturesetConsumerManager</tt> is dropped, then reopened.
     * Otherwise, further processing is done on the last
     * <tt>StreamFeatureset</tt> reported.</p>
     *
     * @param restart <tt>true</tt> if the <tt>Stream</tt> should be restarted.
     * @since   JSO-0.12
     */
    public void completeSucceeded(boolean restart);
}