/*
 * DateTimeProfileFormat.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.format;


import java.text.AttributedCharacterIterator;
import java.text.AttributedString;
import java.text.FieldPosition;
import java.text.Format;
import java.text.ParseException;
import java.text.ParsePosition;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Class for formatting <tt>Date</tt> objects to and from the date/time
 * profile, as defined in JEP-0082. Instances of this class can handle both
 * formatting and parsing for one of the defined profiles (date, time, or
 * date/time).</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0082.html'>JEP-0082: Jabber
 * Date and Time Profiles.</a>
 */
public class DateTimeProfileFormat extends Format implements Cloneable {
    //Fields
    private static class Field extends Format.Field {
        //Constructors
        public Field(String name) {
            super(name);
        }
    }
    
    //"Constants"
    /** Constant for "date" profile. The format for this profile is
     * "YYYY-MM-dd".
     *
     * @since   JSO-0.12
     */
    public static final int DATE                    = 1;
    /** Constant for "time" profile. The format for this profile is
     * "HH:mm:ss[.ms]TZD".
     *
     * @since   JSO-0.12
     */
    public static final int TIME                    = 2;
    /** Constant for "date/time" profile. The format for this profile is
     * "YYYY-MM-dd'T'HH:mm:ss[.ms]TZD".
     *
     * @since   JSO-0.12
     */
    public static final int DATETIME                = 3;
    
    /** Constant for "year" field alignment. Used in <tt>FieldPosition</tt> in
     * formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    YEAR_FIELD      = new Field("year");
    /** Constant for "month" field alignment. Used in <tt>FieldPosition</tt> in
     * formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    MONTH_FIELD     = new Field("month");
    /** Constant for "day" field alignment. Used in <tt>FieldPosition</tt> in
     * formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    DAY_FIELD       = new Field("day");
    /** Constant for "hour" field alignment. Used in <tt>FieldPosition</tt> in
     * formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    HOUR_FIELD      = new Field("hour");
    /** Constant for "minute" field alignment. Used in <tt>FieldPosition</tt> in
     * formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    MINUTE_FIELD    = new Field("minute");
    /** Constant for "second" field alignment. Used in <tt>FieldPosition</tt> in
     * formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    SECOND_FIELD    = new Field("second");
    /** Constant for "timezone" field alignment. Used in <tt>FieldPosition</tt>
     * in formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Format.Field    TIMEZONE_FIELD  = new Field("timezone");
    
    private static final int    YEAR_GROUP          = 1;
    private static final int    MONTH_GROUP         = 2;
    private static final int    DAY_GROUP           = 3;
    private static final int    HOUR_GROUP          = 1;
    private static final int    MINUTE_GROUP        = 2;
    private static final int    SECOND_GROUP        = 3;
    private static final int    TIMEZONE_GROUP      = 5;
    
    private static final DateTimeProfileFormat  DATE_INSTANCE;
    private static final DateTimeProfileFormat  TIME_INSTANCE;
    private static final DateTimeProfileFormat  DATETIME_INSTANCE;
    
    static {
        String  reDate, reTime;
        Pattern ptn;
        
        reDate = "(\\d{1,})-(\\d{2})-(\\d{2})";
        reTime = "(\\d{2}):(\\d{2}):(\\d{2}(\\.\\d+)?)(Z|[-+]\\d{2}:\\d{2})";
        
        DATE_INSTANCE       = new DateTimeProfileFormat(DATE, Pattern.compile(reDate));
        TIME_INSTANCE       = new DateTimeProfileFormat(TIME, Pattern.compile(reTime));
        DATETIME_INSTANCE   = new DateTimeProfileFormat(DATETIME, Pattern.compile(reDate  + "T" + reTime));
    }
    
    //Fields
    private int         _Profile;
    private Pattern     _Pattern;
    private Calendar    _Cal;
    
    //Constructors
    private DateTimeProfileFormat(int profile, Pattern ptn) {
        _Profile = profile;
        _Pattern = ptn;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the profile for this <tt>DateTimeProfileFormat</tt>.</p>
     *
     * <p>
     * The value returned by this method is always one of {@link
     * #DATE}, {@link #TIME}, or {@link
     * #DATETIME}.</p>
     *
     * @return The profile for this <tt>DateTimeProfileFormat</tt>.
     * @since   JSO-0.12
     */
    public final int getProfile() {
        return _Profile;
    }

    /**
     * <p>
     * Retrieves the timezone for this <tt>DateTimeProfileFormat</tt>. This is
     * the timezone in which dates are formatted.</p>
     *
     * <p><b>NOTE: This value will reflect the parsed timezone if this
     * <tt>DateTimeProfileFormat</tt> is used for parsing.</b></p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The timezone to format by.
     * @since   JSO-0.12
     */
    public TimeZone getTimeZone() {
        return getCalendar().getTimeZone();
    }
    /**
     * <p>
     * Sets the timezone for this <tt>DateTimeProfileFormat</tt>.</p>
     *
     * <p><b>NOTE: {@link #getTimeZone} reflect the parsed timezone if this
     * <tt>DateTimeProfileFormat</tt> is used for parsing, and not
     * <tt>tz</tt>.</b></p>
     *
     * <p>
     * The value of <tt>tz</tt> may be <tt>null</tt>, in which case the default
     * timezone for the current platform is used.</p>
     * 
     * @param tz The new timezone.
     * @since   JSO-0.12
     */
    public void setTimeZone(TimeZone tz) {
        getCalendar().setTimeZone((tz != null) ? tz : TimeZone.getDefault());
    }
    
    private final Pattern getPattern() {
        return _Pattern;
    }
    private Calendar getCalendar() {
        if (_Cal == null) {
            _Cal = Calendar.getInstance();
        }
        
        return _Cal;
    }
    private Calendar resetCalendar() {
        Calendar        cal = getCalendar();
        
        _Cal.set(0, 0, 0, 0, 0, 0);
        _Cal.set(Calendar.ERA, 1);
        _Cal.set(Calendar.MILLISECOND, 0);
        
        return cal;
    }
    
    /**
     * <p>
     * Formats the given <tt>Date</tt> into the appropriate date/time profile
     * string.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * <p>
     * The value of <tt>obj</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param obj The date to format.
     * @return The date/time string, appropriate to the profile.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public String format(Date obj) throws IllegalArgumentException {
        return format((Object)obj);
    }
    /**
     * <p>
     * Formats the date into the appropriate date/time profile.</p>
     *
     * <p>
     * The value returned by this method is the <tt>StringBuffer</tt> used for
     * <tt>buffer</tt>, with the formatted date/time profile string
     * appended.</p>
     *
     * <p>
     * The value of <tt>obj</tt> must be a non-<tt>null</tt> instance of
     * <tt>Date</tt> or <tt>Number</tt>, or an <tt>IllegalArgumentException</tt>
     * is thrown.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>pos</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param obj The object to parse.
     * @param buffer The <tt>StringBuffer</tt> to append the formatted text to.
     * @param pos The <tt>FieldPosition</tt> used to mark a particular field
     * within the formatted text.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws NullPointerException If <tt>buffer</tt> or <tt>pos</tt> are
     * <tt>null</tt>.
     * @since   JSO-0.12
     */
    public StringBuffer format(Object obj, StringBuffer buffer, FieldPosition pos) throws IllegalArgumentException, NullPointerException {
        AttributedCharacterIterator     itr = formatToCharacterIterator(obj, buffer);
        
        if (pos != null) {
            //Determine date
            if (pos.getFieldAttribute() == YEAR_FIELD) {
                pos.setBeginIndex(itr.getRunStart(YEAR_FIELD));
                pos.setEndIndex(itr.getRunLimit(YEAR_FIELD));
            }
            itr.setIndex(itr.getRunStart(YEAR_FIELD));
            
            if (pos.getFieldAttribute() == MONTH_FIELD) {
                pos.setBeginIndex(itr.getRunStart(MONTH_FIELD));
                pos.setEndIndex(itr.getRunLimit(MONTH_FIELD));
            }
            itr.setIndex(itr.getRunStart(MONTH_FIELD));
            
            if (pos.getFieldAttribute() == DAY_FIELD) {
                pos.setBeginIndex(itr.getRunStart(DAY_FIELD));
                pos.setEndIndex(itr.getRunLimit(DAY_FIELD));
            }
            itr.setIndex(itr.getRunStart(DAY_FIELD));
            
            //Determine time
            if (pos.getFieldAttribute() == HOUR_FIELD) {
                pos.setBeginIndex(itr.getRunStart(HOUR_FIELD));
                pos.setEndIndex(itr.getRunLimit(HOUR_FIELD));
            }
            itr.setIndex(itr.getRunStart(HOUR_FIELD));
            
            if (pos.getFieldAttribute() == MINUTE_FIELD) {
                pos.setBeginIndex(itr.getRunStart(MINUTE_FIELD));
                pos.setEndIndex(itr.getRunLimit(MINUTE_FIELD));
            }
            itr.setIndex(itr.getRunStart(MINUTE_FIELD));
            
            if (pos.getFieldAttribute() == SECOND_FIELD) {
                pos.setBeginIndex(itr.getRunStart(SECOND_FIELD));
                pos.setEndIndex(itr.getRunLimit(SECOND_FIELD));
            }
            itr.setIndex(itr.getRunStart(SECOND_FIELD));
            
            if (pos.getFieldAttribute() == TIMEZONE_FIELD) {
                pos.setBeginIndex(itr.getRunStart(TIMEZONE_FIELD));
                pos.setEndIndex(itr.getRunLimit(TIMEZONE_FIELD));
            }
            itr.setIndex(itr.getRunStart(TIMEZONE_FIELD));
        }
        
        return buffer;
    }
    /**
     * <p>
     * Formats a date into an <tt>AttributedStringIterator</tt> using the
     * appropriate date/time profile. The returned iterator can be used to
     * build the resulting string, as well as to determine information about
     * the resulting string.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each attribute
     * in the returned iterator is an instance of type
     * <tt>Format.Field</tt>. The order to examine the attributes is:</p>
     * <ol>
     * <li>{@link #YEAR_FIELD}</li>
     * <li>{@link #MONTH_FIELD}</li>
     * <li>{@link #DAY_FIELD}</li>
     * <li>{@link #HOUR_FIELD}</li>
     * <li>{@link #MINUTE_FIELD}</li>
     * <li>{@link #SECOND_FIELD}</li>
     * <li>{@link #TIMEZONE_FIELD}</li>
     * </ol>
     *
     * <p>
     * The value of <tt>obj</tt> must be a non-<tt>null</tt> instance of
     * <tt>Date</tt>, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param obj The date to format.
     * @return The <tt>AttributedCharacterIterator</tt> describing the resulting
     * string.
     * @throws IllegalArgumentException If the paramter is invalid.
     * @since   JSO-0.12
     */
    public AttributedCharacterIterator formatToCharacterIterator(Object obj) throws IllegalArgumentException {
        return formatToCharacterIterator(obj, new StringBuffer());
    }
    private AttributedCharacterIterator formatToCharacterIterator(Object obj, StringBuffer toAppendTo) throws IllegalArgumentException {
        Map                 attrPos, attrMap;
        Iterator            itr;
        AttributedString    attrStr;
        StringBuffer        buffer = new StringBuffer();
        Calendar            cal;
        Date                date = null;
        int                 start, finish;

        //Prep input
        if (obj instanceof Number)
            date = new Date(((Number)obj).longValue());
        else if (obj instanceof Date)
            date = (Date)obj;
        else
            throw new IllegalArgumentException("cannot parse object");
        date = (Date)obj;
        attrPos = new java.util.HashMap();
        attrMap = new java.util.HashMap();
        cal = resetCalendar();
        cal.setTime(date);

        start = toAppendTo.length();
        //Format date
        if ((getProfile() & DATE) == DATE) {
            attrPos.putAll(formatDate(cal, buffer));
        }
        //Format time
        if ((getProfile() & TIME) == TIME) {
            attrPos.putAll(formatTime(cal, buffer));
        }
        toAppendTo.append(buffer);
        finish = toAppendTo.length();
        
        //setup "global" attributes; adjust position
        itr = attrPos.entrySet().iterator();
        while (itr.hasNext()) {
            Map.Entry           entry = (Map.Entry)itr.next();
            Format.Field        key = (Format.Field)entry.getKey();
            FieldPosition       pos = (FieldPosition)entry.getValue();
            String              value = buffer.substring(pos.getBeginIndex(), pos.getEndIndex());

            attrMap.put(key, value);
        }
        
        //Setup attributed string
        attrStr = new AttributedString(buffer.toString(), attrMap);
        itr = attrPos.entrySet().iterator();
        while (itr.hasNext()) {
            Map.Entry           entry = (Map.Entry)itr.next();
            Format.Field        key = (Format.Field)entry.getKey();
            FieldPosition       pos = (FieldPosition)entry.getValue();
            String              value = buffer.substring(pos.getBeginIndex(), pos.getEndIndex());

            attrStr.addAttribute(key, value, start + pos.getBeginIndex(), start + pos.getEndIndex());
        }
        
        return attrStr.getIterator();
    }
    
    private final FieldPosition formatField(StringBuffer buffer, int val, int size, String prefix, String suffix, FieldPosition pos) {
        int             idx, place;
        
        pos.setBeginIndex(buffer.length());
        if (Utilities.isValidString(prefix))
            buffer.append(prefix);
        for (idx = (size - 1); idx > 0; idx--) {
            place = (int)Math.pow(10, idx);
            
            if ((val / place) == 0)
                buffer.append("0");
        }
        buffer.append(val);
        if (Utilities.isValidString(suffix))
            buffer.append(suffix);
        pos.setEndIndex(buffer.length());
        
        return pos;
    }
    private final FieldPosition formatField(StringBuffer buffer, double val, int size, String prefix, String suffix, FieldPosition pos) {
        int             idx, place;
        
        pos.setBeginIndex(buffer.length());
        if (Utilities.isValidString(prefix))
            buffer.append(prefix);
        for (idx = (size - 1); idx > 0; idx--) {
            place = (int)Math.pow(10, idx);
            
            if (((int)(val / place)) == 0)
                buffer.append("0");
        }
        buffer.append(val);
        if (Utilities.isValidString(suffix))
            buffer.append(suffix);
        pos.setEndIndex(buffer.length());
        
        return pos;
    }
    private final Map formatDate(Calendar cal, StringBuffer buffer) {
        Map                 attrs = new java.util.HashMap();
        FieldPosition       pos;
        int                 begin, end;

        pos = formatField(buffer, cal.get(Calendar.YEAR), 1, "", "-", new FieldPosition(YEAR_FIELD));
        attrs.put(YEAR_FIELD, pos);
        pos = formatField(buffer, cal.get(Calendar.MONTH) + 1, 2, "", "-", new FieldPosition(MONTH_FIELD));
        attrs.put(MONTH_FIELD, pos);
        formatField(buffer, cal.get(Calendar.DAY_OF_MONTH), 2, "", "", new FieldPosition(DAY_FIELD));
        attrs.put(DAY_FIELD, pos);
        
        return attrs;
    }
    private final Map formatTime(Calendar cal, StringBuffer buffer) {
        Map             attrs = new java.util.HashMap();
        FieldPosition   pos;
        int             offset, hour, min;
        int             start, end;
        long            time = cal.getTimeInMillis();
        double          sec = ((double)cal.get(Calendar.SECOND)) + (((double)cal.get(Calendar.MILLISECOND)) / 1000);
        TimeZone        tz = cal.getTimeZone();

        pos = formatField(buffer, cal.get(Calendar.HOUR_OF_DAY), 2, ((getProfile() == DATETIME) ? "T" : ""), ":", new FieldPosition(HOUR_FIELD));
        attrs.put(HOUR_FIELD, pos);
        pos = formatField(buffer, cal.get(Calendar.MINUTE), 2, "", ":", new FieldPosition(MINUTE_FIELD));
        attrs.put(MINUTE_FIELD, pos);
        pos = formatField(buffer, sec, 2, "", "", new FieldPosition(SECOND_FIELD));
        attrs.put(SECOND_FIELD, pos);

        pos = new FieldPosition(TIMEZONE_FIELD);
        pos.setBeginIndex(buffer.length());
        if (tz.equals(TimeZone.getTimeZone("UTC"))) {
            buffer.append("Z");
        } else {
            offset = tz.getOffset(time);
            
            buffer.append((offset < 0) ? "-" : "+");
            
            offset = Math.abs(offset);
            hour = offset / 3600000;
            min = (offset % 3600000) / 60000;
            
            if (hour < 10)
                buffer.append("0");
            buffer.append(hour);
            buffer.append(":");
            if (min < 10)
                buffer.append("0");
            buffer.append(min);
        }
        pos.setEndIndex(buffer.length());
        
        return attrs;
    }

    /**
     * <p>
     * Parses the given string into a <tt>Date</tt> object. Parsing begins at
     * the first character of the given string, and may not use up all of the
     * string. The parsing is strict to the profile.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>src</tt> contain a valid representation of a
     * <tt>Date</tt> for the current profile, or a <tt>ParseException</tt> is
     * thrown.</p>
     *
     * @param src The string to parse.
     * @return The parsed date.
     * @throws ParseException If the source could not be parsed.
     * @since   JSO-0.12
     */
    public Date parse(String src) throws ParseException {
        return (Date)parseObject(src);
    }
    /**
     * <p>
     * Parses the given string into a <tt>Date</tt> object, starting at the
     * given position. Parsing may not use up all of the string. The parsing is
     * strict to the profile.</p>
     *
     * <p>
     * The method attempts to parse text starting at the index given by pos. If
     * parsing succeeds, then the index of <tt>pos</tt> is updated to the index
     * after the last character used (parsing does not necessarily use all
     * characters up to the end of the string), and the parsed date is
     * returned. The updated <tt>pos</tt> can be used to indicate the starting
     * point for the next call to this method. If an error occurs, then the
     * index of <tt>pos</tt> is not changed, the error index of <tt>pos</tt> is
     * set to the index of the character where the error occurred, and
     * <tt>null</tt> is returned.
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the string
     * could not be parsed.</p>
     *
     * <p>
     * The value of <tt>src</tt> should contain a valid representation of a
     * <tt>Date</tt> for the current profile at the given position.</p>
     *
     * <p>
     * The value of <tt>pos</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param src The string to parse.
     * @param pos The position to start parsing, and the indicator of where
     * parsing failed or ended.
     * @return The parsed date.
     * @throws NullPointerException If <tt>pos</tt> is <tt>null</tt>.
     * @since   JSO-0.12
     */
    public Object parseObject(String src, ParsePosition pos) throws NullPointerException {
        Calendar    cal = resetCalendar();
        Date        obj = null;
        Matcher     m = getPattern().matcher((src != null) ? src : "");
        
        if (m.find(pos.getIndex())) {
            pos.setIndex(m.end());
            
            if ((getProfile() & DATE) == DATE)
                parseDate(cal, m);
            if ((getProfile() & TIME) == TIME)
                parseTime(cal, m);
            
            obj = cal.getTime();
        } else {
            pos.setErrorIndex(pos.getIndex());
        }
        
        return obj;
    }
    private final void parseDate(Calendar cal, Matcher m) {
        String  val;
        
        val = m.group(YEAR_GROUP);
        cal.set(Calendar.YEAR, Integer.parseInt(val));
        
        val = m.group(MONTH_GROUP);
        cal.set(Calendar.MONTH, Integer.parseInt(val) - 1);
        
        val = m.group(DAY_GROUP);
        cal.set(Calendar.DAY_OF_MONTH, Integer.parseInt(val));
    }
    private final void parseTime(Calendar cal, Matcher m) {
        String      val;
        int         offset = ((getProfile() == DATETIME) ? 3 : 0);
        double      raw = Double.parseDouble(m.group(offset + SECOND_GROUP));
        int         sec, ms;
        TimeZone    tz;
        
        //Setup second and milliseconds
        sec = (int)raw;
        ms = (int)((raw - sec) * 1000);
        
        val = m.group(offset + HOUR_GROUP);
        cal.set(Calendar.HOUR_OF_DAY, Integer.parseInt(val));
        
        val = m.group(offset + MINUTE_GROUP);
        cal.set(Calendar.MINUTE, Integer.parseInt(val));

        cal.set(Calendar.SECOND, sec);
        cal.set(Calendar.MILLISECOND, ms);
        
        val = m.group(offset + TIMEZONE_GROUP);
        if (Utilities.equateStrings(val, "Z")) {
            tz = TimeZone.getTimeZone("UTC");
        } else {
            tz = TimeZone.getTimeZone("GMT" + val);
        }
        cal.setTimeZone(tz);
        cal.set(Calendar.ZONE_OFFSET, tz.getRawOffset());
    }
    
    /**
     * <p>
     * Creates a duplicate of this <tt>DateTimeProfileFormat</tt>. The cloned
     * object uses the same profile and timezone information.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The duplicate of this <tt>DateTimeProfileFormat</tt>.
     * @since   JSO-0.12
     */
    public Object clone() {
        DateTimeProfileFormat   fmt = new DateTimeProfileFormat(getProfile(), getPattern());
        fmt._Cal = this._Cal;
        
        return fmt;
    }
    
    /**
     * <p>
     * Retrieves a date/time formatter instance appropriate to the given
     * profile.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Note that each
     * invocation of this method with the same argument returns a different
     * object.</p>
     *
     * <p>
     * The value of <tt>profile</tt> must be one of {@link #DATE},
     * {@link #TIME}, or {@link #DATETIME}, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param profile The profile to retrieve.
     * @return The appropriate <tt>DateTimeProfileFormat</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public static DateTimeProfileFormat getInstance(int profile) throws IllegalArgumentException {
        DateTimeProfileFormat   fmt = null;

        switch (profile) {
            case DATE:      fmt = DATE_INSTANCE;        break;
            case TIME:      fmt = TIME_INSTANCE;        break;
            case DATETIME:  fmt = DATETIME_INSTANCE;    break;
            default:        throw new IllegalArgumentException("profile is not valid");
        }
        
        return (DateTimeProfileFormat)fmt.clone();
    }
}