/*
 * SASLFeatureConsumer.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import java.security.NoSuchAlgorithmException;
import java.util.Collections;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.features.AbstractFeatureConsumer;
import org.jabberstudio.jso.features.FeatureContext;
import org.jabberstudio.jso.util.XPathListener;
import org.jabberstudio.jso.xpath.XPathSupport;

/**
 * <p>
 * <tt>FeatureConsumer</tt> implementation for consuming SASL.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class SASLFeatureConsumer extends AbstractFeatureConsumer {
    //Inner-classes
    private class SASLWatcher extends XPathListener {
        //Constructors
        public SASLWatcher(XPathSupport support) throws SAXPathException {
            super(support, "sasl:*", Collections.singletonMap("sasl", SASLPacket.NAMESPACE));
        }
        
        //Methods
        public void packetMatched(PacketEvent evt) {
            FeatureContext          fc = getFeatureContext();
            SASLMechanism           mech = getMechanism();
            SASLMechanism.Status    stat;
            SASLPacket              input = (SASLPacket)evt.getData(), output;
            SASLFailurePacket       failed;
            
            try {
                output = mech.evaluate(input);
                fc.getRouter().send(output);
                
                stat = mech.getCurrentStatus();
                if (stat.isComplete()) {
                    fc.getDispatcher().removePacketListener(PacketEvent.RECEIVED, this);
                    if (stat == SASLMechanism.SUCCEEDED) {
                        SASLFeatureConsumer.this.completeSucceededConsumption(true);
                    } else {
                        throw new SASLFailedException((SASLFailurePacket)((input instanceof SASLFailurePacket) ? input : null));
                    }
                }
            } catch (Exception e) {
                fc.getDispatcher().removePacketListener(PacketEvent.RECEIVED, this);
                completeFailedConsumption(e);
            }
        }
    }
    
    //Fields
    private SASLClientInfo      _Client;
    private SASLMechanism       _Mech;
    
    private FeatureContext      _FC;
    
    //Constructors
    /**
     * <p>Creates a new <tt>SASLFeatureConsumer</tt>. This constructor creates
     * a default <tt>SASLClientInfo</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public SASLFeatureConsumer() {
        this(null);
    }
    /**
     * <p>
     * Creates a new <tt>SASLFeatureConsumer</tt> with the given
     * <tt>SASLClientInfo</tt>.</p>
     *
     * <p>
     * The value of <tt>sci</tt> may be <tt>null</tt>, in which case a suitable
     * default instance is created.</p>
     *
     * @param sci The <tt>SASLClientInfo</tt> to use.
     * @since   JSO-0.12
     */
    public SASLFeatureConsumer(SASLClientInfo sci) {
        if (sci == null)
            sci = new SASLClientInfo();
        _Client = sci;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>SASLClientInfo</tt> for this
     * <tt>SASLFeatureConsumer</tt>. This is used for obtaining and using the
     * correct <tt>SASLMechanism</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>SASLClientInfo</tt> in use.
     * @since   JSO-0.12
     */
    public SASLClientInfo getClientInfo() {
        return _Client;
    }

    /**
     * <p>
     * Retrieves the <tt>SASLMechanism</tt> for this
     * <tt>SASLFeatureConsumer</tt>. This is provided for any information that
     * is available after SASL is completed.</p>
     *
     * <p>
     * This method may return <tt>null</tt> if SASL has not been started.</p>
     *
     * @return The <tt>SASLMechanism</tt> in use.
     * @since   JSO-0.12
     */
    public SASLMechanism getMechanism() {
        return _Mech;
    }
    private void setMechanism(SASLMechanism mech) {
        _Mech = mech;
    }
    
    /**
     * <p>
     * Finds the <tt>StreamFeature</tt> in the given <tt>StreamFeatureset</tt>
     * that can be consumed. This implementation searches for the first
     * <tt>SASLMechanismsFeature</tt> in <tt>fs</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a consumable
     * <tt>StreamFeature</tt> could not be found.</p>
     *
     * <p>
     * The value of <tt>fs</tt> will never be <tt>null</tt>.</p>
     *
     * @param fs The <tt>StreamFeatureset</tt> to search in
     * @return The consumable <tt>StreamFeature</tt>.
     * @since   JSO-0.12
     */
    public StreamFeature findFeature(StreamFeatureset fs) {
        return (SASLMechanismsFeature)fs.getFirstElement(SASLMechanismsFeature.class);
    }
    /**
     * <p>
     * Begins the SASL authentication process. This method obtains the
     * appropriate <tt>SASLMechanism</tt> to use, registers a
     * <tt>PacketListener</tt> for SASL packets, and send the initial
     * <tt>SASLAuthPacket</tt>.</p>
     *
     * @throws NoSupportedMechanismsException If a suitable
     * <tt>SASLMechanism</tt> could not be obtained.
     * @throws IllegalStateException If the SASL <tt>PacketListener</tt> could
     * not be registered.
     * @throws StreamException If there is a stream-level error
     * @since   JSO-0.12
     */
    protected void startConsumption() throws NoSupportedMechanismsException, IllegalStateException, StreamException {
        FeatureContext          ctx = getFeatureContext();
        PacketRouter            router = ctx.getRouter();
        PacketDispatcher        dispatch = ctx.getDispatcher();
        SASLMechanismsFeature   feat = (SASLMechanismsFeature)ctx.getFeature();
        SASLClientInfo          sci = getClientInfo();
        SASLMechanism           mech;
        SASLPacket              packet;

        sci.setMechanismNames(feat.getMechanismNames());
        mech = SASLMechanismManager.getInstance().createClientMechanism(sci);
        if (mech == null)
            throw new NoSupportedMechanismsException("Could not find a suitable SASLMEchanism", sci.getMechanismNames());

        try {
            dispatch.addPacketListener(PacketEvent.RECEIVED, new SASLWatcher(router.getDataFactory()));
        } catch (SAXPathException spe) {
            throw new IllegalStateException("Could not register SASL packet listener");
        }
        
        setMechanism(mech);
        router.send(mech.evaluate(null));
    }
    public void reset() {
        super.reset();
        
        setMechanism(null);
    }
}