/*
 * SASLPacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Primary interface for representing the packets for SASL interaction.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface SASLPacket extends Packet {
    //Inner-classes
    /**
     * <p>
     * Class to enumerate the possible actions.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static class Action extends Enumerator {
        //Constructors
        private Action(NSI name) {
            super(name.toString());
        }
    }
    
    //"Constants"
    /** The namespace URI for all <tt>SASLPacket</tt> instances
     * (urn:ietf:params:xml:ns:xmpp-sasl).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE       = "urn:ietf:params:xml:ns:xmpp-sasl";
    
    /** The qualified name for &lt;abort/&gt; packets
     * ({urn:ietf:params:xml:ns:xmpp-sasl}auth).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_AUTH       = new NSI("auth", NAMESPACE);
    /** The qualified name for &lt;challenge/&gt; packets
     * ({urn:ietf:params:xml:ns:xmpp-sasl}challenge).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_CHALLENGE  = new NSI("challenge", NAMESPACE);
    /** The qualified name for &lt;response/&gt; packets
     * ({urn:ietf:params:xml:ns:xmpp-sasl}response).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_RESPONSE   = new NSI("response", NAMESPACE);
    /** The qualified name for &lt;abort/&gt; packets
     * ({urn:ietf:params:xml:ns:xmpp-sasl}failure).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_ABORT      = new NSI("abort", NAMESPACE);
    /** The qualified name for &lt;failure/&gt; packets
     * ({urn:ietf:params:xml:ns:xmpp-sasl}failure).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_FAILURE    = new NSI("failure", NAMESPACE);
    /** The qualified name for &lt;success/&gt; packets
     * ({urn:ietf:params:xml:ns:xmpp-sasl}success).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_SUCCESS    = new NSI("success", NAMESPACE);

    /** The action for starting authentication.
     *
     * @since   JSO-0.12
     */
    public static final Action  AUTHENTICATE    = new Action(NAME_AUTH);
    /** The action for aborting authentication.
     *
     * @since   JSO-0.12
     */
    public static final Action  ABORT           = new Action(NAME_ABORT);
    /** The action for an authentication challenge.
     *
     * @since   JSO-0.12
     */
    public static final Action  CHALLENGE       = new Action(NAME_CHALLENGE);
    /** The action for a authentication response.
     *
     * @since   JSO-0.12
     */
    public static final Action  RESPONSE        = new Action(NAME_RESPONSE);
    /** The action for failed authentication.
     *
     * @since   JSO-0.12
     */
    public static final Action  FAILURE         = new Action(NAME_FAILURE);
    /** The action for succeeded authentication.
     *
     * @since   JSO-0.12
     */
    public static final Action  SUCCESS         = new Action(NAME_SUCCESS);
    
    //Methods
    /**
     * <p>
     * Retrieves the action state this <tt>SASLPacket</tt> represents.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The action state.
     * @since   JSO-0.12
     */
    public Action getAction();
    
    /**
     * <p>
     * Retrieves the binary data content for this <tt>SASLPacket</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The binary data content.
     * @since   JSO-0.12
     */
    public byte[] getData();
    /**
     * <p>
     * Sets the binary data content for this <tt>SASLPacket</tt>.</p>
     *
     * <p>
     * The value of <tt>data</tt> may be <tt>null</tt>, in which case it is
     * considered to be an array of 0 length.</p>
     *
     * @param data The new binary data content.
     * @since   JSO-0.12
     */
    public void setData(byte[] data);
}