/*
 * SASLStreamSource.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

import org.jabberstudio.jso.io.StreamSource;
import org.jabberstudio.jso.util.StreamSourceProxy;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Class for wrapping another <tt>StreamSource</tt> to provide SASL security
 * layer wrapping.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class SASLStreamSource extends StreamSourceProxy {
    //Fields
    private SASLMechanism       _Mech;
    
    private boolean             _Secured;
    private int                 _MaxRcvd;
    private int                 _MaxSend;
    private ByteBuffer          _Buffer;
    
    //Constructors
    public SASLStreamSource(StreamSource base) throws IllegalArgumentException {
        super(base);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>SASLMechanism</tt> for this <tt>SASLStreamSource</tt>.
     * The <tt>SASLMechanism</tt> is used to wrap/unwrap the proxied
     * <tt>StreamSource</tt>.</p>
     *
     * <p>
     * The value returned by this method may return <tt>null</tt>.</p>
     *
     * @return The <tt>SASLMechanism</tt> in use.
     * @since   JSO-0.12
     */
    public SASLMechanism getMechanism() {
        return _Mech;
    }
    /**
     * <p>
     * Sets the <tt>SASLMechanism</tt> for this <tt>SASLStreamSource</tt>.</p>
     *
     * <p>
     * The value of <tt>mech</tt> may be <tt>null</tt>.</p>
     *
     * @param mech The new <tt>SASLMechanism</tt> to use.
     * @since   JSO-0.12
     */
    public synchronized void setMechanism(SASLMechanism mech) {
        String      qop;
        int         maxbuf;
        
        _Mech = mech;
        if (_Mech != null) {
            qop = (String)_Mech.getProperties().get(SASLMechanismManager.PROPERTY_QOP);
            _Secured = (Utilities.equateStrings(qop, "auth-int") || Utilities.equateStrings(qop, "auth-conf"));
            
            if (_Secured) {
                try {
                    _MaxRcvd = Integer.parseInt((String)_Mech.getProperties().get(SASLMechanismManager.PROPERTY_MAX_BUFFER));
                } catch (NumberFormatException nfe) {
                    
                }
                
                try {
                    _MaxSend = _MaxRcvd / 2;
                } catch (NumberFormatException nfe) {
                    _MaxSend = _MaxRcvd / 2;
                }
            }
        } else {
            _Secured = false;
            _MaxRcvd = 0;
            _MaxSend = 0;
        }
    }
    
    /**
     * <p>
     * Determines if this <tt>SASLStreamSource</tt> has an active security
     * layer.</p>
     *
     * <p>
     * This method always returns <tt>false</tt> if {@link #getMechanism}
     * returns <tt>null</tt>.</p>
     *
     * @return <tt>true</tt> if the security layer is active.
     * @since   JSO-0.12
     */
    public boolean isSecured() {
        return _Secured;
    }
    private int getMaxReceive() {
        return _MaxRcvd;
    }
    private int getMaxSend() {
        return _MaxSend;
    }
    
    private ByteBuffer getBuffer() {
        return _Buffer;
    }
    private void setBuffer(ByteBuffer buffer) {
        _Buffer = buffer;
    }
    
    public synchronized int read(byte[] buffer, int offset, int length) throws IOException {
        int             amt = length;
        
        if (isSecured()) {
            amt = readBuffer(buffer, offset, length);
        } else {
            amt = super.read(buffer, offset, length);
        }
        
        return amt;
    }
    public synchronized int write(byte[] buffer, int offset, int length) throws IOException {
        int             amt = length;
        
        if (isSecured()) {
            amt = writeBuffer(buffer, offset, length);
        } else {
            amt = super.write(buffer, offset, length);
        }
        
        return amt;
    }
    
    private final int readBuffer(byte[] data, int offset, int length) throws IOException {
        SASLMechanism       mech = getMechanism();
        ByteBuffer          buffer = getBuffer();
        byte[]              header, input;
        int                 size, amt = 0;
        
        if ((buffer == null) || !buffer.hasRemaining()) {
            //Determine buffer size
            header = new byte[4];
            super.read(header, 0, header.length);
            size = (header[0] << 24) | ((header[1] & 0xff) << 16) | ((header[2] & 0xff) << 8) | (header[3] & 0xff);

            //Read buffer data
            input = new byte[size];
            amt = super.read(input, 0, input.length);
            input = mech.unwrap(input);
            setBuffer(buffer = ByteBuffer.allocate(input.length));
        }
        
        amt = ((buffer.remaining() < length) ? buffer.remaining() : length);
        buffer.get(data, offset, amt);
        
        return amt;
    }
    private final int writeBuffer(byte[] data, int offset, int length) throws IOException {
        SASLMechanism   mech = getMechanism();
        byte[]          output, body;
        int             amt, size, max;
        
        for (size = length, amt = 0, max = getMaxSend(); size > 0; size -= max) {
            body = new byte[(size > max) ? max : size];
            System.arraycopy(data, offset + amt, body, 0, body.length);
            
            amt += body.length;
            body = mech.wrap(body);
            output = new byte[body.length + 4];
            output[0] = (byte)(body.length >>> 24);
            output[1] = (byte)(body.length >>> 16);
            output[2] = (byte)(body.length >>>  8);
            output[3] = (byte)body.length;
            System.arraycopy(body, 0, output, 4, body.length);
            
            super.write(output, 0, output.length);
        }
        
        return amt;
    }
}