/*
 * AuthorizationIDCallback.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl.callback;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Underlying security services instantiate and pass an
 * <tt>AuthorizationIDCallback</tt> to the <tt>handle</tt> method of a
 * <tt>CallbackHandler</tt> to retrieve authorization information.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class AuthorizationIDCallback implements javax.security.auth.callback.Callback {
    //Fields
    private String  _Prompt;
    
    private JID     _DefAuthzid;
    private JID     _Authzid;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>AuthorizationIDCallback</tt> with the given
     * prompt.</p>
     *
     * <p>
     * The value of <tt>prompt</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param prompt The prompt used to request the name.
     * @throws IlegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public AuthorizationIDCallback(String prompt) throws IllegalArgumentException {
        this(prompt, null);
    }
    /**
     * <p>
     * Creates a new <tt>AuthorizationIDCallback</tt> with the given
     * prompt and default authorization ID.</p>
     *
     * <p>
     * The value of <tt>prompt</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>authz</tt> may be <tt>null</tt>.</p>
     *
     * @param prompt The prompt used to request the name.
     * @param authz The default authorization ID to display.
     * @throws IlegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public AuthorizationIDCallback(String prompt, JID authz) throws IllegalArgumentException {
        if (!Utilities.isValidString(prompt))
            throw new IllegalArgumentException("prompt cannot be null or \"\"");
        
        _Prompt = prompt;
        _DefAuthzid = _Authzid = authz;
    }

    //Methods
    /**
     * <p>
     * Retrieves the prompt.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * @return The prompt.
     * @since   JSO-0.12
     */
    public String getPrompt() {
        return _Prompt;
    }
    /**
     * <p>
     * Retrieves the default authorization ID.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The default authorization ID.
     * @since   JSO-0.12
     */
    public JID getDefaultAuthorizationID() {
        return _DefAuthzid;
    }
    
    /**
     * <p>
     * Retrieves the authorization ID.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The authorization ID.
     * @since   JSO-0.12
     */
    public JID getAuthorizationID() {
        return _Authzid;
    }
    /**
     * <p>
     * Sets the authorization ID.</p>
     *
     * <p>
     * The value of <tt>authz</tt> may be <tt>null</tt>.</p>
     *
     * @param authz The authorization ID.
     * @since   JSO-0.12
     */
    public void setAuthorizationID(JID authz) {
        _Authzid = authz;
    }
}