/*
 * RealmChoiceCallback.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl.callback;


import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Underlying security services instantiate and pass a
 * <tt>RealmChoiceCallback</tt> to the <tt>handle</tt> method of a
 * <tt>CallbackHandler</tt> to retrieve realm information.</p>
 *
 * <p>
 * Note that <tt>RealmChoiceCallback</tt> only allows a single realm to be
 * selected.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class RealmChoiceCallback extends javax.security.auth.callback.ChoiceCallback {
    /**
     * <p>
     * Creates a new <tt>RealmChoiceCallback</tt> with the given prompt and list
     * of choices, using the first choice as the default.</p>
     *
     * <p>
     * The value of <tt>prompt</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>choices</tt> cannot be <tt>null</tt> or of zero length,
     * or an <tt>IllegalArgumentException</tt> is thrown. If any element of
     * <tt>choices</tt> is <tt>null</tt> or "", an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param prompt The prompt used to request the realm.
     * @param choices The available realm choices.
     * @throws IllegalArgumentException If any paramters are invalid.
     * @since   JSO-0.12
     */
    public RealmChoiceCallback(String prompt, String[] choices) {
        super(prompt, choices, 0, false);
    }
    /**
     * <p>
     * Creates a new <tt>RealmChoiceCallback</tt> with the given prompt, list
     * of choices, and default choice.</p>
     *
     * <p>
     * The value of <tt>prompt</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>choices</tt> cannot be <tt>null</tt> or of zero length,
     * or an <tt>IllegalArgumentException</tt> is thrown. If any element of
     * <tt>choices</tt> is <tt>null</tt> or "", an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>def</tt> must be within the range of valid indices for
     * <tt>choices</tt>, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param prompt The prompt used to request the realm.
     * @param choices The available realm choices.
     * @param def The index of the default realm.
     * @throws IllegalArgumentException If any paramters are invalid.
     * @since   JSO-0.12
     */
    public RealmChoiceCallback(String prompt, String[] choices, int def) {
        super(prompt, choices, def, false);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the default realm. This method is a convenience to calling
     * <tt>getChoices()</tt> and <tt>getDefaultChoice</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * @return The default realm.
     * @since   JSO-0.12
     */
    public String getDefaultRealm() {
        return getChoices()[getDefaultChoice()];
    }

    /**
     * <p>
     * Retrieves the selected realm. This method is a convenience to calling
     * <tt>getSelectedIndexes</tt> and <tt>getChoices</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a valid realm
     * has not been selected.</p>
     *
     * @return The selected realm.
     * @since   JSO-0.12
     */
    public String getRealm() {
        int[]       selects = getSelectedIndexes();
        String[]    choices = getChoices();
        int         selected = -1;
        String      realm = null;
        
        if ((selects != null) && (selects.length > 0))
            selected = selects[0];
        
        if ((selected >= 0) && (selected < choices.length))
            realm = choices[selected];
        
        return realm;
    }
    /**
     * <p>
     * Sets the selected realm.</p>
     *
     * <p>
     * The value of <tt>realm</tt> must be equal to one of the elements within
     * <tt>getChoices()</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param realm The new selected realm.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void setRealm(String realm) throws IllegalArgumentException {
        String[]    choices = getChoices();
        int         idx, selected = -1;
        
        for (idx = 0; (selected != -1) && (idx < choices.length); idx++) {
            if (Utilities.equateStrings(realm, choices[idx]))
                selected = idx;
        }
        
        if (selected == -1)
            throw new IllegalArgumentException("realm not found in choices");
        
        setSelectedIndex(selected);
    }
}