/*
 * PacketWatcher.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.event.PacketListener;

/**
 * <p>
 * Convenience class to watch for a <tt>Packet</tt> with a specific name and ID.
 * The watching is implemented as an inner-class, using instances of
 * <tt>PacketWatcher</tt> to set up and shut down the listener.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 * @deprecated As of JSO-0.12, this class is replaced with {@link
 * PacketMonitor}.
 */
public class PacketWatcher {
    protected class Watcher implements PacketListener {
        public void packetTransferred(PacketEvent evt) {
            Packet  data = evt.getData(), template = getTemplatePacket();
            NSI     name1 = template.getNSI(), name2 = data.getNSI();
            String  id1 = template.getID(), id2 = data.getID();

            if ((NSI.NS_LOOSE_COMPARATOR.compare(name1, name2) == 0) && Utilities.equateStrings(id1, id2))
                setFoundPacket(data);
        }
    }
    
    //Fields
    private Packet      _Lost;
    private Packet      _Found;
    
    //Constructors
    /**
     * <p>
     * Creates a new <code>PacketWatcher</code> with the given "template"
     * <code>Packet</code>.</p>
     *
     * <p>
     * The value of <code>template</code> cannot be <code>null</code>, or an
     * <Code>IllegalArgumentException</code>.</p>
     *
     * @param template The <code>Packet</code> info to watch for.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public PacketWatcher(Packet template) throws IllegalArgumentException {
        setTemplatePacket(template);
        setFoundPacket(null);
    }
    
    //Methods
    /**
     * <p>
     * Retreives the <code>Packet</code> this <code>PacketWatcher</code> is
     * waiting for.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return The <code>Packet</code> (information) being watched for.
     * @since   JSO-0.2
     */
    public Packet getTemplatePacket() {
        return _Lost;
    }
    protected void setTemplatePacket(Packet template) throws IllegalArgumentException {
        if (template == null)
            throw new IllegalArgumentException("Packet cannot be null");
        _Lost = template;
    }
    
    /**
     * <p>
     * Retrieves the <code>Packet</code> this <code>PacketWatcher</code>
     * received and matched.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if no
     * matching <code>Packet</code> has been found.</p>
     *
     * @return The <code>Packet</code> matching the "template".
     * @since   JSO-0.2
     */
    public Packet getFoundPacket() {
        return _Found;
    }
    protected void setFoundPacket(Packet found) {
        _Found = found;
    }
    
    /**
     * <p>
     * Watches the given <code>Stream</code> for an inbound <code>Packet</code>
     * matching its "template".  This method continuously calls {@link
     * Stream#process} until a matching <code>Packet</code> is received, or
     * <code>s</code> is no longer open.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.10, this method is removed in favor of
     * {@link #sendAndWatch(Stream, long)}.
     */
    public Packet watch(Stream s) throws IllegalArgumentException, StreamException {
        return watch(s, -1);
    }
    /**
     * <p>
     * Watches the given <code>Stream</code> for an inbound <tt>Packet</tt>
     * matching its "template" until the given time elapses. This method
     * continuously calls {@link Stream#process} until a matching
     * <tt>Packet</tt> is received, <tt>s</tt> is no longer open, or the
     * specified amount of time elapses.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * If the value of <tt>timeout</tt> is non-negative, it specifies the amount
     * of time (in milliseconds) to keep watching. Any negative value means to
     * watch indefinitely.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @param timeout The amount of time to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.3
     * @deprecated As of JSO-0.10, this method is removed in favor of
     * {@link #sendAndWatch(Stream, long)}.
     */
    public Packet watch(Stream s, long timeout) throws IllegalArgumentException, StreamException {
        PacketListener      l;
        long                start, end;
        boolean             found, elapsed;

        //Validate router and dispatcher
        if (s == null)
            throw new IllegalArgumentException("Stream cannot be null");
        
        synchronized (this) {
            l = new Watcher();

            try {
                //setup listener
                s.addPacketListener(PacketEvent.RECEIVED, l);
                //"start" time
                start = System.currentTimeMillis();
                //now we watch
                do {
                    try { wait(1); } catch (InterruptedException ie) {}
                    if (timeout >= 0) {
                        end = System.currentTimeMillis();
                        elapsed = (end - start) >= timeout;
                    } else {
                        elapsed = false;
                    }
                    s.process();
                    found = (getFoundPacket() != null);
                } while (s.getCurrentStatus().isOpened() && !found && !elapsed);
            }finally {
                //No matter what, remove listener
                s.removePacketListener(PacketEvent.RECEIVED, l);
            }
        }
        
        return getFoundPacket();
    }

    /**
     * <p>
     * Sends the "template" <tt>Packet</tt>, then watches the given
     * <tt>Stream</tt> for a <tt>Packet</tt> matching {@link #getTemplatePacket}
     * until the given time elapses. This method continuously calls {@link
     * Stream#process} until a matching <tt>Packet</tt> is received or
     * <tt>s</tt> is no longer open.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.5
     */
    public Packet sendAndWatch(Stream s) throws IllegalArgumentException, StreamException {
        return sendAndWatch(s, s, -1);
    }
    /**
     * <p>
     * Sends the "template" <tt>Packet</tt>, then watches the given
     * <tt>PacketRouter</tt> for a <tt>Packet</tt> matching the "template"
     * until the given time elapses. This method continuously calls {@link
     * PacketRouter#process} until a matching <tt>Packet</tt> is received or
     * <tt>r</tt> is no longer open.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a
     * matching <tt>Packet</tt> is not received before <tt>r</tt>
     * is closed.</p>
     *
     * @param r The <tt>PacketRouter</tt> to process against.
     * @param d The <tt>PacketDispatcher</tt> to watch against.
     * @return The matched <tt>Packet</tt> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.5
     */
    public Packet sendAndWatch(PacketRouter r, PacketDispatcher d) throws IllegalArgumentException, StreamException {
        return sendAndWatch(r, d, -1);
    }
    /**
     * <p>
     * Sends the "template" <tt>Packet</tt>, then watches the given
     * <tt>Stream</tt> for a <tt>Packet</tt> matching {@link #getTemplatePacket}
     * until the given time elapses. This method continuously calls {@link
     * Stream#process} until a matching <tt>Packet</tt> is received, <tt>s</tt>
     * is no longer open, or the specified amount of time elapses.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * If the value of <tt>timeout</tt> is non-negative, it specifies the amount
     * of time (in milliseconds) to keep watching. Any negative value means to
     * watch indefinitely.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @param timeout The amount of time to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.5
     */
    public Packet sendAndWatch(Stream s, long timeout) throws IllegalArgumentException, StreamException {
        return sendAndWatch(s, s, timeout);
    }
    /**
     * <p>
     * Sends the "template" <tt>Packet</tt>, then watches the given
     * <tt>PacketRouter</tt> for a <tt>Packet</tt> matching "template"
     * until the given time elapses. This method continuously calls {@link
     * PacketRouter#process} until a matching <tt>Packet</tt> is received,
     * <tt>r</tt> is no longer open, or the specified amount of time
     * elapses.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the value of <tt>timeout</tt> is non-negative, it specifies the amount
     * of time (in milliseconds) to keep watching. Any negative value means to
     * watch indefinitely.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a
     * matching <tt>Packet</tt> is not received before <tt>r</tt>
     * is closed.</p>
     *
     * @param r The <tt>PacketRouter</tt> to process against.
     * @param d The <tt>PacketDispatcher</tt> to watch against.
     * @param timeout The amount of time to watch.
     * @return The matched <tt>Packet</tt> found.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If any errors occur in the <tt>PacketRouter</tt>.
     * @since   JSO-0.10
     */
    public Packet sendAndWatch(PacketRouter r, PacketDispatcher d, long timeout) throws IllegalArgumentException, StreamException {
        PacketListener      l;
        long                start, end;
        boolean             found, elapsed;

        //Validate router and dispatcher
        if (r == null)
            throw new IllegalArgumentException("PacketRouter cannot be null");
        if (d == null)
            throw new IllegalArgumentException("PacketDispatcher cannot be null");
        
        synchronized (this) {
            l = new Watcher();

            try {
                //setup listener
                d.addPacketListener(PacketEvent.RECEIVED, l);
                //send template
                r.send(getTemplatePacket());
                //"start" time
                start = System.currentTimeMillis();
                //now we watch
                do {
                    try { wait(1); } catch (InterruptedException ie) {}
                    if (timeout >= 0) {
                        end = System.currentTimeMillis();
                        elapsed = (end - start) >= timeout;
                    } else {
                        elapsed = false;
                    }
                    r.process();
                    found = (getFoundPacket() != null);
                } while (r.getCurrentStatus().isOpened() && !found && !elapsed);
            }finally {
                //No matter what, remove listener
                d.removePacketListener(PacketEvent.RECEIVED, l);
            }
        }
        
        return getFoundPacket();
    }
    
    /**
     * <p>
     * Watches the given <code>Stream</code> for an inbound <code>Packet</code>
     * matching the given "template".  This method continuously calls {@link
     * Stream#process} until a matching <code>Packet</code> is received, or
     * <code>s</code> is no longer open.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.3
     * @deprecated As of JSO-0.10, this method is replaced with {@link
     * #sendAndWatch(Stream, Packet)}.
     */
    public static Packet watch(Stream s, Packet p) throws IllegalArgumentException, StreamException {
        return watch(s, p, -1);
    }
    /**
     * <p>
     * Watches the given <code>Stream</code> for an inbound <tt>Packet</tt>
     * matching the given "template" until the given time elapses. This
     * method continuously calls {@link Stream#process} until a matching
     * <tt>Packet</tt> is received, <tt>s</tt> is no longer open, or the
     * specified amount of time elapses.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the value of <tt>timeout</tt> is non-negative, it specifies the amount
     * of time (in milliseconds) to keep watching. Any negative value means to
     * watch indefinitely.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @param p The <tt>Packet</tt> template to watch for.
     * @param timeout The amount of time to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.3
     * @deprecated As of JSO-0.10, this method is replaced with {@link
     * #sendAndWatch(Stream, Packet, long)}.
     */
    public static Packet watch(Stream s, Packet p, long timeout) throws IllegalArgumentException, StreamException {
        PacketWatcher   w = new PacketWatcher(p);
        
        return w.watch(s, timeout);
    }
    
    /**
     * <p>
     * Sends the given <tt>Packet</tt>, then watches the given <tt>Stream</tt>
     * for its matching <tt>Packet</tt> until the given time elapses. This
     * method continuously calls {@link Stream#process} until a matching
     * <tt>Packet</tt> is received or <tt>s</tt> is no longer open.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @param p The <tt>Packet</tt> template to send then watch for.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.5
     */
    public static Packet sendAndWatch(Stream s, Packet p) throws IllegalArgumentException, StreamException {
        return sendAndWatch(s, s, p, -1);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt>, then watches the given
     * <tt>PacketRouter</tt> for its matching <tt>Packet</tt> until the given
     * time elapses. This method continuously calls {@link PacketRouter#process}
     * until a matching <tt>Packet</tt> is received or <tt>r</tt> is no longer
     * open.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <tt>Packet</tt> is not received before <code>r</code>
     * is closed.</p>
     *
     * @param r The <tt>PacketRouter</tt> to process against.
     * @param d The <tt>PacketDispatcher</tt> to watch against.
     * @param p The <tt>Packet</tt> template to send then watch for.
     * @return The matched <tt>Packet</tt> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <tt>PacketRouter</tt>.
     * @since   JSO-0.10
     */
    public static Packet sendAndWatch(PacketRouter r, PacketDispatcher d, Packet p) throws IllegalArgumentException, StreamException {
        return sendAndWatch(r, d, p, -1);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt>, then watches the given <tt>Stream</tt>
     * for its matching <tt>Packet</tt> until the given time elapses. This
     * method continuously calls {@link Stream#process} until a matching
     * <tt>Packet</tt> is received, <tt>s</tt> is no longer open, or the
     * specified amount of time elapses.</p>
     *
     * <p>
     * The value of <code>s</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the value of <tt>timeout</tt> is non-negative, it specifies the amount
     * of time (in milliseconds) to keep watching. Any negative value means to
     * watch indefinitely.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>s</code>
     * is closed.</p>
     *
     * @param s The <code>Stream</code> to watch.
     * @param p The <tt>Packet</tt> template to send then watch for.
     * @param timeout The amount of time to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any errors occur in the <code>Stream</code>.
     * @since   JSO-0.8.5
     */
    public static Packet sendAndWatch(Stream s, Packet p, long timeout) throws IllegalArgumentException, StreamException {
        return sendAndWatch(s, s, p, timeout);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt>, then watches the given
     * <tt>PacketRouter</tt> for its matching <tt>Packet</tt> until the given
     * time elapses. This method continuously calls {@link PacketRouter#process}
     * until a matching <tt>Packet</tt> is received, <tt>r</tt> is no longer
     * open, or the specified amount of time elapses.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the value of <tt>timeout</tt> is non-negative, it specifies the amount
     * of time (in milliseconds) to keep watching. Any negative value means to
     * watch indefinitely.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if a
     * matching <code>Packet</code> is not received before <code>r</code>
     * is closed or the timeout is reached.</p>
     *
     * @param r The <tt>PacketRouter</tt> to process against.
     * @param d The <tt>PacketDispatcher</tt> to watch against.
     * @param p The <tt>Packet</tt> template to send then watch for.
     * @param timeout The amount of time to watch.
     * @return The matched <Code>Packet</code> found.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If any errors occur in the <tt>PacketRouter</tt>.
     * @since   JSO-0.10
     */
    public static Packet sendAndWatch(PacketRouter r, PacketDispatcher d, Packet p, long timeout) throws IllegalArgumentException, StreamException {
        PacketWatcher   w = new PacketWatcher(p);

        return w.sendAndWatch(r, d, timeout);
    }
}