/*
 * QueuedPacketRouter.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.util.Iterator;
import java.util.List;

import org.jabberstudio.jso.JSOImplementation;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamError;
import org.jabberstudio.jso.StreamException;

/**
 * <p>
 * Implements the <tt>PacketRouter</tt> interface using a queue.
 * <tt>Packet</tt> objects passed to {@link #send} are placed into the queue,
 * and are removed, one at a time, from the queue in {@link #process}. On its
 * own, this implementation is not usable.</p>
 *
 * <p>
 * Subclasses of this class should override the following methods to perform
 * the desired behavior:</p>
 *
 * <ul>
 * <li><b>{@link #performSendPre}:</b> This is called by {@link #send} before
 * a <tt>Packet</tt> is enqued.</li>
 * <li><b>{@link #performSendPost}:</b> This is called by {@link #send} after
 * a <tt>Packet</tt> is enqued.</li>
 * <li><b>{@link #performProcessPre}:</b> This is called by {@link #process}
 * before a <tt>Packet</tt> is dequed.</li>
 * <li><B>{@link #performProcessPost}:</b> This is called by {@link #process}
 * after a <tt>Packet</tt> is dequed.</li>
 * </ul>
 *
 * <p>
 * For the purposes of this implementation, "inbound" refers to the {@link
 * #process} flow, and "outbound" refers to the {@link #send} flow.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.10
 */
public class QueuedPacketRouter implements PacketRouter {
    //Fields
    private StreamDataFactory   _SDF;
    private StreamContext       _Inbound, _Outbound;
    private Stream.Status       _Status;
    
    private List                _Queue;
    private PacketListenerRelay _Relay;
    
    //Constructors
    /**
     * <p>
     * Creates a <tt>QueuedPacketRouter</tt> with the given
     * <tt>StreamContext</tt> used for both directions.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for inbound and outbound
     * operations.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.10
     * @deprecated This constructor is deprecated because it wrongly shares
     * <tt>StreamContext</tt> instances.
     */
    public QueuedPacketRouter(StreamContext ctx) throws IllegalArgumentException {
        this(ctx, ctx);
    }

    /**
     * <p>
     * Creates a new <tt>QueuedPacketRouter</tt> with the
     * <tt>StreamDataFactory</tt> and inbound and outbound
     * <tt>StreamContext</tt> objects created from the current
     * <tt>JSOImplementation</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public QueuedPacketRouter() {
        this(null, null, null);
    }
    /**
     * <p>
     * Creates a new <tt>QueuedPacketRouter</tt> with the given
     * <tt>StreamDataFactory</tt>. The inbound and outbound
     * <tt>StreamContext</tt> objects are created from the current
     * <tt>JSOImplementation</tt>.</p>
     *
     * <p>
     * If the value of <tt>sdf</tt> is <tt>null</tt>, a new
     * <tt>StreamDataFactory</tt> is obtained via {@link
     * JSOImplementation#createDataFactory}.</p>
     *
     * @param sdf The <tt>StreamDataFactory</tt>.
     * @since   JSO-0.12
     */
    public QueuedPacketRouter(StreamDataFactory sdf) {
        this(sdf, null, null);
    }
    /**
     * <p>
     * Creates a new <tt>QueuedPacketRouter</tt> with the given inbound
     * and outbound <tt>StreamContext</tt> objects. The
     * <tt>StreamDataFactory</tt> is created from the current
     * <tt>JSOImplementation</tt>.</p>
     *
     * <p>
     * If the value of <tt>in</tt> is <tt>null</tt>, a new
     * <tt>StreamContext</tt> via {@link
     * JSOImplementation#createStreamContext()}.</p>
     *
     * <p>
     * If the value of <tt>out</tt> is <tt>null</tt>, a new
     * <tt>StreamContext</tt> via {@link
     * JSOImplementation#createStreamContext()}.</p>
     *
     * <p>
     * If <tt>in</tt> and <tt>out</tt> refer to the same <tt>StreamContext</tt>,
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param in The <tt>StreamContext</tt> for inbound operations.
     * @param out The <tt>StreamContext</tt> for outbound operations.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.10
     */
    public QueuedPacketRouter(StreamContext in, StreamContext out) throws IllegalArgumentException {
        this(null, in, out);
    }
    /**
     * <p>
     * Creates a new <tt>QueuedPacketRouter</tt> with the given
     * <tt>StreamDataFactory</tt> and inbound and outbound
     * <tt>StreamContext</tt> instances.</p>
     *
     * <p>
     * If the value of <tt>sdf</tt> is <tt>null</tt>, a new
     * <tt>StreamDataFactory</tt> is obtained via {@link
     * JSOImplementation#createDataFactory}.</p>
     *
     * <p>
     * If the value of <tt>in</tt> is <tt>null</tt>, a new
     * <tt>StreamContext</tt> via {@link
     * JSOImplementation#createStreamContext()}.</p>
     *
     * <p>
     * If the value of <tt>out</tt> is <tt>null</tt>, a new
     * <tt>StreamContext</tt> via {@link
     * JSOImplementation#createStreamContext()}.</p>
     *
     * <p>
     * If <tt>in</tt> and <tt>out</tt> refer to the same <tt>StreamContext</tt>,
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param sdf The <tt>StreamDataFactory</tt>.
     * @param in The inbound <tt>StreamContext</tt>.
     * @param out the outbound <tt>StreamContext</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public QueuedPacketRouter(StreamDataFactory sdf, StreamContext in, StreamContext out) {
        JSOImplementation   jso = JSOImplementation.getInstance();

        //Validate input
        if (sdf == null)
            sdf = jso.createDataFactory();
        if (in == null)
            in = jso.createStreamContext();
        if (out == null)
            out = jso.createStreamContext();
        
        if (in == out)
            throw new IllegalArgumentException("inbound and outbound StreamContext cannot be the same");
        
        setInboundContext(in);
        setOutboundContext(out);
        setCurrentStatus(Stream.DISCONNECTED);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>StreamDataFactory</tt> for this <tt>PacketRouter</tt>.
     * This instance should be used to create all <tt>Packet</tt> objects sent
     * to this <tt>PacketRouter</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamDataFactory</tt>.
     * @since   JSO-0.12
     */
    public StreamDataFactory getDataFactory() {
        return _SDF;
    }
    private void setDataFactory(StreamDataFactory sdf) {
        _SDF = initDataFactory(sdf);
    }
    /**
     * <p>
     * Initializes the <tt>StreamDataFactory</tt> for this
     * <tt>PacketRouter</tt>. Subclasses may use this method to properly
     * configure the <tt>StreamDataFactory</tt>, or "wrap" it with a custom
     * implementation.</p>
     *
     * <p> This implementation simply returns the value of <tt>sdf</tt>.</p>
     *
     * <p>
     * The value of <tt>sdt</tt> is never <tt>null</tt>.</p>
     *
     * <p>
     * The value returned by this method cannot be <tt>null</tt>.</p>
     *
     * @param sdf The <tt>StreamDataFactory</tt> to initialize.
     * @return The initialized <tt>StreamDataFactory</tt>.
     * @since   JSO-0.12
     */
    protected StreamDataFactory initDataFactory(StreamDataFactory sdf) {
        return sdf;
    }
    
    /**
     * <p>
     * Retrieves the <tt>StreamContext</tt> for inbound operations. This
     * <tt>StreamContext</tt> should be referred to within the {@link #process}
     * chain of methods.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamContext</tt> for inbound operations.
     */
    public StreamContext getInboundContext() {
        return _Inbound;
    }
    private void setInboundContext(StreamContext ctx) {
        _Inbound = initInboundContext(ctx);
    }
    /**
     * <p>
     * Initializes the <tt>StreamContext</tt> for inbound opeations. This method
     * is called by the constructor(s). Implementations should override this
     * method to provide their own initialization.</p>
     *
     * <p>
     * The default implementation creates a <tt>ContextDecorator</tt> around
     * <tt>ctx</tt>, with the following methods implemented/overridden:</p>
     *
     * <ul>
     * <li>{@link ContextDecorator#getDecoratedContext} returns
     * <tt>ctx</tt>.</li>
     * <li>{@link StreamContext#isInbound} returns <tt>true</tt>.</li>
     * <li>{@link StreamContext#getDataFactory} returns {@link
     * #getDataFactory}.</li>
     * <li>{@link StreamContext#getRouter} returns this
     * <tt>QueuedPacketRouter</tt>.</li>
     * <li>{@link StreamContext#getCurrentStatus} returns {@link
     * #getCurrentStatus}.</li>
     * </ul>
     *
     * <p>
     * The value of <tt>ctx</tt> is never <tt>null</tt>.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for inbound operations.
     * @return The initialized <tt>StreamCotnext</tt>.
     * @since   JSO-0.10
     */
    protected StreamContext initInboundContext(final StreamContext ctx) {
        return new ContextDecorator() {
            public StreamContext getDecoratedContext() {
                return ctx;
            }
            
            public boolean isInbound() {
                return true;
            }
            
            public StreamDataFactory getDataFactory() {
                return QueuedPacketRouter.this.getDataFactory();
            }
            
            public PacketRouter getRouter() {
                return QueuedPacketRouter.this;
            }
            
            public Stream.Status getCurrentStatus() {
                return QueuedPacketRouter.this.getCurrentStatus();
            }
        };
    }

    /**
     * <p>
     * Retrieves the <tt>StreamContext</tt> for outbound operations. This
     * <tt>StreamContext</tt> should be referred to within the {@link #send}
     * chain of methods.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamContext</tt> for inbound operations.
     */
    public StreamContext getOutboundContext() {
        return _Outbound;
    }
    private void setOutboundContext(StreamContext ctx) throws IllegalArgumentException {
        if (ctx == null)
            throw new IllegalArgumentException("Outbound StreamContext cannot be null");
        _Outbound = initOutboundContext(ctx);
    }
    /**
     * <p>
     * Initializes the <tt>StreamContext</tt> for outbound opeations. This
     * method is called by the constructor(s). Implementations should override
     * this method to provide their own initialization.</p>
     *
     * <ul>
     * <li>{@link ContextDecorator#getDecoratedContext} returns
     * <tt>ctx</tt>.</li>
     * <li>{@link StreamContext#isOutbound} returns <tt>true</tt>.</li>
     * <li>{@link StreamContext#getDataFactory} returns {@link
     * #getDataFactory}.</li>
     * <li>{@link StreamContext#getRouter} returns this
     * <tt>QueuedPacketRouter</tt>.</li>
     * <li>{@link StreamContext#getCurrentStatus} returns {@link
     * #getCurrentStatus}.</li>
     * </ul>
     *
     * <p>
     * The value of <tt>ctx</tt> is never <tt>null</tt>.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for outbound operations.
     * @return The initialized <tt>StreamCotnext</tt>.
     * @since   JSO-0.10
     */
    protected StreamContext initOutboundContext(final StreamContext ctx) {
        return new ContextDecorator() {
            public StreamContext getDecoratedContext() {
                return ctx;
            }
            
            public boolean isOutbound() {
                return true;
            }
            
            public StreamDataFactory getDataFactory() {
                return QueuedPacketRouter.this.getDataFactory();
            }
            
            public PacketRouter getRouter() {
                return QueuedPacketRouter.this;
            }
            
            public Stream.Status getCurrentStatus() {
                return QueuedPacketRouter.this.getCurrentStatus();
            }
        };
    }

    /**
     * <p>
     * Retrieves the default namespace for application-level <tt>Packet</tt>s.
     * This namespace is obtained from the inbound <tt>StreamContext</tt> via
     * {@link StreamContext#getNamespaceURI(String)}, using "" as the
     * prefix.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The default namespace.
     * @since   JSO-0.12
     */
    public String getDefaultNamespace() {
        return getInboundContext().getNamespaceURI("");
    }
    
    /**
     * <p>
     * Retrieves the current <tt>Stream.Status</tt> for this
     * <tt>QueuedPacketRouter</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The current <tt>Stream.Status</tt>
     * @since   JSO-0.10
     */
    public Stream.Status getCurrentStatus() {
        return _Status;
    }
    /**
     * <p>
     * Sets the current <tt>Stream.Status</tt> for this
     * <tt>QueuedPacketRouter</tt>.</p>
     *
     * <p>
     * The value of <tt>stat</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param stat The new <tt>Stream.Status</tt>
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.10
     */
    protected void setCurrentStatus(Stream.Status stat) throws IllegalArgumentException {
        if (stat == null)
            throw new IllegalArgumentException("Stream.Status cannot be null");
        _Status = stat;
    }
    
    private List getQueue() {
        if (_Queue == null)
            _Queue = new java.util.LinkedList();
        return _Queue;
    }

    /**
     * <p>
     * Places the given <tt>Packet</tt> on the tail of the queue. This method
     * is called by {@link #send}, after {@link #performSendPre} and before
     * {@link #performSendPost}.</p>
     *
     * <p>
     * The value of <tt>p</tt> should not be <tt>null</tt>. No actions are
     * performed is this is true.</p>
     *
     * @param p The <tt>Packet</tt> to enque.
     */
    protected synchronized void enque(Packet p) {
        if (p != null) {
            getQueue().add(p);
        }
    }
    /**
     * <p>
     * Retrieves the next <tt>Packet</tt> on the queue, without removing it.
     * This method is called to provide {@link #performProcessPre} with a
     * valid <tt>Packet</tt> (if available).</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if there are no
     * more <tt>Packet</tt>s in the queue.</p>
     *
     * @return The next <tt>Packet</tt> to deque.
     * @since   JSO-0.10
     */
    protected synchronized Packet head() {
        Iterator    itr = getQueue().iterator();
        
        return (itr.hasNext()) ? (Packet)itr.next() : null;
    }
    /**
     * <p>
     * Removes a <tt>Packet</tt> from the head of the queue. This method
     * is called by {@link #process}, after {@link #performProcessPre} and
     * before {@link #performProcessPost}.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if there are no
     * more <tt>Packet</tt>s in the queue.</p>
     *
     * @return The <tt>Packet</tt> that was dequed.
     * @since   JSO-0.10
     */
    protected synchronized Packet deque() {
        Packet      p = null;
        Iterator    itr = getQueue().iterator();
        
        if (itr.hasNext()) {
            p = (Packet)itr.next();
            itr.remove();
        }
        
        return p;
    }

    public void send(Packet p) throws StreamException {
        StreamContext   ctx = getOutboundContext();
        
        if (p != null) {
            performSendPre(ctx, p);
            enque(p);
            performSendPost(ctx, p);
        }
    }
    /**
     * <p>
     * Method called before the given <tt>Packet</tt> is placed on the
     * queue. This implementation throws a <tt>StreamException</tt> if {@link
     * StreamContext#getCurrentStatus} for <tt>ctx</tt> is not in an opened
     * state.</p>
     *
     * <p>
     * This method should never be called directly. Subclasses should override
     * this implementation to provide their own pre-"send" logic.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> is never <tt>null</tt>.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for this operation.
     * @param p The <tt>Packet</tt> for this operation.
     * @throws StreamException If any errors occur.
     * @since   JSO-0.10
     */
    protected void performSendPre(StreamContext ctx, Packet p) throws StreamException {
        if (!ctx.getCurrentStatus().isOpened()) {
            StreamError     err = ctx.getDataFactory().createStreamError(StreamError.RESOURCE_CONSTRAINT_CONDITION);
            
            err.setText("Router is not open");
            throw new StreamException(err);
        }
    }
    /**
     * <p>
     * Method called after the given <tt>Packet</tt> is placed on the
     * queue. This implementation does nothing.</p>
     *
     * <p>
     * This method should never be called directly. Subclasses should override
     * this implementation to provide their own post-"send" logic.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> is never <tt>null</tt>.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for this operation.
     * @param p The <tt>Packet</tt> for this operation.
     * @throws StreamException If any errors occur.
     * @since   JSO-0.10
     */
    protected void performSendPost(StreamContext ctx, Packet p) throws StreamException {
    }
    
    public void process() throws StreamException {
        StreamContext   ctx = getInboundContext();
        Packet          p;
        
        performProcessPre(ctx, head());
        p = deque();
        performProcessPost(ctx, p);
    }
    /**
     * <p>
     * Method called before the given <tt>Packet</tt> is removed from the
     * queue. This implementation throws a <tt>StreamException</tt> if {@link
     * StreamContext#getCurrentStatus} for <tt>ctx</tt> is not in an opened
     * state.</p>
     *
     * <p>
     * This method should never be called directly. Subclasses should override
     * this implementation to provide their own pre-"process" logic.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> is never <tt>null</tt>.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for this operation.
     * @param p The <tt>Packet</tt> for this operation.
     * @throws StreamException If any errors occur.
     * @since   JSO-0.10
     */
    protected void performProcessPre(StreamContext ctx, Packet p) throws StreamException {
        if (!ctx.getCurrentStatus().isOpened()) {
            StreamError     err = ctx.getDataFactory().createStreamError(StreamError.RESOURCE_CONSTRAINT_CONDITION);
            
            err.setText("Router is not open");
            throw new StreamException(err);
        }
    }
    /**
     * <p>
     * Method called after the given <tt>Packet</tt> is removed from the
     * queue. This implementation does nothing.</p>
     *
     * <p>
     * This method should never be called directly. Subclasses should override
     * this implementation to provide their own post-"process" logic.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> is never <tt>null</tt>.</p>
     *
     * @param ctx The <tt>StreamContext</tt> for this operation.
     * @param p The <tt>Packet</tt> for this operation.
     * @throws StreamException If any errors occur.
     * @since   JSO-0.10
     */
    protected void performProcessPost(StreamContext ctx, Packet p) throws StreamException {
    }
}