/*
 * StringprepProfile.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

/**
 * <p>
 * Base class for all stringprep profile implementations.</p>
 *
 * <p>
 * Implementation must override {@link #prepare} to apply their specific
 * rules. This usually involves calling the following methods (in order):</p>
 *
 * <ol>
 * <li>{@link #unassigned} to check for unassigned code poitns</li>
 * <li>{@link #mapping} to map the input</li>
 * <li>{@link #normalize} to normalize the input using Form KC</li>
 * <li>{@link #prohibited} to check for prohibited characters</li>
 * <li>{@link #bidirectionality} to check bidirectionality constraints</li>
 * </ol>
 *
 * @see     <a href='http://www.ietf.org/rfc/rfc3454.txt'>Preparation of Internationalized Strings ("stringprep")</a>
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12.3
 */
public abstract class StringprepProfile {
    //"Constants"
    /**
     * Table of unassigned characters. This table is defined in RFC 3454,
     * Appendix A.
     *
     * @since   JSO-0.12.3
     */
    public static final UnassignedTable UNASSIGNED_A_1 = UnassignedTable.create(new char[][] {{'\u0221'}, {'\u0234', '\u024F'}, {'\u02AE', '\u02AF'}, {'\u02EF', '\u02FF'}, {'\u0350', '\u035F'}, {'\u0370', '\u0373'}, {'\u0376', '\u0379'}, {'\u037B', '\u037D'}, {'\u037F', '\u0383'}, {'\u038B'}, {'\u038D'}, {'\u03A2'}, {'\u03CF'}, {'\u03F7', '\u03FF'}, {'\u0487'}, {'\u04CF'}, {'\u04F6', '\u04F7'}, {'\u04FA', '\u04FF'}, {'\u0510', '\u0530'}, {'\u0557', '\u0558'}, {'\u0560'}, {'\u0588'}, {'\u058B', '\u0590'}, {'\u05A2'}, {'\u05BA'}, {'\u05C5', '\u05CF'}, {'\u05EB', '\u05EF'}, {'\u05F5', '\u060B'}, {'\u060D', '\u061A'}, {'\u061C', '\u061E'}, {'\u0620'}, {'\u063B', '\u063F'}, {'\u0656', '\u065F'}, {'\u06EE', '\u06EF'}, {'\u06FF'}, {'\u070E'}, {'\u072D', '\u072F'}, {'\u074B', '\u077F'}, {'\u07B2', '\u0900'}, {'\u2002'}, {'\u0904'}, {'\u093A', '\u093B'}, {'\u094E', '\u094F'}, {'\u0955', '\u0957'}, {'\u0971', '\u0980'}, {'\u0984'}, {'\u098D', '\u098E'}, {'\u0991', '\u0992'}, {'\u09A9'}, {'\u09B1'}, {'\u09B3', '\u09B5'}, {'\u09BA', '\u09BB'}, {'\u09BD'}, {'\u09C5', '\u09C6'}, {'\u09C9', '\u09CA'}, {'\u09CE', '\u09D6'}, {'\u09D8', '\u09DB'}, {'\u09DE'}, {'\u09E4', '\u09E5'}, {'\u09FB', '\u0A01'}, {'\u0A03', '\u0A04'}, {'\u0A0B', '\u0A0E'}, {'\u0A11', '\u0A12'}, {'\u0A29'}, {'\u0A31'}, {'\u0A34'}, {'\u0A37'}, {'\u0A3A', '\u0A3B'}, {'\u0A3D'}, {'\u0A43', '\u0A46'}, {'\u0A49', '\u0A4A'}, {'\u0A4E', '\u0A58'}, {'\u0A5D'}, {'\u0A5F', '\u0A65'}, {'\u0A75', '\u0A80'}, {'\u0A84'}, {'\u0A8C'}, {'\u0A8E'}, {'\u0A92'}, {'\u0AA9'}, {'\u0AB1'}, {'\u0AB4'}, {'\u0ABA', '\u0ABB'}, {'\u0AC6'}, {'\u0ACA'}, {'\u0ACE', '\u0ACF'}, {'\u0AD1', '\u0ADF'}, {'\u0AE1', '\u0AE5'}, {'\u2002'}, {'\u0AF0', '\u0B00'}, {'\u0B04'}, {'\u0B0D', '\u0B0E'}, {'\u0B11', '\u0B12'}, {'\u0B29'}, {'\u0B31'}, {'\u0B34', '\u0B35'}, {'\u0B3A', '\u0B3B'}, {'\u0B44', '\u0B46'}, {'\u0B49', '\u0B4A'}, {'\u0B4E', '\u0B55'}, {'\u0B58', '\u0B5B'}, {'\u0B5E'}, {'\u0B62', '\u0B65'}, {'\u0B71', '\u0B81'}, {'\u0B84'}, {'\u0B8B', '\u0B8D'}, {'\u0B91'}, {'\u0B96', '\u0B98'}, {'\u0B9B'}, {'\u0B9D'}, {'\u0BA0', '\u0BA2'}, {'\u0BA5', '\u0BA7'}, {'\u0BAB', '\u0BAD'}, {'\u0BB6'}, {'\u0BBA', '\u0BBD'}, {'\u0BC3', '\u0BC5'}, {'\u0BC9'}, {'\u0BCE', '\u0BD6'}, {'\u0BD8', '\u0BE6'}, {'\u0BF3', '\u0C00'}, {'\u0C04'}, {'\u0C0D'}, {'\u0C11'}, {'\u0C29'}, {'\u0C34'}, {'\u0C3A', '\u0C3D'}, {'\u0C45'}, {'\u0C49'}, {'\u0C4E', '\u0C54'}, {'\u0C57', '\u0C5F'}, {'\u0C62', '\u0C65'}, {'\u0C70', '\u0C81'}, {'\u0C84'}, {'\u0C8D'}, {'\u0C91'}, {'\u0CA9'}, {'\u0CB4'}, {'\u2002'}, {'\u0CBA', '\u0CBD'}, {'\u0CC5'}, {'\u0CC9'}, {'\u0CCE', '\u0CD4'}, {'\u0CD7', '\u0CDD'}, {'\u0CDF'}, {'\u0CE2', '\u0CE5'}, {'\u0CF0', '\u0D01'}, {'\u0D04'}, {'\u0D0D'}, {'\u0D11'}, {'\u0D29'}, {'\u0D3A', '\u0D3D'}, {'\u0D44', '\u0D45'}, {'\u0D49'}, {'\u0D4E', '\u0D56'}, {'\u0D58', '\u0D5F'}, {'\u0D62', '\u0D65'}, {'\u0D70', '\u0D81'}, {'\u0D84'}, {'\u0D97', '\u0D99'}, {'\u0DB2'}, {'\u0DBC'}, {'\u0DBE', '\u0DBF'}, {'\u0DC7', '\u0DC9'}, {'\u0DCB', '\u0DCE'}, {'\u0DD5'}, {'\u0DD7'}, {'\u0DE0', '\u0DF1'}, {'\u0DF5', '\u0E00'}, {'\u0E3B', '\u0E3E'}, {'\u0E5C', '\u0E80'}, {'\u0E83'}, {'\u0E85', '\u0E86'}, {'\u0E89'}, {'\u0E8B', '\u0E8C'}, {'\u0E8E', '\u0E93'}, {'\u0E98'}, {'\u0EA0'}, {'\u0EA4'}, {'\u0EA6'}, {'\u0EA8', '\u0EA9'}, {'\u0EAC'}, {'\u0EBA'}, {'\u0EBE', '\u0EBF'}, {'\u0EC5'}, {'\u0EC7'}, {'\u0ECE', '\u0ECF'}, {'\u2002'}, {'\u0EDA', '\u0EDB'}, {'\u0EDE', '\u0EFF'}, {'\u0F48'}, {'\u0F6B', '\u0F70'}, {'\u0F8C', '\u0F8F'}, {'\u0F98'}, {'\u0FBD'}, {'\u0FCD', '\u0FCE'}, {'\u0FD0', '\u0FFF'}, {'\u1022'}, {'\u1028'}, {'\u102B'}, {'\u1033', '\u1035'}, {'\u103A', '\u103F'}, {'\u105A', '\u109F'}, {'\u10C6', '\u10CF'}, {'\u10F9', '\u10FA'}, {'\u10FC', '\u10FF'}, {'\u115A', '\u115E'}, {'\u11A3', '\u11A7'}, {'\u11FA', '\u11FF'}, {'\u1207'}, {'\u1247'}, {'\u1249'}, {'\u124E', '\u124F'}, {'\u1257'}, {'\u1259'}, {'\u125E', '\u125F'}, {'\u1287'}, {'\u1289'}, {'\u128E', '\u128F'}, {'\u12AF'}, {'\u12B1'}, {'\u12B6', '\u12B7'}, {'\u12BF'}, {'\u12C1'}, {'\u12C6', '\u12C7'}, {'\u12CF'}, {'\u12D7'}, {'\u12EF'}, {'\u130F'}, {'\u1311'}, {'\u1316', '\u1317'}, {'\u131F'}, {'\u1347'}, {'\u135B', '\u1360'}, {'\u137D', '\u139F'}, {'\u13F5', '\u1400'}, {'\u2002'}, {'\u1677', '\u167F'}, {'\u169D', '\u169F'}, {'\u16F1', '\u16FF'}, {'\u170D'}, {'\u1715', '\u171F'}, {'\u1737', '\u173F'}, {'\u1754', '\u175F'}, {'\u176D'}, {'\u1771'}, {'\u1774', '\u177F'}, {'\u17DD', '\u17DF'}, {'\u17EA', '\u17FF'}, {'\u180F'}, {'\u181A', '\u181F'}, {'\u1878', '\u187F'}, {'\u18AA', '\u1DFF'}, {'\u1E9C', '\u1E9F'}, {'\u1EFA', '\u1EFF'}, {'\u1F16', '\u1F17'}, {'\u1F1E', '\u1F1F'}, {'\u1F46', '\u1F47'}, {'\u1F4E', '\u1F4F'}, {'\u1F58'}, {'\u1F5A'}, {'\u1F5C'}, {'\u1F5E'}, {'\u1F7E', '\u1F7F'}, {'\u1FB5'}, {'\u1FC5'}, {'\u1FD4', '\u1FD5'}, {'\u1FDC'}, {'\u1FF0', '\u1FF1'}, {'\u1FF5'}, {'\u1FFF'}, {'\u2053', '\u2056'}, {'\u2058', '\u205E'}, {'\u2064', '\u2069'}, {'\u2072', '\u2073'}, {'\u208F', '\u209F'}, {'\u20B2', '\u20CF'}, {'\u20EB', '\u20FF'}, {'\u213B', '\u213C'}, {'\u214C', '\u2152'}, {'\u2184', '\u218F'}, {'\u23CF', '\u23FF'}, {'\u2427', '\u243F'}, {'\u244B', '\u245F'}, {'\u24FF'}, {'\u2002'}, {'\u2614', '\u2615'}, {'\u2618'}, {'\u267E', '\u267F'}, {'\u268A', '\u2700'}, {'\u2705'}, {'\u270A', '\u270B'}, {'\u2728'}, {'\u274C'}, {'\u274E'}, {'\u2753', '\u2755'}, {'\u2757'}, {'\u275F', '\u2760'}, {'\u2795', '\u2797'}, {'\u27B0'}, {'\u27BF', '\u27CF'}, {'\u27EC', '\u27EF'}, {'\u2B00', '\u2E7F'}, {'\u2E9A'}, {'\u2EF4', '\u2EFF'}, {'\u2FD6', '\u2FEF'}, {'\u2FFC', '\u2FFF'}, {'\u3040'}, {'\u3097', '\u3098'}, {'\u3100', '\u3104'}, {'\u312D', '\u3130'}, {'\u318F'}, {'\u31B8', '\u31EF'}, {'\u321D', '\u321F'}, {'\u3244', '\u3250'}, {'\u327C', '\u327E'}, {'\u32CC', '\u32CF'}, {'\u32FF'}, {'\u3377', '\u337A'}, {'\u33DE', '\u33DF'}, {'\u33FF'}, {'\u4DB6', '\u4DFF'}, {'\u9FA6', '\u9FFF'}, {'\uA48D', '\uA48F'}, {'\uA4C7', '\uABFF'}, {'\uD7A4', '\uD7FF'}, {'\uFA2E', '\uFA2F'}, {'\uFA6B', '\uFAFF'}, {'\uFB07', '\uFB12'}, {'\uFB18', '\uFB1C'}, {'\uFB37'}, {'\uFB3D'}, {'\uFB3F'}, {'\uFB42'}, {'\u2002'}, {'\uFB45'}, {'\uFBB2', '\uFBD2'}, {'\uFD40', '\uFD4F'}, {'\uFD90', '\uFD91'}, {'\uFDC8', '\uFDCF'}, {'\uFDFD', '\uFDFF'}, {'\uFE10', '\uFE1F'}, {'\uFE24', '\uFE2F'}, {'\uFE47', '\uFE48'}, {'\uFE53'}, {'\uFE67'}, {'\uFE6C', '\uFE6F'}, {'\uFE75'}, {'\uFEFD', '\uFEFE'}, {'\uFF00'}, {'\uFFBF', '\uFFC1'}, {'\uFFC8', '\uFFC9'}, {'\uFFD0', '\uFFD1'}, {'\uFFD8', '\uFFD9'}, {'\uFFDD', '\uFFDF'}, {'\uFFE7'}, {'\uFFEF', '\uFFF8'}, {'\u2002'}});

    /**
     * Mapping of characters-to-nothing. This table is defined in RFC 3454,
     * Appendix B.1.
     *
     * @since   JSO-0.12.3
     */
    public static final MappingTable MAPPING_B_1 = MappingTable.create(new char[] {'\u00AD', '\u034F', '\u1806', '\u180B', '\u180C', '\u180D', '\u200B', '\u200C', '\u200D', '\u2060', '\uFE00', '\uFE01', '\uFE02', '\uFE03', '\uFE04', '\uFE05', '\uFE06', '\uFE07', '\uFE08', '\uFE09', '\uFE0A', '\uFE0B', '\uFE0C', '\uFE0D', '\uFE0E', '\uFE0F', '\uFEFF'}, new String[] {null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null});
    /**
     * Mapping for case-folding over NFKC. This table is defined in RFC 3454,
     * Appendix B.2.
     *
     * @since   JSO-0.12.3
     */
    public static final MappingTable MAPPING_B_2 = MappingTable.create(new char[] {'\u0041', '\u0042', '\u0043', '\u0044', '\u0045', '\u0046', '\u0047', '\u0048', '\u0049', '\u004A', '\u004B', '\u004C', '\u004D', '\u004E', '\u004F', '\u0050', '\u0051', '\u0052', '\u0053', '\u0054', '\u0055', '\u0056', '\u0057', '\u0058', '\u0059', '\u005A', '\u00B5', '\u00C0', '\u00C1', '\u00C2', '\u00C3', '\u00C4', '\u00C5', '\u00C6', '\u00C7', '\u00C8', '\u00C9', '\u00CA', '\u00CB', '\u00CC', '\u00CD', '\u00CE', '\u00CF', '\u00D0', '\u00D1', '\u00D2', '\u00D3', '\u00D4', '\u00D5', '\u00D6', '\u00D8', '\u00D9', '\u00DA', '\u00DB', '\u00DC', '\u00DD', '\u00DE', '\u00DF', '\u0100', '\u0102', '\u0104', '\u0106', '\u0108', '\u010A', '\u010C', '\u010E', '\u0110', '\u0112', '\u0114', '\u0116', '\u0118', '\u011A', '\u011C', '\u011E', '\u0120', '\u0122', '\u0124', '\u0126', '\u0128', '\u012A', '\u012C', '\u012E', '\u0130', '\u0132', '\u0134', '\u0136', '\u0139', '\u013B', '\u013D', '\u013F', '\u0141', '\u0143', '\u0145', '\u0147', '\u0149', '\u014A', '\u014C', '\u014E', '\u0150', '\u0152', '\u0154', '\u0156', '\u0158', '\u015A', '\u015C', '\u015E', '\u0160', '\u0162', '\u0164', '\u0166', '\u0168', '\u016A', '\u016C', '\u016E', '\u0170', '\u0172', '\u0174', '\u0176', '\u0178', '\u0179', '\u017B', '\u017D', '\u017F', '\u0181', '\u0182', '\u0184', '\u0186', '\u0187', '\u0189', '\u018A', '\u018B', '\u018E', '\u018F', '\u0190', '\u0191', '\u0193', '\u0194', '\u0196', '\u0197', '\u0198', '\u019C', '\u019D', '\u019F', '\u01A0', '\u01A2', '\u01A4', '\u01A6', '\u01A7', '\u01A9', '\u01AC', '\u01AE', '\u01AF', '\u01B1', '\u01B2', '\u01B3', '\u01B5', '\u01B7', '\u01B8', '\u01BC', '\u01C4', '\u01C5', '\u01C7', '\u01C8', '\u01CA', '\u01CB', '\u01CD', '\u01CF', '\u01D1', '\u01D3', '\u01D5', '\u01D7', '\u01D9', '\u01DB', '\u01DE', '\u01E0', '\u01E2', '\u01E4', '\u01E6', '\u01E8', '\u01EA', '\u01EC', '\u01EE', '\u01F0', '\u01F1', '\u01F2', '\u01F4', '\u01F6', '\u01F7', '\u01F8', '\u01FA', '\u01FC', '\u01FE', '\u0200', '\u0202', '\u0204', '\u0206', '\u0208', '\u020A', '\u020C', '\u020E', '\u0210', '\u0212', '\u0214', '\u0216', '\u0218', '\u021A', '\u021C', '\u021E', '\u0220', '\u0222', '\u0224', '\u0226', '\u0228', '\u022A', '\u022C', '\u022E', '\u0230', '\u0232', '\u0345', '\u037A', '\u0386', '\u0388', '\u0389', '\u038A', '\u038C', '\u038E', '\u038F', '\u0390', '\u0391', '\u0392', '\u0393', '\u0394', '\u0395', '\u0396', '\u0397', '\u0398', '\u0399', '\u039A', '\u039B', '\u039C', '\u039D', '\u039E', '\u039F', '\u03A0', '\u03A1', '\u03A3', '\u03A4', '\u03A5', '\u03A6', '\u03A7', '\u03A8', '\u03A9', '\u03AA', '\u03AB', '\u03B0', '\u03C2', '\u03D0', '\u03D1', '\u03D2', '\u03D3', '\u03D4', '\u03D5', '\u03D6', '\u03D8', '\u03DA', '\u03DC', '\u03DE', '\u03E0', '\u03E2', '\u03E4', '\u03E6', '\u03E8', '\u03EA', '\u03EC', '\u03EE', '\u03F0', '\u03F1', '\u03F2', '\u03F4', '\u03F5', '\u0400', '\u0401', '\u0402', '\u0403', '\u0404', '\u0405', '\u0406', '\u0407', '\u0408', '\u0409', '\u040A', '\u040B', '\u040C', '\u040D', '\u040E', '\u040F', '\u0410', '\u0411', '\u0412', '\u0413', '\u0414', '\u0415', '\u0416', '\u0417', '\u0418', '\u0419', '\u041A', '\u041B', '\u041C', '\u041D', '\u041E', '\u041F', '\u0420', '\u0421', '\u0422', '\u0423', '\u0424', '\u0425', '\u0426', '\u0427', '\u0428', '\u0429', '\u042A', '\u042B', '\u042C', '\u042D', '\u042E', '\u042F', '\u0460', '\u0462', '\u0464', '\u0466', '\u0468', '\u046A', '\u046C', '\u046E', '\u0470', '\u0472', '\u0474', '\u0476', '\u0478', '\u047A', '\u047C', '\u047E', '\u0480', '\u048A', '\u048C', '\u048E', '\u0490', '\u0492', '\u0494', '\u0496', '\u0498', '\u049A', '\u049C', '\u049E', '\u04A0', '\u04A2', '\u04A4', '\u04A6', '\u04A8', '\u04AA', '\u04AC', '\u04AE', '\u04B0', '\u04B2', '\u04B4', '\u04B6', '\u04B8', '\u04BA', '\u04BC', '\u04BE', '\u04C1', '\u04C3', '\u04C5', '\u04C7', '\u04C9', '\u04CB', '\u04CD', '\u04D0', '\u04D2', '\u04D4', '\u04D6', '\u04D8', '\u04DA', '\u04DC', '\u04DE', '\u04E0', '\u04E2', '\u04E4', '\u04E6', '\u04E8', '\u04EA', '\u04EC', '\u04EE', '\u04F0', '\u04F2', '\u04F4', '\u04F8', '\u0500', '\u0502', '\u0504', '\u0506', '\u0508', '\u050A', '\u050C', '\u050E', '\u0531', '\u0532', '\u0533', '\u0534', '\u0535', '\u0536', '\u0537', '\u0538', '\u0539', '\u053A', '\u053B', '\u053C', '\u053D', '\u053E', '\u053F', '\u0540', '\u0541', '\u0542', '\u0543', '\u0544', '\u0545', '\u0546', '\u0547', '\u0548', '\u0549', '\u054A', '\u054B', '\u054C', '\u054D', '\u054E', '\u054F', '\u0550', '\u0551', '\u0552', '\u0553', '\u0554', '\u0555', '\u0556', '\u0587', '\u1E00', '\u1E02', '\u1E04', '\u1E06', '\u1E08', '\u1E0A', '\u1E0C', '\u1E0E', '\u1E10', '\u1E12', '\u1E14', '\u1E16', '\u1E18', '\u1E1A', '\u1E1C', '\u1E1E', '\u1E20', '\u1E22', '\u1E24', '\u1E26', '\u1E28', '\u1E2A', '\u1E2C', '\u1E2E', '\u1E30', '\u1E32', '\u1E34', '\u1E36', '\u1E38', '\u1E3A', '\u1E3C', '\u1E3E', '\u1E40', '\u1E42', '\u1E44', '\u1E46', '\u1E48', '\u1E4A', '\u1E4C', '\u1E4E', '\u1E50', '\u1E52', '\u1E54', '\u1E56', '\u1E58', '\u1E5A', '\u1E5C', '\u1E5E', '\u1E60', '\u1E62', '\u1E64', '\u1E66', '\u1E68', '\u1E6A', '\u1E6C', '\u1E6E', '\u1E70', '\u1E72', '\u1E74', '\u1E76', '\u1E78', '\u1E7A', '\u1E7C', '\u1E7E', '\u1E80', '\u1E82', '\u1E84', '\u1E86', '\u1E88', '\u1E8A', '\u1E8C', '\u1E8E', '\u1E90', '\u1E92', '\u1E94', '\u1E96', '\u1E97', '\u1E98', '\u1E99', '\u1E9A', '\u1E9B', '\u1EA0', '\u1EA2', '\u1EA4', '\u1EA6', '\u1EA8', '\u1EAA', '\u1EAC', '\u1EAE', '\u1EB0', '\u1EB2', '\u1EB4', '\u1EB6', '\u1EB8', '\u1EBA', '\u1EBC', '\u1EBE', '\u1EC0', '\u1EC2', '\u1EC4', '\u1EC6', '\u1EC8', '\u1ECA', '\u1ECC', '\u1ECE', '\u1ED0', '\u1ED2', '\u1ED4', '\u1ED6', '\u1ED8', '\u1EDA', '\u1EDC', '\u1EDE', '\u1EE0', '\u1EE2', '\u1EE4', '\u1EE6', '\u1EE8', '\u1EEA', '\u1EEC', '\u1EEE', '\u1EF0', '\u1EF2', '\u1EF4', '\u1EF6', '\u1EF8', '\u1F08', '\u1F09', '\u1F0A', '\u1F0B', '\u1F0C', '\u1F0D', '\u1F0E', '\u1F0F', '\u1F18', '\u1F19', '\u1F1A', '\u1F1B', '\u1F1C', '\u1F1D', '\u1F28', '\u1F29', '\u1F2A', '\u1F2B', '\u1F2C', '\u1F2D', '\u1F2E', '\u1F2F', '\u1F38', '\u1F39', '\u1F3A', '\u1F3B', '\u1F3C', '\u1F3D', '\u1F3E', '\u1F3F', '\u1F48', '\u1F49', '\u1F4A', '\u1F4B', '\u1F4C', '\u1F4D', '\u1F50', '\u1F52', '\u1F54', '\u1F56', '\u1F59', '\u1F5B', '\u1F5D', '\u1F5F', '\u1F68', '\u1F69', '\u1F6A', '\u1F6B', '\u1F6C', '\u1F6D', '\u1F6E', '\u1F6F', '\u1F80', '\u1F81', '\u1F82', '\u1F83', '\u1F84', '\u1F85', '\u1F86', '\u1F87', '\u1F88', '\u1F89', '\u1F8A', '\u1F8B', '\u1F8C', '\u1F8D', '\u1F8E', '\u1F8F', '\u1F90', '\u1F91', '\u1F92', '\u1F93', '\u1F94', '\u1F95', '\u1F96', '\u1F97', '\u1F98', '\u1F99', '\u1F9A', '\u1F9B', '\u1F9C', '\u1F9D', '\u1F9E', '\u1F9F', '\u1FA0', '\u1FA1', '\u1FA2', '\u1FA3', '\u1FA4', '\u1FA5', '\u1FA6', '\u1FA7', '\u1FA8', '\u1FA9', '\u1FAA', '\u1FAB', '\u1FAC', '\u1FAD', '\u1FAE', '\u1FAF', '\u1FB2', '\u1FB3', '\u1FB4', '\u1FB6', '\u1FB7', '\u1FB8', '\u1FB9', '\u1FBA', '\u1FBB', '\u1FBC', '\u1FBE', '\u1FC2', '\u1FC3', '\u1FC4', '\u1FC6', '\u1FC7', '\u1FC8', '\u1FC9', '\u1FCA', '\u1FCB', '\u1FCC', '\u1FD2', '\u1FD3', '\u1FD6', '\u1FD7', '\u1FD8', '\u1FD9', '\u1FDA', '\u1FDB', '\u1FE2', '\u1FE3', '\u1FE4', '\u1FE6', '\u1FE7', '\u1FE8', '\u1FE9', '\u1FEA', '\u1FEB', '\u1FEC', '\u1FF2', '\u1FF3', '\u1FF4', '\u1FF6', '\u1FF7', '\u1FF8', '\u1FF9', '\u1FFA', '\u1FFB', '\u1FFC', '\u20A8', '\u2102', '\u2103', '\u2107', '\u2109', '\u210B', '\u210C', '\u210D', '\u2110', '\u2111', '\u2112', '\u2115', '\u2116', '\u2119', '\u211A', '\u211B', '\u211C', '\u211D', '\u2120', '\u2121', '\u2122', '\u2124', '\u2126', '\u2128', '\u212A', '\u212B', '\u212C', '\u212D', '\u2130', '\u2131', '\u2133', '\u213E', '\u213F', '\u2145', '\u2160', '\u2161', '\u2162', '\u2163', '\u2164', '\u2165', '\u2166', '\u2167', '\u2168', '\u2169', '\u216A', '\u216B', '\u216C', '\u216D', '\u216E', '\u216F', '\u24B6', '\u24B7', '\u24B8', '\u24B9', '\u24BA', '\u24BB', '\u24BC', '\u24BD', '\u24BE', '\u24BF', '\u24C0', '\u24C1', '\u24C2', '\u24C3', '\u24C4', '\u24C5', '\u24C6', '\u24C7', '\u24C8', '\u24C9', '\u24CA', '\u24CB', '\u24CC', '\u24CD', '\u24CE', '\u24CF', '\u3371', '\u3373', '\u3375', '\u3380', '\u3381', '\u3382', '\u3383', '\u3384', '\u3385', '\u3386', '\u3387', '\u338A', '\u338B', '\u338C', '\u3390', '\u3391', '\u3392', '\u3393', '\u3394', '\u33A9', '\u33AA', '\u33AB', '\u33AC', '\u33B4', '\u33B5', '\u33B6', '\u33B7', '\u33B8', '\u33B9', '\u33BA', '\u33BB', '\u33BC', '\u33BD', '\u33BE', '\u33BF', '\u33C0', '\u33C1', '\u33C3', '\u33C6', '\u33C7', '\u33C8', '\u33C9', '\u33CB', '\u33CD', '\u33CE', '\u33D7', '\u33D9', '\u33DA', '\u33DC', '\u33DD', '\uFB00', '\uFB01', '\uFB02', '\uFB03', '\uFB04', '\uFB05', '\uFB06', '\uFB13', '\uFB14', '\uFB15', '\uFB16', '\uFB17', '\uFF21', '\uFF22', '\uFF23', '\uFF24', '\uFF25', '\uFF26', '\uFF27', '\uFF28', '\uFF29', '\uFF2A', '\uFF2B', '\uFF2C', '\uFF2D', '\uFF2E', '\uFF2F', '\uFF30', '\uFF31', '\uFF32', '\uFF33', '\uFF34', '\uFF35', '\uFF36', '\uFF37', '\uFF38', '\uFF39', '\uFF3A'}, new String[] {"\u0061", "\u0062", "\u0063", "\u0064", "\u0065", "\u0066", "\u0067", "\u0068", "\u0069", "\u006A", "\u006B", "\u006C", "\u006D", "\u006E", "\u006F", "\u0070", "\u0071", "\u0072", "\u0073", "\u0074", "\u0075", "\u0076", "\u0077", "\u0078", "\u0079", "\u007A", "\u03BC", "\u00E0", "\u00E1", "\u00E2", "\u00E3", "\u00E4", "\u00E5", "\u00E6", "\u00E7", "\u00E8", "\u00E9", "\u00EA", "\u00EB", "\u00EC", "\u00ED", "\u00EE", "\u00EF", "\u00F0", "\u00F1", "\u00F2", "\u00F3", "\u00F4", "\u00F5", "\u00F6", "\u00F8", "\u00F9", "\u00FA", "\u00FB", "\u00FC", "\u00FD", "\u00FE", "\u0073\u0073", "\u0101", "\u0103", "\u0105", "\u0107", "\u0109", "\u010B", "\u010D", "\u010F", "\u0111", "\u0113", "\u0115", "\u0117", "\u0119", "\u011B", "\u011D", "\u011F", "\u0121", "\u0123", "\u0125", "\u0127", "\u0129", "\u012B", "\u012D", "\u012F", "\u0069\u0307", "\u0133", "\u0135", "\u0137", "\u013A", "\u013C", "\u013E", "\u0140", "\u0142", "\u0144", "\u0146", "\u0148", "\u02BC\u006E", "\u014B", "\u014D", "\u014F", "\u0151", "\u0153", "\u0155", "\u0157", "\u0159", "\u015B", "\u015D", "\u015F", "\u0161", "\u0163", "\u0165", "\u0167", "\u0169", "\u016B", "\u016D", "\u016F", "\u0171", "\u0173", "\u0175", "\u0177", "\u00FF", "\u017A", "\u017C", "\u017E", "\u0073", "\u0253", "\u0183", "\u0185", "\u0254", "\u0188", "\u0256", "\u0257", "\u018C", "\u01DD", "\u0259", "\u025B", "\u0192", "\u0260", "\u0263", "\u0269", "\u0268", "\u0199", "\u026F", "\u0272", "\u0275", "\u01A1", "\u01A3", "\u01A5", "\u0280", "\u01A8", "\u0283", "\u01AD", "\u0288", "\u01B0", "\u028A", "\u028B", "\u01B4", "\u01B6", "\u0292", "\u01B9", "\u01BD", "\u01C6", "\u01C6", "\u01C9", "\u01C9", "\u01CC", "\u01CC", "\u01CE", "\u01D0", "\u01D2", "\u01D4", "\u01D6", "\u01D8", "\u01DA", "\u01DC", "\u01DF", "\u01E1", "\u01E3", "\u01E5", "\u01E7", "\u01E9", "\u01EB", "\u01ED", "\u01EF", "\u006A\u030C", "\u01F3", "\u01F3", "\u01F5", "\u0195", "\u01BF", "\u01F9", "\u01FB", "\u01FD", "\u01FF", "\u0201", "\u0203", "\u0205", "\u0207", "\u0209", "\u020B", "\u020D", "\u020F", "\u0211", "\u0213", "\u0215", "\u0217", "\u0219", "\u021B", "\u021D", "\u021F", "\u019E", "\u0223", "\u0225", "\u0227", "\u0229", "\u022B", "\u022D", "\u022F", "\u0231", "\u0233", "\u03B9", "\u0020\u03B9", "\u03AC", "\u03AD", "\u03AE", "\u03AF", "\u03CC", "\u03CD", "\u03CE", "\u03B9\u0308\u0301", "\u03B1", "\u03B2", "\u03B3", "\u03B4", "\u03B5", "\u03B6", "\u03B7", "\u03B8", "\u03B9", "\u03BA", "\u03BB", "\u03BC", "\u03BD", "\u03BE", "\u03BF", "\u03C0", "\u03C1", "\u03C3", "\u03C4", "\u03C5", "\u03C6", "\u03C7", "\u03C8", "\u03C9", "\u03CA", "\u03CB", "\u03C5\u0308\u0301", "\u03C3", "\u03B2", "\u03B8", "\u03C5", "\u03CD", "\u03CB", "\u03C6", "\u03C0", "\u03D9", "\u03DB", "\u03DD", "\u03DF", "\u03E1", "\u03E3", "\u03E5", "\u03E7", "\u03E9", "\u03EB", "\u03ED", "\u03EF", "\u03BA", "\u03C1", "\u03C3", "\u03B8", "\u03B5", "\u0450", "\u0451", "\u0452", "\u0453", "\u0454", "\u0455", "\u0456", "\u0457", "\u0458", "\u0459", "\u045A", "\u045B", "\u045C", "\u045D", "\u045E", "\u045F", "\u0430", "\u0431", "\u0432", "\u0433", "\u0434", "\u0435", "\u0436", "\u0437", "\u0438", "\u0439", "\u043A", "\u043B", "\u043C", "\u043D", "\u043E", "\u043F", "\u0440", "\u0441", "\u0442", "\u0443", "\u0444", "\u0445", "\u0446", "\u0447", "\u0448", "\u0449", "\u044A", "\u044B", "\u044C", "\u044D", "\u044E", "\u044F", "\u0461", "\u0463", "\u0465", "\u0467", "\u0469", "\u046B", "\u046D", "\u046F", "\u0471", "\u0473", "\u0475", "\u0477", "\u0479", "\u047B", "\u047D", "\u047F", "\u0481", "\u048B", "\u048D", "\u048F", "\u0491", "\u0493", "\u0495", "\u0497", "\u0499", "\u049B", "\u049D", "\u049F", "\u04A1", "\u04A3", "\u04A5", "\u04A7", "\u04A9", "\u04AB", "\u04AD", "\u04AF", "\u04B1", "\u04B3", "\u04B5", "\u04B7", "\u04B9", "\u04BB", "\u04BD", "\u04BF", "\u04C2", "\u04C4", "\u04C6", "\u04C8", "\u04CA", "\u04CC", "\u04CE", "\u04D1", "\u04D3", "\u04D5", "\u04D7", "\u04D9", "\u04DB", "\u04DD", "\u04DF", "\u04E1", "\u04E3", "\u04E5", "\u04E7", "\u04E9", "\u04EB", "\u04ED", "\u04EF", "\u04F1", "\u04F3", "\u04F5", "\u04F9", "\u0501", "\u0503", "\u0505", "\u0507", "\u0509", "\u050B", "\u050D", "\u050F", "\u0561", "\u0562", "\u0563", "\u0564", "\u0565", "\u0566", "\u0567", "\u0568", "\u0569", "\u056A", "\u056B", "\u056C", "\u056D", "\u056E", "\u056F", "\u0570", "\u0571", "\u0572", "\u0573", "\u0574", "\u0575", "\u0576", "\u0577", "\u0578", "\u0579", "\u057A", "\u057B", "\u057C", "\u057D", "\u057E", "\u057F", "\u0580", "\u0581", "\u0582", "\u0583", "\u0584", "\u0585", "\u0586", "\u0565\u0582", "\u1E01", "\u1E03", "\u1E05", "\u1E07", "\u1E09", "\u1E0B", "\u1E0D", "\u1E0F", "\u1E11", "\u1E13", "\u1E15", "\u1E17", "\u1E19", "\u1E1B", "\u1E1D", "\u1E1F", "\u1E21", "\u1E23", "\u1E25", "\u1E27", "\u1E29", "\u1E2B", "\u1E2D", "\u1E2F", "\u1E31", "\u1E33", "\u1E35", "\u1E37", "\u1E39", "\u1E3B", "\u1E3D", "\u1E3F", "\u1E41", "\u1E43", "\u1E45", "\u1E47", "\u1E49", "\u1E4B", "\u1E4D", "\u1E4F", "\u1E51", "\u1E53", "\u1E55", "\u1E57", "\u1E59", "\u1E5B", "\u1E5D", "\u1E5F", "\u1E61", "\u1E63", "\u1E65", "\u1E67", "\u1E69", "\u1E6B", "\u1E6D", "\u1E6F", "\u1E71", "\u1E73", "\u1E75", "\u1E77", "\u1E79", "\u1E7B", "\u1E7D", "\u1E7F", "\u1E81", "\u1E83", "\u1E85", "\u1E87", "\u1E89", "\u1E8B", "\u1E8D", "\u1E8F", "\u1E91", "\u1E93", "\u1E95", "\u0068\u0331", "\u0074\u0308", "\u0077\u030A", "\u0079\u030A", "\u0061\u02BE", "\u1E61", "\u1EA1", "\u1EA3", "\u1EA5", "\u1EA7", "\u1EA9", "\u1EAB", "\u1EAD", "\u1EAF", "\u1EB1", "\u1EB3", "\u1EB5", "\u1EB7", "\u1EB9", "\u1EBB", "\u1EBD", "\u1EBF", "\u1EC1", "\u1EC3", "\u1EC5", "\u1EC7", "\u1EC9", "\u1ECB", "\u1ECD", "\u1ECF", "\u1ED1", "\u1ED3", "\u1ED5", "\u1ED7", "\u1ED9", "\u1EDB", "\u1EDD", "\u1EDF", "\u1EE1", "\u1EE3", "\u1EE5", "\u1EE7", "\u1EE9", "\u1EEB", "\u1EED", "\u1EEF", "\u1EF1", "\u1EF3", "\u1EF5", "\u1EF7", "\u1EF9", "\u1F00", "\u1F01", "\u1F02", "\u1F03", "\u1F04", "\u1F05", "\u1F06", "\u1F07", "\u1F10", "\u1F11", "\u1F12", "\u1F13", "\u1F14", "\u1F15", "\u1F20", "\u1F21", "\u1F22", "\u1F23", "\u1F24", "\u1F25", "\u1F26", "\u1F27", "\u1F30", "\u1F31", "\u1F32", "\u1F33", "\u1F34", "\u1F35", "\u1F36", "\u1F37", "\u1F40", "\u1F41", "\u1F42", "\u1F43", "\u1F44", "\u1F45", "\u03C5\u0313", "\u03C5\u0313\u0300", "\u03C5\u0313\u0301", "\u03C5\u0313\u0342", "\u1F51", "\u1F53", "\u1F55", "\u1F57", "\u1F60", "\u1F61", "\u1F62", "\u1F63", "\u1F64", "\u1F65", "\u1F66", "\u1F67", "\u1F00\u03B9", "\u1F01\u03B9", "\u1F02\u03B9", "\u1F03\u03B9", "\u1F04\u03B9", "\u1F05\u03B9", "\u1F06\u03B9", "\u1F07\u03B9", "\u1F00\u03B9", "\u1F01\u03B9", "\u1F02\u03B9", "\u1F03\u03B9", "\u1F04\u03B9", "\u1F05\u03B9", "\u1F06\u03B9", "\u1F07\u03B9", "\u1F20\u03B9", "\u1F21\u03B9", "\u1F22\u03B9", "\u1F23\u03B9", "\u1F24\u03B9", "\u1F25\u03B9", "\u1F26\u03B9", "\u1F27\u03B9", "\u1F20\u03B9", "\u1F21\u03B9", "\u1F22\u03B9", "\u1F23\u03B9", "\u1F24\u03B9", "\u1F25\u03B9", "\u1F26\u03B9", "\u1F27\u03B9", "\u1F60\u03B9", "\u1F61\u03B9", "\u1F62\u03B9", "\u1F63\u03B9", "\u1F64\u03B9", "\u1F65\u03B9", "\u1F66\u03B9", "\u1F67\u03B9", "\u1F60\u03B9", "\u1F61\u03B9", "\u1F62\u03B9", "\u1F63\u03B9", "\u1F64\u03B9", "\u1F65\u03B9", "\u1F66\u03B9", "\u1F67\u03B9", "\u1F70\u03B9", "\u03B1\u03B9", "\u03AC\u03B9", "\u03B1\u0342", "\u03B1\u0342\u03B9", "\u1FB0", "\u1FB1", "\u1F70", "\u1F71", "\u03B1\u03B9", "\u03B9", "\u1F74\u03B9", "\u03B7\u03B9", "\u03AE\u03B9", "\u03B7\u0342", "\u03B7\u0342\u03B9", "\u1F72", "\u1F73", "\u1F74", "\u1F75", "\u03B7\u03B9", "\u03B9\u0308\u0300", "\u03B9\u0308\u0301", "\u03B9\u0342", "\u03B9\u0308\u0342", "\u1FD0", "\u1FD1", "\u1F76", "\u1F77", "\u03C5\u0308\u0300", "\u03C5\u0308\u0301", "\u03C1\u0313", "\u03C5\u0342", "\u03C5\u0308\u0342", "\u1FE0", "\u1FE1", "\u1F7A", "\u1F7B", "\u1FE5", "\u1F7C\u03B9", "\u03C9\u03B9", "\u03CE\u03B9", "\u03C9\u0342", "\u03C9\u0342\u03B9", "\u1F78", "\u1F79", "\u1F7C", "\u1F7D", "\u03C9\u03B9", "\u0072\u0073", "\u0063", "\u00B0\u0063", "\u025B", "\u00B0\u0066", "\u0068", "\u0068", "\u0068", "\u0069", "\u0069", "\u006C", "\u006E", "\u006E\u006F", "\u0070", "\u0071", "\u0072", "\u0072", "\u0072", "\u0073\u006D", "\u0074\u0065\u006C", "\u0074\u006D", "\u007A", "\u03C9", "\u007A", "\u006B", "\u00E5", "\u0062", "\u0063", "\u0065", "\u0066", "\u006D", "\u03B3", "\u03C0", "\u0064", "\u2170", "\u2171", "\u2172", "\u2173", "\u2174", "\u2175", "\u2176", "\u2177", "\u2178", "\u2179", "\u217A", "\u217B", "\u217C", "\u217D", "\u217E", "\u217F", "\u24D0", "\u24D1", "\u24D2", "\u24D3", "\u24D4", "\u24D5", "\u24D6", "\u24D7", "\u24D8", "\u24D9", "\u24DA", "\u24DB", "\u24DC", "\u24DD", "\u24DE", "\u24DF", "\u24E0", "\u24E1", "\u24E2", "\u24E3", "\u24E4", "\u24E5", "\u24E6", "\u24E7", "\u24E8", "\u24E9", "\u0068\u0070\u0061", "\u0061\u0075", "\u006F\u0076", "\u0070\u0061", "\u006E\u0061", "\u03BC\u0061", "\u006D\u0061", "\u006B\u0061", "\u006B\u0062", "\u006D\u0062", "\u0067\u0062", "\u0070\u0066", "\u006E\u0066", "\u03BC\u0066", "\u0068\u007A", "\u006B\u0068\u007A", "\u006D\u0068\u007A", "\u0067\u0068\u007A", "\u0074\u0068\u007A", "\u0070\u0061", "\u006B\u0070\u0061", "\u006D\u0070\u0061", "\u0067\u0070\u0061", "\u0070\u0076", "\u006E\u0076", "\u03BC\u0076", "\u006D\u0076", "\u006B\u0076", "\u006D\u0076", "\u0070\u0077", "\u006E\u0077", "\u03BC\u0077", "\u006D\u0077", "\u006B\u0077", "\u006D\u0077", "\u006B\u03C9", "\u006D\u03C9", "\u0062\u0071", "\u0063\u2215\u006B\u0067", "\u0063\u006F\u002E", "\u0064\u0062", "\u0067\u0079", "\u0068\u0070", "\u006B\u006B", "\u006B\u006D", "\u0070\u0068", "\u0070\u0070\u006D", "\u0070\u0072", "\u0073\u0076", "\u0077\u0062", "\u0066\u0066", "\u0066\u0069", "\u0066\u006C", "\u0066\u0066\u0069", "\u0066\u0066\u006C", "\u0073\u0074", "\u0073\u0074", "\u0574\u0576", "\u0574\u0565", "\u0574\u056B", "\u057E\u0576", "\u0574\u056D", "\uFF41", "\uFF42", "\uFF43", "\uFF44", "\uFF45", "\uFF46", "\uFF47", "\uFF48", "\uFF49", "\uFF4A", "\uFF4B", "\uFF4C", "\uFF4D", "\uFF4E", "\uFF4F", "\uFF50", "\uFF51", "\uFF52", "\uFF53", "\uFF54", "\uFF55", "\uFF56", "\uFF57", "\uFF58", "\uFF59", "\uFF5A"});
    /**
     * Mapping for case-folding not over NFKC. This table is defined in RFC
     * 3454, Appendix B.3.
     *
     * @since   JSO-0.12.3
     */
    public static final MappingTable MAPPING_B_3 = MappingTable.create(new char[] {'\u0041', '\u0042', '\u0043', '\u0044', '\u0045', '\u0046', '\u0047', '\u0048', '\u0049', '\u004A', '\u004B', '\u004C', '\u004D', '\u004E', '\u004F', '\u0050', '\u0051', '\u0052', '\u0053', '\u0054', '\u0055', '\u0056', '\u0057', '\u0058', '\u0059', '\u005A', '\u00B5', '\u00C0', '\u00C1', '\u00C2', '\u00C3', '\u00C4', '\u00C5', '\u00C6', '\u00C7', '\u00C8', '\u00C9', '\u00CA', '\u00CB', '\u00CC', '\u00CD', '\u00CE', '\u00CF', '\u00D0', '\u00D1', '\u00D2', '\u00D3', '\u00D4', '\u00D5', '\u00D6', '\u00D8', '\u00D9', '\u00DA', '\u00DB', '\u00DC', '\u00DD', '\u00DE', '\u00DF', '\u0100', '\u0102', '\u0104', '\u0106', '\u0108', '\u010A', '\u010C', '\u010E', '\u0110', '\u0112', '\u0114', '\u0116', '\u0118', '\u011A', '\u011C', '\u011E', '\u0120', '\u0122', '\u0124', '\u0126', '\u0128', '\u012A', '\u012C', '\u012E', '\u0130', '\u0132', '\u0134', '\u0136', '\u0139', '\u013B', '\u013D', '\u013F', '\u0141', '\u0143', '\u0145', '\u0147', '\u0149', '\u014A', '\u014C', '\u014E', '\u0150', '\u0152', '\u0154', '\u0156', '\u0158', '\u015A', '\u015C', '\u015E', '\u0160', '\u0162', '\u0164', '\u0166', '\u0168', '\u016A', '\u016C', '\u016E', '\u0170', '\u0172', '\u0174', '\u0176', '\u0178', '\u0179', '\u017B', '\u017D', '\u017F', '\u0181', '\u0182', '\u0184', '\u0186', '\u0187', '\u0189', '\u018A', '\u018B', '\u018E', '\u018F', '\u0190', '\u0191', '\u0193', '\u0194', '\u0196', '\u0197', '\u0198', '\u019C', '\u019D', '\u019F', '\u01A0', '\u01A2', '\u01A4', '\u01A6', '\u01A7', '\u01A9', '\u01AC', '\u01AE', '\u01AF', '\u01B1', '\u01B2', '\u01B3', '\u01B5', '\u01B7', '\u01B8', '\u01BC', '\u01C4', '\u01C5', '\u01C7', '\u01C8', '\u01CA', '\u01CB', '\u01CD', '\u01CF', '\u01D1', '\u01D3', '\u01D5', '\u01D7', '\u01D9', '\u01DB', '\u01DE', '\u01E0', '\u01E2', '\u01E4', '\u01E6', '\u01E8', '\u01EA', '\u01EC', '\u01EE', '\u01F0', '\u01F1', '\u01F2', '\u01F4', '\u01F6', '\u01F7', '\u01F8', '\u01FA', '\u01FC', '\u01FE', '\u0200', '\u0202', '\u0204', '\u0206', '\u0208', '\u020A', '\u020C', '\u020E', '\u0210', '\u0212', '\u0214', '\u0216', '\u0218', '\u021A', '\u021C', '\u021E', '\u0220', '\u0222', '\u0224', '\u0226', '\u0228', '\u022A', '\u022C', '\u022E', '\u0230', '\u0232', '\u0345', '\u0386', '\u0388', '\u0389', '\u038A', '\u038C', '\u038E', '\u038F', '\u0390', '\u0391', '\u0392', '\u0393', '\u0394', '\u0395', '\u0396', '\u0397', '\u0398', '\u0399', '\u039A', '\u039B', '\u039C', '\u039D', '\u039E', '\u039F', '\u03A0', '\u03A1', '\u03A3', '\u03A4', '\u03A5', '\u03A6', '\u03A7', '\u03A8', '\u03A9', '\u03AA', '\u03AB', '\u03B0', '\u03C2', '\u03D0', '\u03D1', '\u03D5', '\u03D6', '\u03D8', '\u03DA', '\u03DC', '\u03DE', '\u03E0', '\u03E2', '\u03E4', '\u03E6', '\u03E8', '\u03EA', '\u03EC', '\u03EE', '\u03F0', '\u03F1', '\u03F2', '\u03F4', '\u03F5', '\u0400', '\u0401', '\u0402', '\u0403', '\u0404', '\u0405', '\u0406', '\u0407', '\u0408', '\u0409', '\u040A', '\u040B', '\u040C', '\u040D', '\u040E', '\u040F', '\u0410', '\u0411', '\u0412', '\u0413', '\u0414', '\u0415', '\u0416', '\u0417', '\u0418', '\u0419', '\u041A', '\u041B', '\u041C', '\u041D', '\u041E', '\u041F', '\u0420', '\u0421', '\u0422', '\u0423', '\u0424', '\u0425', '\u0426', '\u0427', '\u0428', '\u0429', '\u042A', '\u042B', '\u042C', '\u042D', '\u042E', '\u042F', '\u0460', '\u0462', '\u0464', '\u0466', '\u0468', '\u046A', '\u046C', '\u046E', '\u0470', '\u0472', '\u0474', '\u0476', '\u0478', '\u047A', '\u047C', '\u047E', '\u0480', '\u048A', '\u048C', '\u048E', '\u0490', '\u0492', '\u0494', '\u0496', '\u0498', '\u049A', '\u049C', '\u049E', '\u04A0', '\u04A2', '\u04A4', '\u04A6', '\u04A8', '\u04AA', '\u04AC', '\u04AE', '\u04B0', '\u04B2', '\u04B4', '\u04B6', '\u04B8', '\u04BA', '\u04BC', '\u04BE', '\u04C1', '\u04C3', '\u04C5', '\u04C7', '\u04C9', '\u04CB', '\u04CD', '\u04D0', '\u04D2', '\u04D4', '\u04D6', '\u04D8', '\u04DA', '\u04DC', '\u04DE', '\u04E0', '\u04E2', '\u04E4', '\u04E6', '\u04E8', '\u04EA', '\u04EC', '\u04EE', '\u04F0', '\u04F2', '\u04F4', '\u04F8', '\u0500', '\u0502', '\u0504', '\u0506', '\u0508', '\u050A', '\u050C', '\u050E', '\u0531', '\u0532', '\u0533', '\u0534', '\u0535', '\u0536', '\u0537', '\u0538', '\u0539', '\u053A', '\u053B', '\u053C', '\u053D', '\u053E', '\u053F', '\u0540', '\u0541', '\u0542', '\u0543', '\u0544', '\u0545', '\u0546', '\u0547', '\u0548', '\u0549', '\u054A', '\u054B', '\u054C', '\u054D', '\u054E', '\u054F', '\u0550', '\u0551', '\u0552', '\u0553', '\u0554', '\u0555', '\u0556', '\u0587', '\u1E00', '\u1E02', '\u1E04', '\u1E06', '\u1E08', '\u1E0A', '\u1E0C', '\u1E0E', '\u1E10', '\u1E12', '\u1E14', '\u1E16', '\u1E18', '\u1E1A', '\u1E1C', '\u1E1E', '\u1E20', '\u1E22', '\u1E24', '\u1E26', '\u1E28', '\u1E2A', '\u1E2C', '\u1E2E', '\u1E30', '\u1E32', '\u1E34', '\u1E36', '\u1E38', '\u1E3A', '\u1E3C', '\u1E3E', '\u1E40', '\u1E42', '\u1E44', '\u1E46', '\u1E48', '\u1E4A', '\u1E4C', '\u1E4E', '\u1E50', '\u1E52', '\u1E54', '\u1E56', '\u1E58', '\u1E5A', '\u1E5C', '\u1E5E', '\u1E60', '\u1E62', '\u1E64', '\u1E66', '\u1E68', '\u1E6A', '\u1E6C', '\u1E6E', '\u1E70', '\u1E72', '\u1E74', '\u1E76', '\u1E78', '\u1E7A', '\u1E7C', '\u1E7E', '\u1E80', '\u1E82', '\u1E84', '\u1E86', '\u1E88', '\u1E8A', '\u1E8C', '\u1E8E', '\u1E90', '\u1E92', '\u1E94', '\u1E96', '\u1E97', '\u1E98', '\u1E99', '\u1E9A', '\u1E9B', '\u1EA0', '\u1EA2', '\u1EA4', '\u1EA6', '\u1EA8', '\u1EAA', '\u1EAC', '\u1EAE', '\u1EB0', '\u1EB2', '\u1EB4', '\u1EB6', '\u1EB8', '\u1EBA', '\u1EBC', '\u1EBE', '\u1EC0', '\u1EC2', '\u1EC4', '\u1EC6', '\u1EC8', '\u1ECA', '\u1ECC', '\u1ECE', '\u1ED0', '\u1ED2', '\u1ED4', '\u1ED6', '\u1ED8', '\u1EDA', '\u1EDC', '\u1EDE', '\u1EE0', '\u1EE2', '\u1EE4', '\u1EE6', '\u1EE8', '\u1EEA', '\u1EEC', '\u1EEE', '\u1EF0', '\u1EF2', '\u1EF4', '\u1EF6', '\u1EF8', '\u1F08', '\u1F09', '\u1F0A', '\u1F0B', '\u1F0C', '\u1F0D', '\u1F0E', '\u1F0F', '\u1F18', '\u1F19', '\u1F1A', '\u1F1B', '\u1F1C', '\u1F1D', '\u1F28', '\u1F29', '\u1F2A', '\u1F2B', '\u1F2C', '\u1F2D', '\u1F2E', '\u1F2F', '\u1F38', '\u1F39', '\u1F3A', '\u1F3B', '\u1F3C', '\u1F3D', '\u1F3E', '\u1F3F', '\u1F48', '\u1F49', '\u1F4A', '\u1F4B', '\u1F4C', '\u1F4D', '\u1F50', '\u1F52', '\u1F54', '\u1F56', '\u1F59', '\u1F5B', '\u1F5D', '\u1F5F', '\u1F68', '\u1F69', '\u1F6A', '\u1F6B', '\u1F6C', '\u1F6D', '\u1F6E', '\u1F6F', '\u1F80', '\u1F81', '\u1F82', '\u1F83', '\u1F84', '\u1F85', '\u1F86', '\u1F87', '\u1F88', '\u1F89', '\u1F8A', '\u1F8B', '\u1F8C', '\u1F8D', '\u1F8E', '\u1F8F', '\u1F90', '\u1F91', '\u1F92', '\u1F93', '\u1F94', '\u1F95', '\u1F96', '\u1F97', '\u1F98', '\u1F99', '\u1F9A', '\u1F9B', '\u1F9C', '\u1F9D', '\u1F9E', '\u1F9F', '\u1FA0', '\u1FA1', '\u1FA2', '\u1FA3', '\u1FA4', '\u1FA5', '\u1FA6', '\u1FA7', '\u1FA8', '\u1FA9', '\u1FAA', '\u1FAB', '\u1FAC', '\u1FAD', '\u1FAE', '\u1FAF', '\u1FB2', '\u1FB3', '\u1FB4', '\u1FB6', '\u1FB7', '\u1FB8', '\u1FB9', '\u1FBA', '\u1FBB', '\u1FBC', '\u1FBE', '\u1FC2', '\u1FC3', '\u1FC4', '\u1FC6', '\u1FC7', '\u1FC8', '\u1FC9', '\u1FCA', '\u1FCB', '\u1FCC', '\u1FD2', '\u1FD3', '\u1FD6', '\u1FD7', '\u1FD8', '\u1FD9', '\u1FDA', '\u1FDB', '\u1FE2', '\u1FE3', '\u1FE4', '\u1FE6', '\u1FE7', '\u1FE8', '\u1FE9', '\u1FEA', '\u1FEB', '\u1FEC', '\u1FF2', '\u1FF3', '\u1FF4', '\u1FF6', '\u1FF7', '\u1FF8', '\u1FF9', '\u1FFA', '\u1FFB', '\u1FFC', '\u2126', '\u212A', '\u212B', '\u2160', '\u2161', '\u2162', '\u2163', '\u2164', '\u2165', '\u2166', '\u2167', '\u2168', '\u2169', '\u216A', '\u216B', '\u216C', '\u216D', '\u216E', '\u216F', '\u24B6', '\u24B7', '\u24B8', '\u24B9', '\u24BA', '\u24BB', '\u24BC', '\u24BD', '\u24BE', '\u24BF', '\u24C0', '\u24C1', '\u24C2', '\u24C3', '\u24C4', '\u24C5', '\u24C6', '\u24C7', '\u24C8', '\u24C9', '\u24CA', '\u24CB', '\u24CC', '\u24CD', '\u24CE', '\u24CF', '\uFB00', '\uFB01', '\uFB02', '\uFB03', '\uFB04', '\uFB05', '\uFB06', '\uFB13', '\uFB14', '\uFB15', '\uFB16', '\uFB17', '\uFF21', '\uFF22', '\uFF23', '\uFF24', '\uFF25', '\uFF26', '\uFF27', '\uFF28', '\uFF29', '\uFF2A', '\uFF2B', '\uFF2C', '\uFF2D', '\uFF2E', '\uFF2F', '\uFF30', '\uFF31', '\uFF32', '\uFF33', '\uFF34', '\uFF35', '\uFF36', '\uFF37', '\uFF38', '\uFF39', '\uFF3A'}, new String[] {"\u0061", "\u0062", "\u0063", "\u0064", "\u0065", "\u0066", "\u0067", "\u0068", "\u0069", "\u006A", "\u006B", "\u006C", "\u006D", "\u006E", "\u006F", "\u0070", "\u0071", "\u0072", "\u0073", "\u0074", "\u0075", "\u0076", "\u0077", "\u0078", "\u0079", "\u007A", "\u03BC", "\u00E0", "\u00E1", "\u00E2", "\u00E3", "\u00E4", "\u00E5", "\u00E6", "\u00E7", "\u00E8", "\u00E9", "\u00EA", "\u00EB", "\u00EC", "\u00ED", "\u00EE", "\u00EF", "\u00F0", "\u00F1", "\u00F2", "\u00F3", "\u00F4", "\u00F5", "\u00F6", "\u00F8", "\u00F9", "\u00FA", "\u00FB", "\u00FC", "\u00FD", "\u00FE", "\u0073\u0073", "\u0101", "\u0103", "\u0105", "\u0107", "\u0109", "\u010B", "\u010D", "\u010F", "\u0111", "\u0113", "\u0115", "\u0117", "\u0119", "\u011B", "\u011D", "\u011F", "\u0121", "\u0123", "\u0125", "\u0127", "\u0129", "\u012B", "\u012D", "\u012F", "\u0069\u0307", "\u0133", "\u0135", "\u0137", "\u013A", "\u013C", "\u013E", "\u0140", "\u0142", "\u0144", "\u0146", "\u0148", "\u02BC\u006E", "\u014B", "\u014D", "\u014F", "\u0151", "\u0153", "\u0155", "\u0157", "\u0159", "\u015B", "\u015D", "\u015F", "\u0161", "\u0163", "\u0165", "\u0167", "\u0169", "\u016B", "\u016D", "\u016F", "\u0171", "\u0173", "\u0175", "\u0177", "\u00FF", "\u017A", "\u017C", "\u017E", "\u0073", "\u0253", "\u0183", "\u0185", "\u0254", "\u0188", "\u0256", "\u0257", "\u018C", "\u01DD", "\u0259", "\u025B", "\u0192", "\u0260", "\u0263", "\u0269", "\u0268", "\u0199", "\u026F", "\u0272", "\u0275", "\u01A1", "\u01A3", "\u01A5", "\u0280", "\u01A8", "\u0283", "\u01AD", "\u0288", "\u01B0", "\u028A", "\u028B", "\u01B4", "\u01B6", "\u0292", "\u01B9", "\u01BD", "\u01C6", "\u01C6", "\u01C9", "\u01C9", "\u01CC", "\u01CC", "\u01CE", "\u01D0", "\u01D2", "\u01D4", "\u01D6", "\u01D8", "\u01DA", "\u01DC", "\u01DF", "\u01E1", "\u01E3", "\u01E5", "\u01E7", "\u01E9", "\u01EB", "\u01ED", "\u01EF", "\u006A\u030C", "\u01F3", "\u01F3", "\u01F5", "\u0195", "\u01BF", "\u01F9", "\u01FB", "\u01FD", "\u01FF", "\u0201", "\u0203", "\u0205", "\u0207", "\u0209", "\u020B", "\u020D", "\u020F", "\u0211", "\u0213", "\u0215", "\u0217", "\u0219", "\u021B", "\u021D", "\u021F", "\u019E", "\u0223", "\u0225", "\u0227", "\u0229", "\u022B", "\u022D", "\u022F", "\u0231", "\u0233", "\u03B9", "\u03AC", "\u03AD", "\u03AE", "\u03AF", "\u03CC", "\u03CD", "\u03CE", "\u03B9\u0308\u0301", "\u03B1", "\u03B2", "\u03B3", "\u03B4", "\u03B5", "\u03B6", "\u03B7", "\u03B8", "\u03B9", "\u03BA", "\u03BB", "\u03BC", "\u03BD", "\u03BE", "\u03BF", "\u03C0", "\u03C1", "\u03C3", "\u03C4", "\u03C5", "\u03C6", "\u03C7", "\u03C8", "\u03C9", "\u03CA", "\u03CB", "\u03C5\u0308\u0301", "\u03C3", "\u03B2", "\u03B8", "\u03C6", "\u03C0", "\u03D9", "\u03DB", "\u03DD", "\u03DF", "\u03E1", "\u03E3", "\u03E5", "\u03E7", "\u03E9", "\u03EB", "\u03ED", "\u03EF", "\u03BA", "\u03C1", "\u03C3", "\u03B8", "\u03B5", "\u0450", "\u0451", "\u0452", "\u0453", "\u0454", "\u0455", "\u0456", "\u0457", "\u0458", "\u0459", "\u045A", "\u045B", "\u045C", "\u045D", "\u045E", "\u045F", "\u0430", "\u0431", "\u0432", "\u0433", "\u0434", "\u0435", "\u0436", "\u0437", "\u0438", "\u0439", "\u043A", "\u043B", "\u043C", "\u043D", "\u043E", "\u043F", "\u0440", "\u0441", "\u0442", "\u0443", "\u0444", "\u0445", "\u0446", "\u0447", "\u0448", "\u0449", "\u044A", "\u044B", "\u044C", "\u044D", "\u044E", "\u044F", "\u0461", "\u0463", "\u0465", "\u0467", "\u0469", "\u046B", "\u046D", "\u046F", "\u0471", "\u0473", "\u0475", "\u0477", "\u0479", "\u047B", "\u047D", "\u047F", "\u0481", "\u048B", "\u048D", "\u048F", "\u0491", "\u0493", "\u0495", "\u0497", "\u0499", "\u049B", "\u049D", "\u049F", "\u04A1", "\u04A3", "\u04A5", "\u04A7", "\u04A9", "\u04AB", "\u04AD", "\u04AF", "\u04B1", "\u04B3", "\u04B5", "\u04B7", "\u04B9", "\u04BB", "\u04BD", "\u04BF", "\u04C2", "\u04C4", "\u04C6", "\u04C8", "\u04CA", "\u04CC", "\u04CE", "\u04D1", "\u04D3", "\u04D5", "\u04D7", "\u04D9", "\u04DB", "\u04DD", "\u04DF", "\u04E1", "\u04E3", "\u04E5", "\u04E7", "\u04E9", "\u04EB", "\u04ED", "\u04EF", "\u04F1", "\u04F3", "\u04F5", "\u04F9", "\u0501", "\u0503", "\u0505", "\u0507", "\u0509", "\u050B", "\u050D", "\u050F", "\u0561", "\u0562", "\u0563", "\u0564", "\u0565", "\u0566", "\u0567", "\u0568", "\u0569", "\u056A", "\u056B", "\u056C", "\u056D", "\u056E", "\u056F", "\u0570", "\u0571", "\u0572", "\u0573", "\u0574", "\u0575", "\u0576", "\u0577", "\u0578", "\u0579", "\u057A", "\u057B", "\u057C", "\u057D", "\u057E", "\u057F", "\u0580", "\u0581", "\u0582", "\u0583", "\u0584", "\u0585", "\u0586", "\u0565\u0582", "\u1E01", "\u1E03", "\u1E05", "\u1E07", "\u1E09", "\u1E0B", "\u1E0D", "\u1E0F", "\u1E11", "\u1E13", "\u1E15", "\u1E17", "\u1E19", "\u1E1B", "\u1E1D", "\u1E1F", "\u1E21", "\u1E23", "\u1E25", "\u1E27", "\u1E29", "\u1E2B", "\u1E2D", "\u1E2F", "\u1E31", "\u1E33", "\u1E35", "\u1E37", "\u1E39", "\u1E3B", "\u1E3D", "\u1E3F", "\u1E41", "\u1E43", "\u1E45", "\u1E47", "\u1E49", "\u1E4B", "\u1E4D", "\u1E4F", "\u1E51", "\u1E53", "\u1E55", "\u1E57", "\u1E59", "\u1E5B", "\u1E5D", "\u1E5F", "\u1E61", "\u1E63", "\u1E65", "\u1E67", "\u1E69", "\u1E6B", "\u1E6D", "\u1E6F", "\u1E71", "\u1E73", "\u1E75", "\u1E77", "\u1E79", "\u1E7B", "\u1E7D", "\u1E7F", "\u1E81", "\u1E83", "\u1E85", "\u1E87", "\u1E89", "\u1E8B", "\u1E8D", "\u1E8F", "\u1E91", "\u1E93", "\u1E95", "\u0068\u0331", "\u0074\u0308", "\u0077\u030A", "\u0079\u030A", "\u0061\u02BE", "\u1E61", "\u1EA1", "\u1EA3", "\u1EA5", "\u1EA7", "\u1EA9", "\u1EAB", "\u1EAD", "\u1EAF", "\u1EB1", "\u1EB3", "\u1EB5", "\u1EB7", "\u1EB9", "\u1EBB", "\u1EBD", "\u1EBF", "\u1EC1", "\u1EC3", "\u1EC5", "\u1EC7", "\u1EC9", "\u1ECB", "\u1ECD", "\u1ECF", "\u1ED1", "\u1ED3", "\u1ED5", "\u1ED7", "\u1ED9", "\u1EDB", "\u1EDD", "\u1EDF", "\u1EE1", "\u1EE3", "\u1EE5", "\u1EE7", "\u1EE9", "\u1EEB", "\u1EED", "\u1EEF", "\u1EF1", "\u1EF3", "\u1EF5", "\u1EF7", "\u1EF9", "\u1F00", "\u1F01", "\u1F02", "\u1F03", "\u1F04", "\u1F05", "\u1F06", "\u1F07", "\u1F10", "\u1F11", "\u1F12", "\u1F13", "\u1F14", "\u1F15", "\u1F20", "\u1F21", "\u1F22", "\u1F23", "\u1F24", "\u1F25", "\u1F26", "\u1F27", "\u1F30", "\u1F31", "\u1F32", "\u1F33", "\u1F34", "\u1F35", "\u1F36", "\u1F37", "\u1F40", "\u1F41", "\u1F42", "\u1F43", "\u1F44", "\u1F45", "\u03C5\u0313", "\u03C5\u0313\u0300", "\u03C5\u0313\u0301", "\u03C5\u0313\u0342", "\u1F51", "\u1F53", "\u1F55", "\u1F57", "\u1F60", "\u1F61", "\u1F62", "\u1F63", "\u1F64", "\u1F65", "\u1F66", "\u1F67", "\u1F00\u03B9", "\u1F01\u03B9", "\u1F02\u03B9", "\u1F03\u03B9", "\u1F04\u03B9", "\u1F05\u03B9", "\u1F06\u03B9", "\u1F07\u03B9", "\u1F00\u03B9", "\u1F01\u03B9", "\u1F02\u03B9", "\u1F03\u03B9", "\u1F04\u03B9", "\u1F05\u03B9", "\u1F06\u03B9", "\u1F07\u03B9", "\u1F20\u03B9", "\u1F21\u03B9", "\u1F22\u03B9", "\u1F23\u03B9", "\u1F24\u03B9", "\u1F25\u03B9", "\u1F26\u03B9", "\u1F27\u03B9", "\u1F20\u03B9", "\u1F21\u03B9", "\u1F22\u03B9", "\u1F23\u03B9", "\u1F24\u03B9", "\u1F25\u03B9", "\u1F26\u03B9", "\u1F27\u03B9", "\u1F60\u03B9", "\u1F61\u03B9", "\u1F62\u03B9", "\u1F63\u03B9", "\u1F64\u03B9", "\u1F65\u03B9", "\u1F66\u03B9", "\u1F67\u03B9", "\u1F60\u03B9", "\u1F61\u03B9", "\u1F62\u03B9", "\u1F63\u03B9", "\u1F64\u03B9", "\u1F65\u03B9", "\u1F66\u03B9", "\u1F67\u03B9", "\u1F70\u03B9", "\u03B1\u03B9", "\u03AC\u03B9", "\u03B1\u0342", "\u03B1\u0342\u03B9", "\u1FB0", "\u1FB1", "\u1F70", "\u1F71", "\u03B1\u03B9", "\u03B9", "\u1F74\u03B9", "\u03B7\u03B9", "\u03AE\u03B9", "\u03B7\u0342", "\u03B7\u0342\u03B9", "\u1F72", "\u1F73", "\u1F74", "\u1F75", "\u03B7\u03B9", "\u03B9\u0308\u0300", "\u03B9\u0308\u0301", "\u03B9\u0342", "\u03B9\u0308\u0342", "\u1FD0", "\u1FD1", "\u1F76", "\u1F77", "\u03C5\u0308\u0300", "\u03C5\u0308\u0301", "\u03C1\u0313", "\u03C5\u0342", "\u03C5\u0308\u0342", "\u1FE0", "\u1FE1", "\u1F7A", "\u1F7B", "\u1FE5", "\u1F7C\u03B9", "\u03C9\u03B9", "\u03CE\u03B9", "\u03C9\u0342", "\u03C9\u0342\u03B9", "\u1F78", "\u1F79", "\u1F7C", "\u1F7D", "\u03C9\u03B9", "\u03C9", "\u006B", "\u00E5", "\u2170", "\u2171", "\u2172", "\u2173", "\u2174", "\u2175", "\u2176", "\u2177", "\u2178", "\u2179", "\u217A", "\u217B", "\u217C", "\u217D", "\u217E", "\u217F", "\u24D0", "\u24D1", "\u24D2", "\u24D3", "\u24D4", "\u24D5", "\u24D6", "\u24D7", "\u24D8", "\u24D9", "\u24DA", "\u24DB", "\u24DC", "\u24DD", "\u24DE", "\u24DF", "\u24E0", "\u24E1", "\u24E2", "\u24E3", "\u24E4", "\u24E5", "\u24E6", "\u24E7", "\u24E8", "\u24E9", "\u0066\u0066", "\u0066\u0069", "\u0066\u006C", "\u0066\u0066\u0069", "\u0066\u0066\u006C", "\u0073\u0074", "\u0073\u0074", "\u0574\u0576", "\u0574\u0565", "\u0574\u056B", "\u057E\u0576", "\u0574\u056D", "\uFF41", "\uFF42", "\uFF43", "\uFF44", "\uFF45", "\uFF46", "\uFF47", "\uFF48", "\uFF49", "\uFF4A", "\uFF4B", "\uFF4C", "\uFF4D", "\uFF4E", "\uFF4F", "\uFF50", "\uFF51", "\uFF52", "\uFF53", "\uFF54", "\uFF55", "\uFF56", "\uFF57", "\uFF58", "\uFF59", "\uFF5A"});

    /**
     * Table of prohibited ASCII space characters. This table is defined in RFC
     * 3454, Appendix C.1.1.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_1_1 = ProhibitedTable.create(new char[][] {{'\u0020'}});
    /**
     * Table of prohibited non-ASCII space characters. This table is defined in
     * RFC 3454, Appendix C.1.2.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_1_2 = ProhibitedTable.create(new char[][] {{'\u00A0'}, {'\u1680'}, {'\u2000'}, {'\u2001'}, {'\u2002'}, {'\u2003'}, {'\u2004'}, {'\u2005'}, {'\u2006'}, {'\u2007'}, {'\u2008'}, {'\u2009'}, {'\u200A'}, {'\u200B'}, {'\u202F'}, {'\u205F'}, {'\u3000'}});
    /**
     * Table of prohibited ASCII control characters. This table is defined in
     * RFC 3454, Appendix C.2.1.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_2_1 = ProhibitedTable.create(new char[][] {{'\u0000', '\u001F'}, {'\u007F'}});
    /**
     * Table of prohibited non-ASCII control characters. This table is defined
     * in RFC 3454, Appendix C.2.2.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_2_2 = ProhibitedTable.create(new char[][] {{'\u0080', '\u009F'}, {'\u06DD'}, {'\u070F'}, {'\u180E'}, {'\u200C'}, {'\u200D'}, {'\u2028'}, {'\u2029'}, {'\u2060'}, {'\u2061'}, {'\u2062'}, {'\u2063'}, {'\u206A', '\u206F'}, {'\uFEFF'}, {'\uFFF9', '\uFFFC'}});
    /**
     * Table of prohibited private use characters. This table is defined in RFC
     * 3454, Appendix C.3.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_3 = ProhibitedTable.create(new char[][] {{'\uE000', '\uF8FF'}});
    /**
     * Table of prohibited non-character code points. This table is defined in
     * RFC 3454, Appendix C.4.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_4 = ProhibitedTable.create(new char[][] {{'\uFDD0', '\uFDEF'}, {'\uFFFE', '\uFFFF'}});
    /**
     * Table of prohibited surrogate codes. This table is defined in RFC 3454,
     * Appendix C.5.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_5 = ProhibitedTable.create(new char[][] {{'\uD800', '\uDFFF'}});
    /**
     * Table of prohibited characters inappropriate for plain text. This table
     * is defined in RFC 3454, Appendix C.6.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_6 = ProhibitedTable.create(new char[][] {{'\uFFF9'}, {'\uFFFA'}, {'\uFFFB'}, {'\uFFFC'}, {'\uFFFD'}});
    /**
     * Table of prohibited characters inappropriate for canonical
     * representation. This table is defined in RFC 3454, Appendix C.7.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_7 = ProhibitedTable.create(new char[][] {{'\u2FF0', '\u2FFB'}});
    /**
     * Table of prohibited characters that change display properties or are
     * deprecated. This table is defined in RFC 3454, Appendix C.8.
     *
     * @since   JSO-0.12.3
     */
    public static final ProhibitedTable PROHIBITED_C_8 = ProhibitedTable.create(new char[][] {{'\u0340'}, {'\u0341'}, {'\u200E'}, {'\u200F'}, {'\u202A'}, {'\u202B'}, {'\u202C'}, {'\u202D'}, {'\u202E'}, {'\u206A'}, {'\u206B'}, {'\u206C'}, {'\u206D'}, {'\u206E'}, {'\u206F'}});

    /**
     * Table of characters with bidirectional property "R" or "AL". This table
     * is defined in RFC 3454, Appendix D.1.
     *
     * @since   JSO-0.12.3
     */
    private static final char[][] BIDI_D_1 = new char[][] {{'\u05BE'}, {'\u05C0'}, {'\u05C3'}, {'\u05D0', '\u05EA'}, {'\u05F0', '\u05F4'}, {'\u061B'}, {'\u061F'}, {'\u0621', '\u063A'}, {'\u2002'}, {'\u0640', '\u064A'}, {'\u066D', '\u066F'}, {'\u0671', '\u06D5'}, {'\u06DD'}, {'\u06E5', '\u06E6'}, {'\u06FA', '\u06FE'}, {'\u0700', '\u070D'}, {'\u0710'}, {'\u0712', '\u072C'}, {'\u0780', '\u07A5'}, {'\u07B1'}, {'\u200F'}, {'\uFB1D'}, {'\uFB1F', '\uFB28'}, {'\uFB2A', '\uFB36'}, {'\uFB38', '\uFB3C'}, {'\uFB3E'}, {'\uFB40', '\uFB41'}, {'\uFB43', '\uFB44'}, {'\uFB46', '\uFBB1'}, {'\uFBD3', '\uFD3D'}, {'\uFD50', '\uFD8F'}, {'\uFD92', '\uFDC7'}, {'\uFDF0', '\uFDFC'}, {'\uFE70', '\uFE74'}, {'\uFE76', '\uFEFC'}};
    /**
     * Table of characters with bidirectional property "L". This table is
     * defined in RFC 3454, Appendix D.2.
     *
     * @since   JSO-0.12.3
     */
    private static final char[][] BIDI_D_2 = new char[][] {{'\u0041', '\u005A'}, {'\u0061', '\u007A'}, {'\u00AA'}, {'\u00B5'}, {'\u00BA'}, {'\u00C0', '\u00D6'}, {'\u00D8', '\u00F6'}, {'\u00F8', '\u0220'}, {'\u0222', '\u0233'}, {'\u0250', '\u02AD'}, {'\u02B0', '\u02B8'}, {'\u02BB', '\u02C1'}, {'\u02D0', '\u02D1'}, {'\u02E0', '\u02E4'}, {'\u02EE'}, {'\u037A'}, {'\u0386'}, {'\u2002'}, {'\u0388', '\u038A'}, {'\u038C'}, {'\u038E', '\u03A1'}, {'\u03A3', '\u03CE'}, {'\u03D0', '\u03F5'}, {'\u0400', '\u0482'}, {'\u048A', '\u04CE'}, {'\u04D0', '\u04F5'}, {'\u04F8', '\u04F9'}, {'\u0500', '\u050F'}, {'\u0531', '\u0556'}, {'\u0559', '\u055F'}, {'\u0561', '\u0587'}, {'\u0589'}, {'\u0903'}, {'\u0905', '\u0939'}, {'\u093D', '\u0940'}, {'\u0949', '\u094C'}, {'\u0950'}, {'\u0958', '\u0961'}, {'\u0964', '\u0970'}, {'\u0982', '\u0983'}, {'\u0985', '\u098C'}, {'\u098F', '\u0990'}, {'\u0993', '\u09A8'}, {'\u09AA', '\u09B0'}, {'\u09B2'}, {'\u09B6', '\u09B9'}, {'\u09BE', '\u09C0'}, {'\u09C7', '\u09C8'}, {'\u09CB', '\u09CC'}, {'\u09D7'}, {'\u09DC', '\u09DD'}, {'\u09DF', '\u09E1'}, {'\u09E6', '\u09F1'}, {'\u09F4', '\u09FA'}, {'\u0A05', '\u0A0A'}, {'\u0A0F', '\u0A10'}, {'\u0A13', '\u0A28'}, {'\u0A2A', '\u0A30'}, {'\u0A32', '\u0A33'}, {'\u0A35', '\u0A36'}, {'\u0A38', '\u0A39'}, {'\u0A3E', '\u0A40'}, {'\u0A59', '\u0A5C'}, {'\u0A5E'}, {'\u0A66', '\u0A6F'}, {'\u0A72', '\u0A74'}, {'\u2002'}, {'\u0A83'}, {'\u0A85', '\u0A8B'}, {'\u0A8D'}, {'\u0A8F', '\u0A91'}, {'\u0A93', '\u0AA8'}, {'\u0AAA', '\u0AB0'}, {'\u0AB2', '\u0AB3'}, {'\u0AB5', '\u0AB9'}, {'\u0ABD', '\u0AC0'}, {'\u0AC9'}, {'\u0ACB', '\u0ACC'}, {'\u0AD0'}, {'\u0AE0'}, {'\u0AE6', '\u0AEF'}, {'\u0B02', '\u0B03'}, {'\u0B05', '\u0B0C'}, {'\u0B0F', '\u0B10'}, {'\u0B13', '\u0B28'}, {'\u0B2A', '\u0B30'}, {'\u0B32', '\u0B33'}, {'\u0B36', '\u0B39'}, {'\u0B3D', '\u0B3E'}, {'\u0B40'}, {'\u0B47', '\u0B48'}, {'\u0B4B', '\u0B4C'}, {'\u0B57'}, {'\u0B5C', '\u0B5D'}, {'\u0B5F', '\u0B61'}, {'\u0B66', '\u0B70'}, {'\u0B83'}, {'\u0B85', '\u0B8A'}, {'\u0B8E', '\u0B90'}, {'\u0B92', '\u0B95'}, {'\u0B99', '\u0B9A'}, {'\u0B9C'}, {'\u0B9E', '\u0B9F'}, {'\u0BA3', '\u0BA4'}, {'\u0BA8', '\u0BAA'}, {'\u0BAE', '\u0BB5'}, {'\u0BB7', '\u0BB9'}, {'\u0BBE', '\u0BBF'}, {'\u0BC1', '\u0BC2'}, {'\u0BC6', '\u0BC8'}, {'\u0BCA', '\u0BCC'}, {'\u0BD7'}, {'\u0BE7', '\u0BF2'}, {'\u0C01', '\u0C03'}, {'\u0C05', '\u0C0C'}, {'\u2002'}, {'\u0C0E', '\u0C10'}, {'\u0C12', '\u0C28'}, {'\u0C2A', '\u0C33'}, {'\u0C35', '\u0C39'}, {'\u0C41', '\u0C44'}, {'\u0C60', '\u0C61'}, {'\u0C66', '\u0C6F'}, {'\u0C82', '\u0C83'}, {'\u0C85', '\u0C8C'}, {'\u0C8E', '\u0C90'}, {'\u0C92', '\u0CA8'}, {'\u0CAA', '\u0CB3'}, {'\u0CB5', '\u0CB9'}, {'\u0CBE'}, {'\u0CC0', '\u0CC4'}, {'\u0CC7', '\u0CC8'}, {'\u0CCA', '\u0CCB'}, {'\u0CD5', '\u0CD6'}, {'\u0CDE'}, {'\u0CE0', '\u0CE1'}, {'\u0CE6', '\u0CEF'}, {'\u0D02', '\u0D03'}, {'\u0D05', '\u0D0C'}, {'\u0D0E', '\u0D10'}, {'\u0D12', '\u0D28'}, {'\u0D2A', '\u0D39'}, {'\u0D3E', '\u0D40'}, {'\u0D46', '\u0D48'}, {'\u0D4A', '\u0D4C'}, {'\u0D57'}, {'\u0D60', '\u0D61'}, {'\u0D66', '\u0D6F'}, {'\u0D82', '\u0D83'}, {'\u0D85', '\u0D96'}, {'\u0D9A', '\u0DB1'}, {'\u0DB3', '\u0DBB'}, {'\u0DBD'}, {'\u0DC0', '\u0DC6'}, {'\u0DCF', '\u0DD1'}, {'\u0DD8', '\u0DDF'}, {'\u0DF2', '\u0DF4'}, {'\u0E01', '\u0E30'}, {'\u0E32', '\u0E33'}, {'\u0E40', '\u0E46'}, {'\u0E4F', '\u0E5B'}, {'\u0E81', '\u0E82'}, {'\u0E84'}, {'\u0E87', '\u0E88'}, {'\u2002'}, {'\u0E8A'}, {'\u0E8D'}, {'\u0E94', '\u0E97'}, {'\u0E99', '\u0E9F'}, {'\u0EA1', '\u0EA3'}, {'\u0EA5'}, {'\u0EA7'}, {'\u0EAA', '\u0EAB'}, {'\u0EAD', '\u0EB0'}, {'\u0EB2', '\u0EB3'}, {'\u0EBD'}, {'\u0EC0', '\u0EC4'}, {'\u0EC6'}, {'\u0ED0', '\u0ED9'}, {'\u0EDC', '\u0EDD'}, {'\u0F00', '\u0F17'}, {'\u0F1A', '\u0F34'}, {'\u0F36'}, {'\u0F38'}, {'\u0F3E', '\u0F47'}, {'\u0F49', '\u0F6A'}, {'\u0F7F'}, {'\u0F85'}, {'\u0F88', '\u0F8B'}, {'\u0FBE', '\u0FC5'}, {'\u0FC7', '\u0FCC'}, {'\u0FCF'}, {'\u1000', '\u1021'}, {'\u1023', '\u1027'}, {'\u1029', '\u102A'}, {'\u102C'}, {'\u1031'}, {'\u1038'}, {'\u1040', '\u1057'}, {'\u10A0', '\u10C5'}, {'\u10D0', '\u10F8'}, {'\u10FB'}, {'\u1100', '\u1159'}, {'\u115F', '\u11A2'}, {'\u11A8', '\u11F9'}, {'\u1200', '\u1206'}, {'\u1208', '\u1246'}, {'\u1248'}, {'\u124A', '\u124D'}, {'\u1250', '\u1256'}, {'\u1258'}, {'\u125A', '\u125D'}, {'\u1260', '\u1286'}, {'\u2002'}, {'\u1288'}, {'\u128A', '\u128D'}, {'\u1290', '\u12AE'}, {'\u12B0'}, {'\u12B2', '\u12B5'}, {'\u12B8', '\u12BE'}, {'\u12C0'}, {'\u12C2', '\u12C5'}, {'\u12C8', '\u12CE'}, {'\u12D0', '\u12D6'}, {'\u12D8', '\u12EE'}, {'\u12F0', '\u130E'}, {'\u1310'}, {'\u1312', '\u1315'}, {'\u1318', '\u131E'}, {'\u1320', '\u1346'}, {'\u1348', '\u135A'}, {'\u1361', '\u137C'}, {'\u13A0', '\u13F4'}, {'\u1401', '\u1676'}, {'\u1681', '\u169A'}, {'\u16A0', '\u16F0'}, {'\u1700', '\u170C'}, {'\u170E', '\u1711'}, {'\u1720', '\u1731'}, {'\u1735', '\u1736'}, {'\u1740', '\u1751'}, {'\u1760', '\u176C'}, {'\u176E', '\u1770'}, {'\u1780', '\u17B6'}, {'\u17BE', '\u17C5'}, {'\u17C7', '\u17C8'}, {'\u17D4', '\u17DA'}, {'\u17DC'}, {'\u17E0', '\u17E9'}, {'\u1810', '\u1819'}, {'\u1820', '\u1877'}, {'\u1880', '\u18A8'}, {'\u1E00', '\u1E9B'}, {'\u1EA0', '\u1EF9'}, {'\u1F00', '\u1F15'}, {'\u1F18', '\u1F1D'}, {'\u1F20', '\u1F45'}, {'\u1F48', '\u1F4D'}, {'\u1F50', '\u1F57'}, {'\u1F59'}, {'\u1F5B'}, {'\u1F5D'}, {'\u2002'}, {'\u1F5F', '\u1F7D'}, {'\u1F80', '\u1FB4'}, {'\u1FB6', '\u1FBC'}, {'\u1FBE'}, {'\u1FC2', '\u1FC4'}, {'\u1FC6', '\u1FCC'}, {'\u1FD0', '\u1FD3'}, {'\u1FD6', '\u1FDB'}, {'\u1FE0', '\u1FEC'}, {'\u1FF2', '\u1FF4'}, {'\u1FF6', '\u1FFC'}, {'\u200E'}, {'\u2071'}, {'\u207F'}, {'\u2102'}, {'\u2107'}, {'\u210A', '\u2113'}, {'\u2115'}, {'\u2119', '\u211D'}, {'\u2124'}, {'\u2126'}, {'\u2128'}, {'\u212A', '\u212D'}, {'\u212F', '\u2131'}, {'\u2133', '\u2139'}, {'\u213D', '\u213F'}, {'\u2145', '\u2149'}, {'\u2160', '\u2183'}, {'\u2336', '\u237A'}, {'\u2395'}, {'\u249C', '\u24E9'}, {'\u3005', '\u3007'}, {'\u3021', '\u3029'}, {'\u3031', '\u3035'}, {'\u3038', '\u303C'}, {'\u3041', '\u3096'}, {'\u309D', '\u309F'}, {'\u30A1', '\u30FA'}, {'\u30FC', '\u30FF'}, {'\u3105', '\u312C'}, {'\u3131', '\u318E'}, {'\u3190', '\u31B7'}, {'\u31F0', '\u321C'}, {'\u3220', '\u3243'}, {'\u3260', '\u327B'}, {'\u327F', '\u32B0'}, {'\u32C0', '\u32CB'}, {'\u32D0', '\u32FE'}, {'\u2002'}, {'\u3300', '\u3376'}, {'\u337B', '\u33DD'}, {'\u33E0', '\u33FE'}, {'\u3400', '\u4DB5'}, {'\u4E00', '\u9FA5'}, {'\uA000', '\uA48C'}, {'\uAC00', '\uD7A3'}, {'\uD800', '\uFA2D'}, {'\uFA30', '\uFA6A'}, {'\uFB00', '\uFB06'}, {'\uFB13', '\uFB17'}, {'\uFF21', '\uFF3A'}, {'\uFF41', '\uFF5A'}, {'\uFF66', '\uFFBE'}, {'\uFFC2', '\uFFC7'}, {'\uFFCA', '\uFFCF'}, {'\uFFD2', '\uFFD7'}, {'\uFFDA', '\uFFDC'}, {'\u2002'}};

    //Fields
    private final UnassignedTable       _Unassigned;
    private final boolean               _Normalize;
    private final MappingTable[]        _Mappings;
    private final ProhibitedTable[]     _Prohibited;

    //Constructors
    /**
     * <p>
     * Creates a new <tt>StringprepProfile</tt> with the given parameters.</p>
     *
     * <p>
     * If <tt>mappings</tt> is <tt>null</tt>, prepared strings are not
     * mapped.</p>
     *
     * <p>
     * If <tt>prohibited</tt> is <tt>null</tt>, prepared strings are not
     * verified against any prohibited characters.</p>
     *
     * @param   unassigned <tt>true</tt> if strings should be checked for
     * unassigned codepoints.
     * @param   normalize <tt>true</tt> if strings are normalized.
     * @param   mappings The array of <tt>MappingTable</tt>s to apply.
     * @param   prohibited The array of <tt>ProhibitedTable</tt>s to apply.
     * @since   JSO-0.12.3
     */
    protected StringprepProfile(boolean unassigned, boolean normalize, MappingTable[] mappings, ProhibitedTable[] prohibited) {
        _Unassigned = (!unassigned ? UNASSIGNED_A_1 : null);
        _Normalize = normalize;
        _Mappings = ((mappings != null) ? mappings : null);
        _Prohibited = ((prohibited != null) ? prohibited : null);
    }

    //Methods
    private final boolean isUnassignedAllowed() {
        return (_Unassigned != null);
    }
    private final boolean isNormalizing() {
        return _Normalize;
    }
    private final MappingTable[] getMappings() {
        return _Mappings;
    }
    private final ProhibitedTable[] getProhibited() {
        return _Prohibited;
    }

    /**
     * <p>
     * Prepares a string using this profile.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * If <tt>in</tt> is <tt>null</tt>, a <tt>StringprepException</tt> is
     * thrown.</p>
     *
     * @param   in The string to prepare.
     * @return  The prepared string.
     * @throws  StringprepException If there are any errors while preparing
     * <tt>in</tt>
     * @since   JSO-0.12.3
     */
    public String prepare(String in) throws StringprepException {
        StringBuffer   buffer;

        if (in == null)
            throw new StringprepException("input string cannot be null");

        buffer = new StringBuffer(in);
        buffer = unassigned(buffer);
        buffer = mapping(buffer);
        buffer = normalize(buffer);
        buffer = prohibited(buffer);
        buffer = bidirectionality(buffer);

        return buffer.toString();
    }
    /**
     * <p>
     * Determines if the given string could be successfully prepared.</p>
     *
     * <p>
     * If <tt>in</tt> is <tt>null</tt>, this method returns <tt>false</tt>.</p>
     *
     * @param   in The string to validate.
     * @return  <tt>true</tt> if <tt>in</tt> can be prepared.
     * @since   JSO-0.12.3
     */
    public boolean validate(String in) {
        boolean     result = true;
        
        try {
            prepare(in);
        } catch (Exception e) {
            result = false;
        }
        
        return result;
    }

    /**
     * <p>
     * Verifies input does not contain unassigned codepoints.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> should never be <tt>null</tt>.</p>
     *
     * @param   buffer The string buffer to verify.
     * @return  <tt>buffer</tt>, verified.
     * @throws  UnassignedException If there are any errors while verifying
     * <tt>buffer</tt>
     * @see     <a href='http://www.rfc/rfc3454.txt">RFC 3454, Section 7. Unassigned Code Points in Stringprep Profiles</a>
     * @since   JSO-0.12.3
     */
    protected StringBuffer unassigned(StringBuffer buffer) throws UnassignedException {
        UnassignedTable     unassigned = _Unassigned;

        if (unassigned != null)
            unassigned.verify(buffer);

        return buffer;
    }
    /**
     * <p>
     * Maps the input.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> should never be <tt>null</tt>.</p>
     *
     * @param   buffer The string buffer to map.
     * @return  <tt>buffer</tt>, mapped.
     * @see     <a href='http://www.rfc/rfc3454.txt">RFC 3454, Section 3. Mapping</a>
     * @since   JSO-0.12.3
     */
    protected StringBuffer mapping(StringBuffer buffer) {
        MappingTable[]      mapping = getMappings();
        int                 idx;

        if (mapping != null) {
            for (idx = 0; idx < mapping.length; idx++)
                buffer = mapping[idx].map(buffer);
        }

        return buffer;
    }
    /**
     * <p>
     * Normalizes the input using Form KC.</p>
     *
     * <p>
     * The value of <tt>in</tt> should never be <tt>null</tt>.</p>
     *
     * @param   in The string buffer to normalize.
     * @return  The normalized string buffer.
     * @see     <a href='http://www.rfc/rfc3454.txt">RFC 3454, Section 4. Normalization</a>
     * @since   JSO-0.12.3
     */
    protected StringBuffer normalize(StringBuffer in) {
        return (isNormalizing() ?
            new StringBuffer(NFKC.normalize(in.toString())) :
            in);
    }
    /**
     * <p>
     * Verifies input does not contain prohibited codepoints.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> should never be <tt>null</tt>.</p>
     *
     * @param   buffer The string buffer to verify.
     * @return  <tt>buffer</tt>, verified.
     * @throws  ProhibitedException If there are any errors while verifying
     * <tt>buffer</tt>
     * @see     <a href='http://www.rfc/rfc3454.txt">RFC 3454, Section 5. Prohibited Output</a>
     * @since   JSO-0.12.3
     */
    protected StringBuffer prohibited(StringBuffer buffer) throws ProhibitedException {
        ProhibitedTable[]   prohibited = getProhibited();
        int                 idx;

        if (prohibited != null) {
            for (idx = 0; idx < prohibited.length; idx++)
                prohibited[idx].verify(buffer);
        }

        return buffer;
    }
    /**
     * <p>
     * Verifies bidirectionality of the input.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> should never be <tt>null</tt>.</p>
     *
     * @param   buffer The string buffer to verify.
     * @return  <tt>buffer</tt>, verified.
     * @throws  BidirectionalException If there are any errors while verifying
     * <tt>buffer</tt>
     * @see     <a href='http://www.rfc/rfc3454.txt">RFC 3454, Section 6. Bidirectional Characters</a>
     * @since   JSO-0.12.3
     */
    protected StringBuffer bidirectionality(StringBuffer buffer) throws BidirectionalException {
        boolean     hasRAL, hasL;

        hasRAL = contains(buffer, BIDI_D_1);
        hasL = contains(buffer, BIDI_D_2);

        //Verify requirement 2
        if (hasRAL && hasL) {
            throw new StringprepException("Contains both R and AL code points");
        }

        //Verify requirement 3
        if (hasRAL) {
            if (!contains(buffer.charAt(0), BIDI_D_1) ||
                    !contains(buffer.charAt(buffer.length() - 1), BIDI_D_2)) {
                throw new StringprepException("Leading and trailing code points not both R or AL");
            }
        }

        return buffer;
    }

    private final boolean contains(char actual, char[] expected) {
        boolean     result = false;
        int         idx;

        for (idx = 0; !result && (idx < expected.length); idx++)
            result = (actual == expected[idx]);

        return result;
    }
    private final boolean contains(char actual, char[][] expected) {
        boolean     result = false;
        int         idx;

        for (idx = 0; !result && (idx < expected.length); idx++)
            result = contains(actual, expected[idx]);

        return result;
    }

    private final boolean contains(CharSequence cs, char[] expected) {
        boolean     result = false;
        int         pos;

        for (pos = 0; !result && (pos < cs.length()); pos++) {
            result = contains(cs.charAt(pos), expected);
        }

        return result;
    }
    private final boolean contains(CharSequence cs, char[][] expected) {
        boolean     result = false;
        int         pos;

        for (pos = 0; !result && (pos < cs.length()); pos++) {
            result = contains(cs.charAt(pos), expected);
        }

        return result;
    }
}