/*
 * PrivacyItem.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.Enumerator;
import org.jabberstudio.jso.x.core.RosterItem;

/**
 * <p>
 * Interface representing a "jabber:iq:privacy" list item.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface PrivacyItem extends StreamElement {
    //Inner-classes
    /**
     * <p>
     * Class enumerating privacy list item actions.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static final class Action extends Enumerator {
        //Constructors
        private Action(String name) {
            super(name);
        }
    }

    /**
     * <p>
     * Class enumerating privacy list item types.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static final class Type extends Enumerator {
        //COnstructors
        private Type(String name) {
            super(name);
        }
    }
    
    //"Constants"
    /** The qualified name for <tt>PrivacyItem</tt> ({jabber:iq:privacy}item).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("item", PrivacyQuery.NAMESPACE);

    /** Action for marking an item as allowed.
     *
     * @since   JSO-0.12
     */
    public static final Action  ALLOW           = new Action("allow");
    /** Action for marking an item as denied.
     *
     * @since   JSO-0.12
     */
    public static final Action  DENY            = new Action("deny");
    
    /** Type for marking an item as applied to a group. The value of {@link
     * #getValue} is a roster group name.
     *
     * @since   JSO-0.12
     */
    public static final Type    GROUP           = new Type("group");
    /** Type for marking an item as applied to a JID. The value from {@link
     * #getValue} is a JID.
     *
     * @since   JSO-0.12
     */
    public static final Type    JID             = new Type("jid");
    /** Type for marking an item as applied to a subscription. The value from
     * {@link #getValue} is a {@link RosterItem.SubscriptionType}.
     *
     * @since   JSO-0.12
     */
    public static final Type    SUBSCRIPTION    = new Type("subscription");
    
    //Methods
    /**
     * <p>
     * Retrieves the action for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value returned by this method should not be <tt>null</tt>.</p>
     *
     * @return The action for this <tt>PrivacyItem</tt>.
     * @since   JSO-0.12
     */
    public Action getAction();
    
    /**
     * <p>
     * Retrieves the type for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The type for this <tt>PrivacyItem</tt>.
     * @since   JSO-0.12
     */
    public Type getType();
    /**
     * <p>
     * Sets the type for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param type The new type.
     * @since   JSO-0.12
     */
    public void setType(Type type);

    /**
     * <p>
     * Retrieves the value for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, although it 
     * may be "" if the value has not been set.</p>
     *
     * @return The value for this <tt>PrivacyItem</tt>.
     * @since   JSO-0.12
     */
    public String getValue();
    /**
     * <p>
     * Sets the value for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param val The new value.
     * @since   JSO-0.12
     */
    public void setValue(String val);

    /**
     * <p>
     * Retrieves the value for this <tt>PrivacyItem</tt> as a JID.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the value has
     * not been set.</p>
     *
     * @return The JID value of this <tt>PrivacyItem</tt>.
     * @throws IllegalStateException If {@link #getType} does not return {@link
     * #JID}.
     * @since   JSO-0.12
     */
    public JID getJIDValue() throws IllegalStateException;
    /**
     * <p>
     * Sets the value for this <tt>PrivacyItem</tt> as a JID. Calling this
     * method results in {@link #getType} returning {@link #JID}.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.
     *
     * @param val The new JID value.
     * @since   JSO-0.12
     */
    public void setJIDValue(JID val);

    /**
     * <p>
     * Retrieves the value for this <tt>PrivacyItem</tt> as a group name.</p>
     *
     * <p>
     * The value returned by this method is never "".</p>
     *
     * @return The group name value of this <tt>PrivacyItem</tt>.
     * @throws IllegalStateException If {@link #getType} does not return {@link
     * #GROUP}.
     * @since   JSO-0.12
     */
    public String getGroupValue() throws IllegalStateException;
    /**
     * <p>
     * Sets the value for this <tt>PrivacyItem</tt> as a group name. Calling
     * this method results in {@link #getType} returning {@link #GROUP}.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.
     *
     * @param val The new group name value.
     * @since   JSO-0.12
     */
    public void setGroupValue(String val);
    
    /**
     * <p>
     * Retrieves the value for this <tt>PrivacyItem</tt> as a subcription
     * type.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The subscription value of this <tt>PrivacyItem</tt>.
     * @throws IllegalStateException If {@link #getType} does not return {@link
     * #SUBSCRIPTION}.
     * @since   JSO-0.12
     */
    public RosterItem.SubscriptionType getSubscriptionValue() throws IllegalStateException;
    /**
     * <p>
     * Sets the value for this <tt>PrivacyItem</tt> as a subscription type.
     * Calling this method results in {@link #getType} returning {@link
     * #SUBSCRIPTION}.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.
     *
     * @param val The new subscription type value.
     * @since   JSO-0.12
     */
    public void setSubscriptionValue(RosterItem.SubscriptionType val);
    
    /**
     * <p>
     * Retrieves the order for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value returned by this method should never be less than 0.</p>
     *
     * @return The order for this <tt>PrivacyItem</tt>.
     * @since   JSO-0.12
     */
    public int getOrder();
    /**
     * <p>
     * Sets the order for this <tt>PrivacyItem</tt>.</p>
     *
     * <p>
     * The value of <tt>order</tt> cannot be less than 0, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param order The new order.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void setOrder(int order) throws IllegalArgumentException;

    /**
     * <p>
     * Determines if this <tt>PrivacyItem</tt> is applied to &lt;iq/&gt;
     * stanzas.</p>
     *
     * @return <tt>true</tt> if this <tt>PrivacyItem</tt> is applied to
     * &lt;iq/&gt; stanzas.
     * @since   JSO-0.12
     */
    public boolean isAppliedToIQ();
    /**
     * <p>
     * Sets the flag on this <tt>PrivacyItem</tt> for applying to &lt;iq/&gt;
     * stanzas.</p>
     *
     * @param block Flag for applying to &lt;iq/&gt; stanzas.
     * @since   JSO-0.12
     */
    public void setAppliedToIQ(boolean block);

    /**
     * <p>
     * Determines if this <tt>PrivacyItem</tt> is applied to &lt;message/&gt;
     * stanzas.</p>
     *
     * @return <tt>true</tt> if this <tt>PrivacyItem</tt> is applied to
     * &lt;message/&gt; stanzas.
     * @since   JSO-0.12
     */
    public boolean isAppliedToMessage();
    /**
     * <p>
     * Sets the flag on this <tt>PrivacyItem</tt> for applying to
     * &lt;message/&gt; stanzas.</p>
     *
     * @param block Flag for applying to &lt;message/&gt; stanzas.
     * @since   JSO-0.12
     */
    public void setAppliedToMessage(boolean block);

    /**
     * <p>
     * Determines if this <tt>PrivacyItem</tt> is applied to incoming
     * &lt;presence/&gt; stanzas.</p>
     *
     * @return <tt>true</tt> if this <tt>PrivacyItem</tt> is applied to
     * incoming &lt;presence/&gt; stanzas.
     * @since   JSO-0.12
     */
    public boolean isAppliedToPresenceIn();
    /**
     * <p>
     * Sets the flag on this <tt>PrivacyItem</tt> for applying to incoming
     * &lt;presence/&gt; stanzas.</p>
     *
     * @param block Flag for applying to incoming &lt;presence/&gt; stanzas.
     * @since   JSO-0.12
     */
    public void setAppliedToPresenceIn(boolean block);
    
    /**
     * <p>
     * Determines if this <tt>PrivacyItem</tt> is applied to outgoing
     * &lt;presence/&gt; stanzas.</p>
     *
     * @return <tt>true</tt> if this <tt>PrivacyItem</tt> is applied to
     * outgoing &lt;presence/&gt; stanzas.
     * @since   JSO-0.12
     */
    public boolean isAppliedToPresenceOut();
    /**
     * <p>
     * Sets the flag on this <tt>PrivacyItem</tt> for applying to incoming
     * &lt;presence/&gt; stanzas.</p>
     *
     * @param block Flag for applying to incoming &lt;presence/&gt; stanzas.
     * @since   JSO-0.12
     */
    public void setAppliedToPresenceOut(boolean block);
}