/*
 * DiscoItem.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.disco;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Interface for representing a Service Discovery item.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0030.html'>JEP-0030</a>
 */
public interface DiscoItem extends StreamElement {
    //Inner-classes
    /**
     * <p>
     * Class to enumerate the possible actions of an <tt>DiscoItem</tt>.</p>
     * 
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     * @see <a href='http://www.jabber.org/jeps/jep-0030.html'>JEP-0030</a>
     */
    public static final class Action extends Enumerator {
        //Constructors
        private Action(String name) {
            super(name);
        }
    }

    //"Constants"
    /**
     * The qualified name for this <tt>DiscoItem</tt>
     * ({http://jabber.org/protocol/disco#items}item).
     *
     * @since   JSO-0.12
     */
    public static final NSI NAME = new NSI("item", DiscoItemsQuery.NAMESPACE);

    /**
     * Represents the "update" action.
     *
     * @since   JSO-0.12
     */
    public static final Action  UPDATE = new Action("update");
    /**
     * Represents the "remove" action.
     *
     * @since   JSO-0.12
     */
    public static final Action  REMOVE = new Action("remove");

    //Methods
    /**
     * <p>
     * Retrieves the JID for this <tt>DiscoItem</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The JID for this <tt>DiscoItem</tt>.
     * @since   JSO-0.12
     */
    public JID getJID();
    /**
     * <p>
     * Retrieves the node for this <tt>DiscoItem</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The node for this <tt>DiscoItem</tt>.
     * @since   JSO-0.12
     */
    public String getNode();

    /**
     * <p>
     * Retrieves the name for this <tt>DiscoItem</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The name for this <tt>DiscoItem</tt>.
     * @since   JSO-0.12
     */
    public String getName();
    /**
     * <p>
     * Sets the name for this <tt>DiscoItem</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed..</p>
     *
     * @param name The new name.
     * @since   JSO-0.12
     */
    public void setName(String name);

    /**
     * <p>
     * Retrieves the action for this <tt>DiscoItem</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The action for this <tt>DiscoItem</tt>.
     * @since   JSO-0.12
     */
    public Action getAction();
    /**
     * <p>
     * Sets the action for this <tt>DiscoItem</tt>.</p>
     *
     * <p>
     * The value of <tt>act</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param act The new action.
     * @since   JSO-0.12
     */
    public void setAction(Action act);
    }
