/*
 * DiscoUtilities.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.disco;



import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Utilities class for constants and methods common to Service Discovery.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0030.html'>JEP-0030</a>
 */
public final class DiscoUtilities {
    //"Constants"
    /**
     * Namespace URI for Service Discovery (http://jabber.org/protocol/disco).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE       = "http://jabber.org/protocol/disco";
    
    /**
     * Namespace URI for Service Discovery information
     * (http://jabber.org/protocol/disco#info).
     *
     * @since   JSO-0.12
     */
    public static final String  INFO_NAMESPACE  = NAMESPACE + "#info";
    /**
     * The qualified name for Service Discovery Information
     * ({http://jabber.org/protocol/disco#info}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     INFO_NAME       = new NSI("query", INFO_NAMESPACE);
    
    /**
     * Namespace URI for Service Discovery items
     * (http://jabber.org/protocol/disco#items).
     *
     * @since   JSO-0.12
     */
    public static final String  ITEMS_NAMESPACE = NAMESPACE + "#items";
    /**
     * The qualified name for Service Discovery items
     * ({http://jabber.org/protocol/disco#items}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     ITEMS_NAME      = new NSI("query", ITEMS_NAMESPACE);

    /**
     * Qualified name for "node" attribute.
     *
     * @since   JSO-0.12
     */
    public static final NSI     ATTRNAME_NODE = new NSI("node", null);
    /**
     * Qualified name for "name" attribute.
     *
     * @since   JSO-0.12
     */
    public static final NSI     ATTRNAME_NAME = new NSI("name", null);
    
    //Constructors
    private DiscoUtilities() {
    }

    //Methods
    /**
     * <p>
     * Generates a string representation of the category and type for the given
     * <tt>DiscoIdentity</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "". The
     * returned string is in the format "&lt;category&gt;/&lt;type&gt;", where
     * &lt;category&gt; is the value of {@link DiscoIdentity#getCategory} and
     * &lt;type&gt; is the value of {@link DiscoIdentity#getType}.</p>
     *
     * <p>
     * The value of <tt>id</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param id The <tt>DiscoIdentity</tt> to generate for.
     * @return The string representing the category and type.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public static final String toIdentityString(DiscoIdentity id) throws IllegalArgumentException {
        if (id == null)
            throw new IllegalArgumentException("Identity cannot be null");

        return toIdentityString(id.getCategory(), id.getType());
    }
    /**
     * <p>
     * Generates a string representation of the given category and type.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "". The
     * returned string is in the format "&lt;category&gt;/&lt;type&gt;".</p>
     *
     * <p>
     * The value of <tt>cat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param cat The category to generate from.
     * @param type The type to generate from.
     * @return The string representing the category and type.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public static final String toIdentityString(String cat, String type) throws IllegalArgumentException {
        if (!Utilities.isValidString(cat))
            throw new IllegalArgumentException("category cannot be null or \"\"");
        if (!Utilities.isValidString(type))
            throw new IllegalArgumentException("type cannot be null or \"\"");
        
        return (cat + "/" + type);
    }
}