/*
 * XDataField.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.xdata;


import java.util.List;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Interface for representing a form field. This interface maps to the
 * &lt;field/&gt; element.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0004.html">JEP-0004</a>
 */
public interface XDataField extends StreamElement {
    //Inner-classes
    /**
     * <p>
     * Class for representing the type of <tt>XDataField</tt>.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static final class Type extends Enumerator {
        //Constructors
        private Type(String name) {
            super(name);
        }
    }

    /**
     * <p>
     * Interface for representing an option of a field. This interface maps to
     * the &lt;option/&gt; element.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     * @see <a href="http://www.jabber.org/jeps/jep-0004.html">JEP-0004</a>
     */
    public static interface Option extends StreamElement {
        //Methods
        /**
         * <p>
         * Determines if this <tt>Option</tt> has an explicit label..</p>
         *
         * @return <tt>true</tt> if an explicit label is set.
         * @since   JSO-0.12
         */
        public boolean hasLabel();
        /**
         * <p>
         * Retrieves the label for this <tt>Option</tt>.</p>
         *
         * <p>
         * The value returned by this method is never <tt>null</tt>. If an
         * explicit label has not been set, this method returns the value of
         * {@link #getValue}.</p>
         *
         * @return The label for this <tt>Option</tt>.
         * @since   JSO-0.12
         */
        public String getLabel();
        /**
         * <p>
         * Sets the label for this <tt>Option</tt>.</p>
         *
         * <p>
         * The value of <tt>label</tt> may be <tt>null</tt> or "", in which
         * case the previous label is removed.</p>
         *
         * @param label The new label.
         * @since   JSO-0.12
         */
        public void setLabel(String label);
        
        /**
         * <p>
         * Retrieves the value for this <tt>Option</tt>.</p>
         *
         * <p>
         * The value returned by this method is never <tt>null</tt>.</p>
         *
         * @return The label for this <tt>Option</tt>.
         * @since   JSO-0.12
         */
        public String getValue();
    }
    
    //"Constants"
    /**
     * The qualified name for <tt>XDataField</tt> ({jabber:x:data}field).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME            = new NSI("field", XDataForm.NAMESPACE);
    
    /** 
     * Represents the type "boolean". Valid values are "0" and "1", and only
     * one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    BOOLEAN         = new Type("boolean");
    /** 
     * Represents the type "fixed". Valid values are arbitrary strings, and
     * only one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    FIXED           = new Type("fixed");
    /** 
     * Represents the type "hidden". Valid values are arbitrary strings, and
     * only one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    HIDDEN          = new Type("hidden");
    /** 
     * Represents the type "jid-multi". Valid values are JIDs, and may be more
     * than one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    JID_MULTI       = new Type("jid-multi");
    /** 
     * Represents the type "jid-single". Valid values are JID, and
     * only one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    JID_SINGLE      = new Type("jid-single");
    /** 
     * Represents the type "list-multi". Valid values are arbitrary strings,
     * and may be more than one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    LIST_MULTI      = new Type("list-multi");
    /** 
     * Represents the type "list-single". Valid values are arbitrary strings,
     * and only one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    LIST_SINGLE     = new Type("list-single");
    /** 
     * Represents the type "text-multi". Valid values are arbitrary strings,
     * and may be more than one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    TEXT_MULTI      = new Type("text-multi");
    /** 
     * Represents the type "text-private". Valid values are arbitrary strings,
     * and only one value, which is usually displayed in a obfuscated format.
     *
     * @since   JSO-0.12
     */
    public static final Type    TEXT_PRIVATE    = new Type("text-private");
    /** 
     * Represents the type "text-single". Valid values are arbitrary strings,
     * and only one value.
     *
     * @since   JSO-0.12
     */
    public static final Type    TEXT_SINGLE     = new Type("text-single");
    
    //Methods
    /**
     * <p>
     * Retreives the "var" for this <tt>XDataField</tt>. The "var" uniquely
     * identifies a field within a <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value returned by this method should never be <tt>null</tt>.</p>
     *
     * @return The "var" for this <tt>XDataField</tt>.
     * @since   JSO-0.12
     */
    public String getVar();
    
    /**
     * <p>
     * Retrieves the type for this <tt>XDataField</tt>. The type provides some
     * guidance how to present and manipulate the value(s).</p>
     *
     * <p>
     * The value returned by this method should never be <tt>null</tt>.</p>
     *
     * @return The type for this <tt>XDataField</tt>.
     * @since   JSO-0.12
     */
    public Type getType();
    /**
     * <p>
     * Sets the type for this <tt>XDataField</tt>.</p>
     *
     * <p>
     * If the value of <tt>type</tt> is <tt>null</tt>, any previous value is
     * removed.</p>
     *
     * @param type The new type.
     * @since   JSO-0.12
     */
    public void setType(Type type);
    
    /**
     * <p>
     * Determines if this <tt>XDataField</tt> is required to have values.</p>
     *
     * @return <tt>true</tt> if this field is required to have values.</p>
     * @since   JSO-0.12
     */
    public boolean isRequired();
    /**
     * <p>
     * Sets the required flag for this <tt>XDataField</tt>.</p>
     *
     * @param required The new required flag.
     * @since   JSO-0.12
     */
    public void setRequired(boolean required);

    /**
     * <p>
     * Determines if this <tt>XDataField</tt> has an explicit label..</p>
     *
     * @return <tt>true</tt> if an explicit label is set.
     * @since   JSO-0.12
     */
    public boolean hasLabel();
    /**
     * <p>
     * Retrieves the label for this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If an explicit
     * label has not been set, this method returns the value of {@link
     * #getVar}.</p>
     *
     * @return The label for this <tt>XDataField</tt>.
     * @since   JSO-0.12
     */
    public String getLabel();
    /**
     * <p>
     * Sets the label for this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value of <tt>label</tt> may be <tt>null</tt> or "", in which case
     * the previous label is removed.</p>
     *
     * @param label The new label.
     * @since   JSO-0.12
     */
    public void setLabel(String label);

    /**
     * <p>
     * Retrieves the description for this <tt>XDataField</tt>. This provides
     * additional information regarding the field.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The description of this <tt>XDataField</tt>.
     * @since   JSO-0.12
     */
    public String getDescription();
    /**
     * <p>
     * Sets the description for this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value of <tt>desc</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param desc The new description.
     * @since   JSO-0.12
     */
    public void setDescription(String desc);
    
    /**
     * <p>
     * Retrieves the list of options for this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non-<tt>null</tt> instance of
     * <tt>XDataField.Option</tt>.</p>
     *
     * @return The list of contained <tt>XDataField.Option</tt> objects.
     * @since   JSO-0.12
     */
    public List listOptions();
    /**
     * <p>
     * Retrieves the option with the given value.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no such
     * option exists.</p>
     *
     * <p>
     * The value of <tt>value</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param value The value of the <tt>XDataField.Option</tt> to locate.
     * @return The <tt>XDataField.Option</tt> for the value.
     * @since   JSO-0.12
     */
    public Option getOption(String value);
    /**
     * <p>
     * Adds an option for the given value, and an unset label.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If a
     * <tt>XDataField.Option</tt> for the given value already exists, it is
     * returned.</p>
     *
     * <p>
     * The value of <tt>value</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param value The value of the <tt>XDataField.Option</tt> to add.
     * @return The new <tt>XDataField.Option</tt>.
     * @since   JSO-0.12
     */
    public Option addOption(String value);
    /**
     * <p>
     * Adds an option for the given value and label.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If a
     * <tt>XDataField.Option</tt> for the given value already exists, it is
     * returned without modifying its label.</p>
     *
     * <p>
     * The value of <tt>label</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * <p>
     * The value of <tt>value</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param label The label of the <tt>XDataField.Option</tt> to add.
     * @param value The value of the <tt>XDataField.Option</tt> to add.
     * @return The new <tt>XDataField.Option</tt>.
     * @since   JSO-0.12
     */
    public Option addOption(String label, String value);
    /**
     * <p>
     * Removes the option for the given value.</p>
     *
     * <p>
     * The value of <tt>value</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param value The value of the <tt>XDataField.Option</tt> to remove.
     * @since   JSO-0.12
     */
    public void removeOption(String value);
    /**
     * <p>
     * Removes all options from this <tt>XDataField</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearOptions();
    
    /**
     * <p>
     * Lists all of the values for this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a string.</p>
     *
     * @return The list of field values.
     * @since   JSO-0.12
     */
    public List listValues();
    /**
     * <p>
     * Retrieves the value for this <tt>XDataField</tt>. Multi-value fields,
     * this is the first value.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a value has
     * not been set.</p>
     *
     * @return The field value.
     * @since   JSO-0.12
     */
    public String getValue();
    /**
     * <p>
     * Sets the value for this <tt>XDataField</tt>. This method replaces all
     * existing values with this value.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case all
     * previous values are removed.</p>
     *
     * @param val The new field value.
     * @since   JSO-0.12
     */
    public void setValue(String val);
    /**
     * <p>
     * Adds a value to this <tt>XDataField</tt>. This method does not verify
     * that the field's type supports multiple values.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param val The value to add.
     * @since   JSO-0.12
     */
    public void addValue(String val);
    /**
     * <p>
     * Removes a value from this <tt>XDataField</tt>. This method does not
     * verify that the field's type supports multiple values.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param val The value to remove.
     * @since   JSO-0.12
     */
    public void removeValue(String val);
    /**
     * <p>
     * Removes all values from this <tt>XDataField</tt>.</p>
     *
     * @since   JSO-0.12
     * 
     */
    public void clearValues();

    /**
     * <p>
     * Retrieves the field value, as a Boolean.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a value has
     * not been set.</p>
     *
     * @return The field's boolean value
     * @since   JSO-0.12
     */
    public Boolean getBooleanValue();
    /**
     * <p>
     * Sets the field value, as a Boolean.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param val The new boolean value
     * @since   JSO-0.12
     */
    public void setBooleanValue(Boolean val);

    /**
     * <p>
     * Retrieves the field's value, as a JID.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a value has
     * not been set, or is not a valid <tt>JID</tt></p>
     *
     * @return The field's JID value
     * @since   JSO-0.12
     */
    public JID getJIDValue();
    /**
     * <p>
     * Sets the field value, as a JID.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param val The new JID value
     * @since   JSO-0.12
     */
    public void setJIDValue(JID val);
    /**
     * <p>
     * Retrieves the list of field values, as JIDs.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a JID.</p>
     *
     * @return The list of JID values
     * @since   JSO-0.12
     */
    public List listJIDValues();
    /**
     * <p>
     * Adds a new JID value to this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case no actions
     * are performed.</p>
     *
     * @param val The JID value to add
     * @since   JSO-0.12
     */
    public void addJIDValue(JID val);
    /**
     * <p>
     * Removes a JID value from this <tt>XDataField</tt>.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt>, in which case no actions
     * are performed.</p>
     *
     * @param val The JID value to remove
     * @since   JSO-0.12
     */
    public void removeJIDValue(JID val);
}