/*
 * XPathSupport.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import java.util.Map;

import org.jaxen.XPath;

import org.saxpath.SAXPathException;

/**
 * <p>
 * Interface for providing XPath support. This interface provides for a default
 * namepsace mapping (prefixes to URIs), and the ability to create XPath
 * objects.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface XPathSupport {
    //Methods
    /**
     * <p>
     * Retrieves the namespace mappings for this <tt>XPathSupport</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The returned
     * map uses the namespace prefix for the key, and the namespace URI for
     * the value. The returned map is modifiable, but such modifications do not
     * reflect back into the stored mappings.</p>
     *
     * @return The namespace mappings.
     * @since   JSO-0.12
     */
    public Map getNamespaceMappings();
    /**
     * <p>
     * Sets the namespace mappings for this <tt>XPathSupport</tt>. The mappings
     * currently stored are removed, and replaced with those from
     * <tt>nss</tt>.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty map. Each key and value is treated as a
     * string, by first calling <tt>toString()</tt>. Any <tt>null</tt> strings
     * are treated as "".</p>
     *
     * @param nss The new namespace mappings.
     * @since   JSO-0.12
     */
    public void setNamespaceMappings(Map nss);
    /**
     * <p>
     * Adds a mapping from the given prefix to the given URI, replacing any
     * existing mapping.</p>
     *
     * <p>
     * The value of <tt>prefix</tt> may be <tt>null</tt> or "", in which case
     * it is considered to be "".</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param prefix The namespace prefix to add or replace.
     * @param uri The namespace URI to add or replace.
     * @since   JS0-0.12
     */
    public void putNamespaceMapping(String prefix, String uri);
    /**
     * <p>
     * Removes the mapping for the given prefix.</p>
     *
     * <P>
     * The value of <tt>prefix</tt> may be <tt>null</tt> or "", in which case
     * it is considered to be <tt>null</tt> or "".</p>
     *
     * @param prefix The namespace prefix mapping to remove.
     * @since   JSO-0.12
     */
    public void removeNamespaceMapping(String prefix);
    
    /**
     * <p>
     * Creates an <code>XPath</code> object for the given expression.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The
     * returned <tt>XPath</tt> object is preconfigured with the stored mappings
     * from this <tt>XPathSupport</tt>.</p>
     *
     * <p>
     * The value of <code>expr</code> must be a valid XPath expression, or
     * a <code>SAXPathException</code> is thrown.</p>
     *
     * @param expr The XPath expression to create from.
     * @return The created <code>XPath</code> object.
     * @throws SAXPathException If there are errors generating the
     * <code>XPath</code> object.
     * @since   JSO-0.12
     */
    public XPath createXPath(String expr) throws SAXPathException;
    /**
     * <p>
     * Creates an <code>XPath</code> object for the given expression and
     * additional namespace mappings.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The
     * returned <tt>XPath</tt> object is preconfigured with a combination of
     * the provided mappings and the stored mappings from this
     * <tt>XPathSupport</tt>.</p>
     *
     * <p>
     * The value of <code>expr</code> must be a valid XPath expression, or
     * a <code>SAXPathException</code> is thrown.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty map. The mappings within <tt>nss</tt> may
     * override those defined within this <tt>XPathSupport</tt>.</p>
     *
     * @param expr The XPath expression to create from.
     * @param nss The additional namespace mappings to use.
     * @return The created <code>XPath</code> object.
     * @throws SAXPathException If there are errors generating the
     * <code>XPath</code> object.
     * @since   JSO-0.12
     */
    public XPath createXPath(String expr, Map nss) throws SAXPathException;
}